#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

# -----------------------COMENTARIO GLOBAL-----------------------
# ---------------------------------------------------------------
# PROPOSITO
# ---------
# Buscar palabras dentro de los indices generados por prontus_index.
# Puede buscar por una o mas palabras, descartar resultados que contengan
# palabra determinadas y buscar con comodines.
# Los resultados aparecen en orden cronologico o por relevancia.
# Para orden cronologico se requiere que los indices sean Prontus.
# ---------------------------------------------------------------
# LLAMADAS A ARCHIVOS EXTERNOS.
# ------------------------------
# El buscador lee los archivos indice de los directorios cpan/data/search/<dir>/aaaa/
# Donde <dir> es un directorio Prontus o el indice de directorios brutos (raw).
#
# Los archivos de indice son:
#
# words.idx      - Indice de palabras.
#   Formato: Palabra=ID. Ordenado por palabras.
#
# cars.idx       - Indice alfabetico de primeros caracteres de palabras.
#   Formato: Letra=Offset.
#
# files.idx      - Indice de articulos.
#   Formato:
#   IDarticulo=fechap|TS.extension|titular|meta1|meta2|meta3|resumen|seccion|tema|subtema
#   seccion, tema y subtema van en texto; no en IDs.
#   fechap, seccion, tema y subtema son nulos si el indice no es Prontus (directorio raw).
#
# wordsindex.idx - Indice de palabras y los articulos que las contienen.
#   Formato:
#   IDpalabra=|IDarticulo count|IDarticulo count|IDarticulo count|IDarticulo count|...
#
# ---------------------------------------------------------------
# INVOCACIONES ACEPTADAS.
# ------------------------
# Como cgi, usando metodo GET o POST. Los parametros son:
# search_prontus  - Nombre del directorio Prontus donde residen los indices.
# search_comodines - Si es 'yes', permite la busqueda usando las wildcards * y ?.
# search_idx      - Indice donde buscar. Puede ser 'ALL', en cuyo caso se busca en todos.
# search_tmp      - Nombre de la plantilla de busqueda.
#                   Las plantillas se toman de /<search_prontus>/plantillas/extra/search/pags
#                   Si no viene este parametro se asume search.html.
# search_resxpag  - Resultados por pagina. Si no viene este parametro se asume 20.
# search_maxpags  - Maximo de paginas de resultados a mostrar. Si no viene este parametro se asume 20.
# search_texto    - Texto a buscar. Este texto es validado y convertido en un conjunto de palabras.
# search_pag      - Pagina a mostrar. Si no viene, se muestra la primera pagina.
# search_orden    - Orden de los resultados (CRO o REL). Si no viene, se ordena cronologico.
# search_form     - Mostrar formulario (0 o 1). Si no viene, el formulario se muestra.
# search_modo     - Hacer AND u OR con las palabras ingresadas (por defecto hace OR).
# search_meta1    - Variable META1, definida en el archivo de configuracion.
# search_meta2    - Variable META2, definida en el archivo de configuracion.
# search_meta3    - Variable META3, definida en el archivo de configuracion.
# search_seccion  - Seccion a mostrar. Si no viene, se muestran todas.
# search_tema     - Tema a mostrar. Si no viene, se muestran todos.
# search_subtema  - Subtema a mostrar. Si no viene, se muestran todos.
# search_fechaini - Fecha de inicio. Si no viene, no se filtra por fecha.
# search_fechafin - Fecha de termino. Si no viene, no se filtra por fecha.
#
# Si el buscador se invoca con solo el parametro search_prontus, se muestra
# la plantilla de busqueda con el formulario visible.
#
# En plataforma Unix se valida que no existan mas de 5 procesos
# de busqueda corriendo en forma simultanea.
# ---------------------------------------------------------------
# PLANTILLAS HTML UTILIZADAS.
# ------------------------
# Se utiliza la plantilla indicada en la invocacion:
# search_tmp = search.html
#
# En base a esta plantilla se construye la pagina de respuesta, que es escrita
# en el directorio:
# /<directorio prontus>/site/cache/search/pags
# El browser se dirige para que lea esa pagina a traves de un header Location.
#
# Las marcas reconocidas son las siguientes:
#
#   <!--formulario--> Delimitador del formulario de búsqueda.
#                     Esta parte de la plantilla es omitida si
#                     el buscador se invoca con el parametro
#                     search_form=0. Es conveniente que siempre exista,
#                     ya que asi se pueden hacer pruebas desde el
#                     administrador.
#     %%scriptname%%  URL de la cgi que se esta invocando. Usado en la pagina de prueba.
#     %%search_form%% Variable que regula si se muestra o no el formulario.
#   <!--/formulario-->
#   <!--separador-->  Separador de la parte fija de la respuesta, usado para
#                     comenzar a entregar la pagina de resultados con rapidez,
#                     evitando asi el doble click.
#     %%msg%%         Mensaje de contexto. P. ej. "50 resultados en orden de importancia." o "No hay resultados.".
#     %%msgsn%%       Mismo mensaje, pero sin referencia a numeros (1.7).
#   <!--resloop--> Trozo de URL que se repite para cada respuesta.
#     %%num%%         Numero ordinal del articulo encontrado.
#     %%rel%%%        % de relevancia relativa.
#     %%lnk%%         URL del articulo.
#     %%tit%%         Titular del articulo.
#     %%fec%%         Fecha de publicacion del articulo.
#     %%res%%         Resumen (primeras palabras) del articulo.
#     %%meta1%%       Variable META1 definida en el cfg.
#     %%meta2%%       Variable META2 definida en el cfg.
#     %%meta3%%       Variable META3 definida en el cfg.
#     %%sec%%         Seccion del articulo.
#     %%tem%%         Tema del articulo.
#     %%sub%%         Subtema del articulo.
#     %%metadatai%%   Variable METADATAi definida en el cfg (i = 1..20).
#     %%if(<var>)%%   Contenido condicionado a la existencia de la variable <var>.
#     %%/if%%
#   <!--/resloop-->
#   <!--pags-->       Delimitador de los links a las distintas
#                     páginas de resultados.
#                     Si la cantidad de resultados es menor que search_resxpag,
#                     esta parte de la plantilla es omitida.
#     %%pags%%        Links hacia otras páginas para buscar.
#   <!--/pags-->
#
# Ademas en esta plantilla se definen los mensajes que entregara el buscador mediante
# comentarios html, de la forma:
#   <!-- STOPWORDS = to,the,an,in,of -->
#   <!-- MSG server_busy = The server is currently busy. Please try again later ... -->
#   <!-- MSG no_results = No pages containing your search terms were found. -->
#   <!-- MSG order_cron = by date. -->
#   <!-- MSG order_rel = by relevance. -->
#   <!-- MSG results = Results  -->
#   <!-- MSG to = to  -->
#   <!-- MSG of = of -->
# ---------------------------------------------------------------
# -------------------------------BEGIN SCRIPT--------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibs = $Bin;
    unshift(@INC, $pathLibs);

    $pathLibs =~ s/\/[^\/]+$/\/cgi\-cpn/;
    unshift(@INC,$pathLibs);
}

# ---------------------------------------------------------------
# DECLARACIONES GLOBALES.
# ------------------------

use strict;
use utf8;

use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);

use prontus_varglb;
prontus_varglb::init();

use lib_search;
use lib_maxrunning;
use lib_prontus;

&lib_prontus::setUtf8();

# Raiz del sitio web.
our $DOCUMENT_ROOT = $prontus_varglb::DIR_SERVER;
our %FORM = ();        # Contenido del formulario de invocacion.
our %CFG = ();         # Variables del archivo de configuracion.
our %RESULTADOS = ();  # Hash de resultados de la busqueda (archivos y cuenta dentro de ellos).
our @RESULTADOS = ();  # Arreglo de resultados (ano e indice de archivo).
our %NUMRESULT = ();   # Hash del numero de palabras encontradas en cada resultado.
our %FECHAP = ();      # Fechas de publicacion de los resultados encontrados.
our %FILEDATA = ();    # Datos de cada archivo encontrado (fechap|TS.extension|titular|meta1|meta2|meta3|resumen|seccion|tema|subtema).
our $RESULTADOSTOTALES = ''; # Total de resultados encontrados.
our %LETRAS = ();      # Posicion de las letras dentro del archivo words.idx.
our %TOSHOW = ();      # Resultados para desplegar.
our $TURBOMODE = '';   # Busqueda sin comodines.
our @YESWORDS = ();    # Palabras a buscar.
our @NOTWORDS = ();    # Palabras a omitir.
our @FRASES = ();      # Frases a buscar.
our @FOUNDWORDSIDX = (); # Indices de las palabras a encontradas en cada indice.
our $MAXWORDS = '';    # Maximo de palabras distintas encontradas en un mismo archivo cuando se busca con wildcards.
our $MAXPALABRAS = ''; # Maximo de palabras repetidas encontradas dentro de un archivo.
our @NOTWORDSIDX = (); # Indices de las palabras a omitir.
our %WORDSINDEXES = (); # Indices de las palabras buscadas. Usado en la busqueda por frases.
our @IDX = ();         # Indices donde buscar.
our @ANOS = ();        # Anos presentes en los indices.
our $FILTROACTIVO = ''; # Flag que indica si se deben aplicar filtros o no.
our $FORMfechaini = ''; # Fecha de inicio en formato ISO, para filtrar resultados.
our $FORMfechafin = ''; # Fecha de termino en formato ISO, para filtrar resultados.
our $PLANTILLA = '';   # Plantilla de la pagina de resultados.
our $EXT = '';
our $RESULTADO = '';
our $MSG = '';    # Fuerza mensaje en la pagina de resultados.
our $CFG_FILENAME = 'buscador_prontus.cfg'; # Nombre del archivo de configuracion.
our $WORDS_FILENAME = 'words.idx';   # Nombre del archivo de palabras.
our $CARS_FILENAME = 'cars.idx';     # Nombre del archivo de caracteres.
our $FILES_FILENAME = 'files.idx';   # Nombre del archivo indice de articulos.
our $FILESINDEXF_FILENAME = 'filesindexf.idx'; # Nombre del archivo de indice de archivos para la busqueda por frases.
our $WORDSINDEX_FILENAME = 'wordsindex.idx'; # Nombre del archivo de indice de palabras.
our @NOBUSCAR = ('de','el','la','los','las','del','en'); # Palabras comunes por las que no hay que buscar.
our %MSGS = ();        # Arreglo de mensajes.
our $VISTA = '';       # Vista a mostrar.
our $RESXPAG = 50; # Maximo de resultados por pagina por defecto (overrideable por la CFG).
our $MAXPAGS = 20;  # Maximo de paginas por defecto (overrideable por la CFG).
our $DEBUG = 0;

if ($DEBUG) {
    print "Content-type: text/html\n";
    print "\n<pre>inicio \n";
}

&getFormData(); # Lee formulario de invocacion y valida las variables.

our $PRONTUS = $FORM{'search_prontus'}; # Nombre del publicador Prontus donde se aloja el buscador.
if ($PRONTUS eq '') { # Muestra pagina en blanco.
    print "Content-type: text/html\n";
    print "\nProntus dir not found. \nDirectorio Prontus no especificado.";
    exit;
}
if ($DEBUG) { print "\n<pre>PRONTUS = $PRONTUS \n"; }


our $PRONTUS_DIR = "$DOCUMENT_ROOT/$PRONTUS";
our $SEARCH_DIR = "$PRONTUS_DIR/cpan/data/search"; # Directorio de trabajo.
our $CACHE_REL_DIR = "/$PRONTUS/site/cache/search/pags"; # Directorio de cache para el direccionamiento web.
our $CACHE_DIR = "$DOCUMENT_ROOT$CACHE_REL_DIR"; # Directorio de cache.
our $TMP_DIR = "$PRONTUS_DIR/plantillas/extra/search/pags$VISTA"; # Directorio de las plantillas.

# Lee archivo de configuracion y determina la cantidad de instancias que se aceptan.
if (!-f "$PRONTUS_DIR/cpan/$CFG_FILENAME") {
    # Si no hay archivo de cfg, aborta.
    print "Content-type: text/html\n";
    print "\nProntus Search cfg file not found. \nArchivo de configuraci&oacute;n Prontus no encontrado.";
    exit;
}
if ($DEBUG) { print "\n<pre>PRONTUS_DIR = $PRONTUS_DIR \n"; }

%CFG = &lib_search::get_config("$PRONTUS_DIR/cpan/$CFG_FILENAME");
if ( ($CFG{'SEARCH_MAXEXEC'} < 0) || ($CFG{'SEARCH_MAXEXEC'} !~ /^\d+$/) ) {
    $CFG{'SEARCH_MAXEXEC'} = 5;
}

# Si no se especifican el la CFG, usa valores por defecto.
if ($CFG{'RESPERPAG'} !~ /^\d+$/) { $CFG{'RESPERPAG'} = $RESXPAG; }
if ($CFG{'MAXPAGS'} !~ /^\d+$/) { $CFG{'MAXPAGS'} = $MAXPAGS; }

# Toma los valores por defecto
# Si los valores del form no se especifican o exceden el maximo, aplica el maximo.
if(($FORM{'search_resxpag'} eq '') || ($FORM{'search_resxpag'} > $CFG{'RESPERPAG'})) {
    $FORM{'search_resxpag'} = $CFG{'RESPERPAG'};
}
if(($FORM{'search_maxpags'} eq '') || ($FORM{'search_maxpags'} > $CFG{'MAXPAGS'})) {
    $FORM{'search_maxpags'} = $CFG{'MAXPAGS'};
}

# Si dsps de todo, sigue vacio, se toma el valor por defecto
if ($FORM{'search_resxpag'} eq '') { $FORM{'search_resxpag'} = 20; }
if ($FORM{'search_maxpags'} eq '') { $FORM{'search_maxpags'} = 20; }

# Cargar variables de configuración
&carga_variables_prontus();

# Parsea y muestra primera parte de la pagina de resultados.
$PLANTILLA = &lib_search::lee_archivo($TMP_DIR .'/'. $FORM{'search_tmp'});
# reemplaza prontus id
$PLANTILLA =~ s/%%_prontus_id%%/$PRONTUS/ig;

# if ($DEBUG) { print "\n<pre>PLANTILLA = $PLANTILLA \n"; };

&inicializaMensajes(); # Carga arreglo de stop words (@NOBUSCAR) y hash de mensajes (%MSGS).
$RESULTADO = &parsea_plantilla1();

# Deduce la extension de la plantilla.
if ($FORM{'search_tmp'} =~ /\.([\w]+)$/) {
    $EXT = $1;
}

our $IDX = $FORM{'search_idx'}; # Nombre del indice a utilizar.
if ( (uc $IDX) eq 'ALL' ) {
    @IDX = &get_idx($SEARCH_DIR);
}else{
    push @IDX, $IDX;
}
if ($DEBUG) { print "\n<pre>IDX = @IDX \n"; }

# Determina si hay menos invocaciones que el maximo permitido (UNIX).
if (&lib_maxrunning::myselfRunning() > $CFG{'SEARCH_MAXEXEC'}) {
    $MSG = $MSGS{'server_busy'}; # 'Servidor ocupado. Intenta m&aacute;s tarde ...';
    $RESULTADO .= &parsea_plantilla2();
    &muestraResultado();
}

# Si no hay variables de invocacion, muestra el formulario solo.
if ($FORM{'search_texto'} !~ /[0-9a-zA-Z]{2}/) {
    # print "debug: texto invalido<br>\n";
    $MSG = $MSGS{'no_results'}; # 'No se encontraron resultados.' ;
    $RESULTADO .= &parsea_plantilla2();
    &muestraResultado();
}

# Depura el texto ingresado para determinar las palabras a buscar (@SEARCHWORDS).
our ($yeswords,$notwords,$turbomode,$frases) = &depura_palabras();
@YESWORDS = @$yeswords;
@NOTWORDS = @$notwords;
$TURBOMODE = $turbomode;
@FRASES = @$frases;

$RESULTADOSTOTALES = 0;
foreach my $idx (@IDX) {
    $IDX = $idx;

    # Encuentra anos indexados dentro de cada indice. El indice raw corresponde al ano 0000.
    @ANOS = &get_anos("$SEARCH_DIR/$IDX");
    foreach my $ANO (sort {$b <=> $a} @ANOS) {

        # Busca los indices de las palabras a encontrar y las stop words.
        my($foundwordsidx,$notwordsidx,$wordsindexes) = &busca_palabras("$SEARCH_DIR/$IDX/$ANO");
        @FOUNDWORDSIDX = sort {$a <=> $b} @$foundwordsidx;
        @NOTWORDSIDX = @$notwordsidx;
        %WORDSINDEXES = %$wordsindexes;

        # Busca las palabras en el indice de palabras, encontrando los indices de archivo.
        &busca_archivos("$SEARCH_DIR/$IDX/$ANO");
    }
}

$RESULTADO .= &muestra_resultados();
&muestraResultado();

# ###################################################
# Funciones


# -------------------------------------------------------------------------#
# Busca, inicializa y elimina mensajes y stopwords dentro de la plantilla.
# Carga arreglo de stop words (@NOBUSCAR) y hash de mensajes (%MSGS).
#     <!-- STOPWORDS = al,de,nos,fu -->
#     <!-- MSG xxx = xxx -->
sub inicializaMensajes {
    my($stopwords);

    # Mensajes por defecto.
    $MSGS{'server_busy'} = 'Servidor ocupado. Intenta m&aacute;s tarde ...';
    $MSGS{'no_results'} = 'No se encontraron resultados.';
    $MSGS{'order_cron'} = 'en orden cronol&oacute;gico.';
    $MSGS{'order_rel'} = 'en orden de importancia.';
    $MSGS{'results'} = 'Resultados';
    $MSGS{'to'} = 'al';
    $MSGS{'of'} = 'de';

    while ($PLANTILLA =~ /<!--\s*MSG\s*(\w+)\s*=\s*(.+?)\s*-->/sg) {
        $MSGS{$1} = $2;
    }

    # Elimina mensajes de la plantilla.
    $PLANTILLA =~ s/<!--\s*MSG\s*(\w+)\s*=\s*(.+?)\s*-->//sg;

    # Stop words. Busca y elimina al mismo tiempo.
    if ($PLANTILLA =~ s/<!--\s*STOPWORDS\s*=\s*([\w\,\ ]+)\s*-->//s) {
        $stopwords = $1;
        $stopwords =~ s/\s+//g;
        @NOBUSCAR = split(/,/,$stopwords);
    }
}; # inicializaMensajes

# -------------------------------------------------------------------------#
# Parsea y retorna la primera parte de la pagina de resultados.
# Si en la plantilla no hay separador no hace nada.
sub parsea_plantilla1 {
    my $plantilla = $PLANTILLA;
    my $scriptname = $ENV{'SCRIPT_NAME'};
    my $aux = '';
    if ($plantilla =~ /<!--separador-->/si) {
        $plantilla = $`;
    }else{
        return; # Si no hay separador no hace nada.
    }

    # Elimina el formulario si es que el mode era 0.
    if ($FORM{'search_form'} eq 'no') {
        $plantilla =~ s/<!--formulario-->(.+?)<!--\/formulario-->//sig;
    }

    # En forma automatica, crea y parsea variables checked y las variables del formulario.
    foreach my $key (keys %FORM) {
        $aux = $FORM{$key};

        if ($aux =~ /\w+/) {
            $plantilla =~ s/%%chk_$key\_$aux%%/checked=\"checked\"/isg;
            $plantilla =~ s/%%sel_$key\_$aux%%/selected=\"selected\"/isg;
        }
        $aux =~ s/(['"\<\>])//g;
        $plantilla =~ s/%%$key%%/$aux/isg;
    }

    # Parsea nombre de la CGI.
    $plantilla =~ s/%%scriptname%%/$scriptname/sg;

    # Elimina tags sin parsear.
    $plantilla =~ s/%%[^%]+%%//sg;
    return $plantilla;
}; # parsea_plantilla1

# -------------------------------------------------------------------------#
# Parsea y retorna la segunda parte de la pagina de resultados.
# Si en la plantilla no hay separador parsea y muestra la pagina completa.
sub parsea_plantilla2 {
    my @resultados = @_;
    my $scriptname = $ENV{'SCRIPT_NAME'};
    my ($searchloop,$searchloop_tmp,$aux,$resultad);

    # $fechap|$ts.$extension|$titular|meta1|meta2|meta3|$descripcion|$seccion|$tema|$subtema
    my ($ext,$rel,$dir,$lnk,$fec,$msg,$msgsn,$pags,$num);
    my @metadata = ();
    my $prontus_id;
    my $search_resxpag = $FORM{'search_resxpag'};
    my $search_pag = $FORM{'search_pag'};
    my $min = ($search_pag - 1) * $search_resxpag + 1; # Numero del primer resultado mostrado.
    my $max = $search_pag * $search_resxpag;
    my $paginastotales = int( 1 + ($RESULTADOSTOTALES - 1) / $search_resxpag);
    my $yeswords;

    if ( $paginastotales > $FORM{'search_maxpags'}) { $paginastotales = $FORM{'search_maxpags'}; }
    if ($max > $RESULTADOSTOTALES) { $max = $RESULTADOSTOTALES; }

    my $moldelink = $scriptname . '?search_prontus=' . $FORM{'search_prontus'}. '&amp;search_idx=' . $FORM{'search_idx'}. '&amp;search_tmp=' . $FORM{'search_tmp'}. '&amp;search_form=' . $FORM{'search_form'}. '&amp;search_pag=%%pagina%%'. '&amp;search_resxpag=' . $FORM{'search_resxpag'}. '&amp;search_maxpags=' . $FORM{'search_maxpags'}. '&amp;search_orden=' . $FORM{'search_orden'}. '&amp;search_meta1=' . &lib_search::escapehtml($FORM{'search_meta1'}). '&amp;search_meta2=' . &lib_search::escapehtml($FORM{'search_meta2'}). '&amp;search_meta3=' . &lib_search::escapehtml($FORM{'search_meta3'}). '&amp;search_seccion=' . &lib_search::escapehtml($FORM{'search_seccion'}). '&amp;search_tema=' . &lib_search::escapehtml($FORM{'search_tema'}). '&amp;search_subtema=' . &lib_search::escapehtml($FORM{'search_subtema'}). '&amp;search_fechaini=' . &lib_search::escapehtml($FORM{'search_fechaini'}). '&amp;search_fechafin=' . &lib_search::escapehtml($FORM{'search_fechafin'}). '&amp;search_texto=' . &lib_search::escapehtml($FORM{'search_texto'}). '&amp;search_modo=' . $FORM{'search_modo'}. '&amp;search_comodines=' . $FORM{'search_comodines'}. '&amp;vista=' . &lib_search::escapehtml($FORM{'vista'});

    if ($TURBOMODE == 1) {
        $yeswords = $#YESWORDS + 1;
    }else{
        $yeswords = $MAXWORDS;
    }

    my $plantilla = $PLANTILLA;
    my $tpl_prevlink = '<a href="%%link%%">%%pag%%</a>';
    my $tpl_nextlink = ' ... <a href="%%link%%">%%pag%%</a>';
    my $tpl_pagelink = '<a href="%%link%%">%%pag%%</a> ';
    my $tpl_pagactual = '<span class="pag_actual">%%pag%%</span> ';

    # Leer configuración de la plantilla.
    while ($plantilla =~ /<!--\s*CONFIG\s*(\w+)\s*=\s*(.+?)\s*-->/sg) {
        my $name = uc $1;
        my $value = $2;

        $tpl_prevlink = $value if ($name eq 'HTML_PREVLINK');
        $tpl_nextlink = $value if ($name eq 'HTML_NEXTLINK');
        $tpl_pagelink = $value if ($name eq 'HTML_PAGELINK');
        $tpl_pagactual = $value if ($name eq 'HTML_CURRENTPAGE');
    }

    $plantilla =~ s/<!--\s*CONFIG\s*(\w+)\s*=\s*.*?\s*-->//sg;

    my ($ini, $fin, $nextlink, $prevlink);
    my $pagcorta_maxpags = $CFG{'SEARCH_PAGCORTA_MAXPAGS'};
    if($CFG{'SEARCH_TIPO_PAGINACION'} eq '1') {
        # Se procesan las paginas hacia abajo
        $ini = ($search_pag - $pagcorta_maxpags);
        if($ini le 1) {
            $ini = 1;
        } else {
            $ini = $ini + 1;
            my $link = $moldelink;
            $link =~ s/%%pagina%%/1/;

            # $prevlink =  '<a href="' . $link . '">1</a>  ... ';

            $prevlink = $tpl_prevlink;
            $prevlink =~ s/%%link%%/$link/sg;
            $prevlink =~ s/%%pag%%/1/sg;
        }

        # Se procesan las páginas hacia arriba
        $fin = ($search_pag + $pagcorta_maxpags);
        if($fin >= $paginastotales) {
            $fin = $paginastotales;
        } else {
            $fin = $fin - 1;
            my $link = $moldelink;
            $link =~ s/%%pagina%%/$paginastotales/;

            $nextlink = $tpl_nextlink;
            $nextlink =~ s/%%link%%/$link/sg;
            $nextlink =~ s/%%pag%%/$paginastotales/sg;
        }

    } else {
        $ini = 1;
        $fin = $paginastotales;
    }

    for(my $i = $ini; $i <= $fin; $i++) {
        if ($i != $search_pag) {
            my $link = $moldelink;
            $link =~ s/%%pagina%%/$i/;
            my $tmp_link = $tpl_pagelink;

            $tmp_link =~ s/%%link%%/$link/sg;
            $tmp_link =~ s/%%pag%%/$i/sg;

            $pags .= $tmp_link;
        }else{
            my $tmp_link = $tpl_pagactual;
            $tmp_link =~ s/%%pag%%/$i/sg;
            $pags .= $tmp_link;
        }
    }
    $pags = $prevlink . $pags . $nextlink;

    if ($plantilla =~ /<!--separador-->/si) {
        $plantilla = $';
    }

    # Elimina el formulario si es que el mode era 0.
    if ($FORM{'search_form'} eq 'no') {
        $plantilla =~ s/<!--formulario-->(.+?)<!--\/formulario-->//sig;
    }
    if ($plantilla =~ /<!--resloop-->(.+?)<!--\/resloop-->/si) {
        $searchloop_tmp = $1;
    }

    # print "searchloop_tmp = [$searchloop_tmp]"; # debug
    $num = $min;

    # Ajusta valores para no pasar el 100%).
    $yeswords++;
    $MAXPALABRAS++;
    foreach my $resultado (@resultados) {
        $aux = $searchloop_tmp;
        my ($numresult,$numrepet,$fechap,$file,$tit,$meta1,$meta2,$meta3,$res,$sec,$tem,$sub,$custom_slug,$slug,$url,@metadata) = split(/\|/,$resultado);
        my ($ts, $fechac) = ('', '');

        # reemplazamos los signos % literales en el texto por una string random para no perderlos al limpiar la plantilla.
        $tit =~ s/%/[reemplazo_caracter_porciento]/g;
        $res =~ s/%/[reemplazo_caracter_porciento]/g;
        $meta1 =~ s/%/[reemplazo_caracter_porciento]/g;
        $meta2 =~ s/%/[reemplazo_caracter_porciento]/g;
        $meta3 =~ s/%/[reemplazo_caracter_porciento]/g;
        $slug =~ s/%/[reemplazo_caracter_porciento]/g;
        $custom_slug =~ s/%/[reemplazo_caracter_porciento]/g;

        # Rescata la extension.
        if ($file =~ /\.([^\.]+)$/) {
            $ext = lc $1;
        }
        $rel = int(100 * ($numresult/$yeswords + $numrepet/($yeswords * $MAXPALABRAS))); # Calcula el ranking de este resultado
        $fec = $fechap;
        $lnk = $file;
        if ($lnk =~ /\/([\w\-]+)\/site\/artic\/(\d{8})\/pags\/(\d{14})\.\w+$/) {
            $prontus_id = $1;
            $fechac = $2;
            $ts =  $3;

            if ($CFG{'USEFRIENDLYURLS'} == 1) {
                my %artic_data = ();
                # no es necesario definir el fid, ya que la variable que lo excluye no esta definida
                # asume que todos los articulos indexados son permitidos de tener f4
                $artic_data{'fid'} = 'fid';
                $artic_data{'custom_slug'} = $custom_slug;
                $artic_data{'slug'} = $slug;
                $artic_data{'_url'} = $url;
                $artic_data{'nom_seccion'} = $sec;
                $artic_data{'nom_tema'} = $tem;
                $artic_data{'nom_subtema'} = $sub;
                $aux = &lib_prontus::parse_filef($aux, $tit, $ts, $prontus_id, $file, \%artic_data);
                $lnk = &lib_prontus::parse_filef('%%_FILEURL%%', $tit, $ts, $prontus_id, $file, \%artic_data);
            }
        }

        # Elimina contenido condicional.
        if ($fec eq '') {
            $aux =~ s/%%if\(fec\)%%.+?%%\/if%%//isg;
        }
        if ($sec eq '') {
            $aux =~ s/%%if\(sec\)%%.+?%%\/if%%//isg;
        }
        if ($tem eq '') {
            $aux =~ s/%%if\(tem\)%%.+?%%\/if%%//isg;
        }
        if ($sub eq '') {
            $aux =~ s/%%if\(sub\)%%.+?%%\/if%%//isg;
        }

        # Agrega procesamiento de ifs para los campos meta.
        if ($meta1 eq '') {
            $aux =~ s/%%if\(meta1\)%%.+?%%\/if%%//isg;
        }
        if ($meta2 eq '') {
            $aux =~ s/%%if\(meta2\)%%.+?%%\/if%%//isg;
        }
        if ($meta3 eq '') {
            $aux =~ s/%%if\(meta3\)%%.+?%%\/if%%//isg;
        }
        for (my $i=1;$i<=20;$i++) {
            if ($metadata[$i-1] eq '') {
                $aux =~ s/%%if\(metadata$i\)%%.+?%%\/if%%//isg;
            }
        }

        # procesamiento de nifs
        if ($fec ne '') {
            $aux =~ s/%%nif\(fec\)%%.+?%%\/nif%%//isg;
        }
        if ($sec ne '') {
            $aux =~ s/%%nif\(sec\)%%.+?%%\/nif%%//isg;
        }
        if ($tem ne '') {
            $aux =~ s/%%nif\(tem\)%%.+?%%\/nif%%//isg;
        }
        if ($sub ne '') {
            $aux =~ s/%%nif\(sub\)%%.+?%%\/nif%%//isg;
        }

        # Agrega procesamiento de nifs para los campos meta.
        if ($meta1 ne '') {
            $aux =~ s/%%nif\(meta1\)%%.+?%%\/nif%%//isg;
        }
        if ($meta2 ne '') {
            $aux =~ s/%%nif\(meta2\)%%.+?%%\/nif%%//isg;
        }
        if ($meta3 ne '') {
            $aux =~ s/%%nif\(meta3\)%%.+?%%\/nif%%//isg;
        }
        for (my $i=1;$i<=20;$i++) {
            if ($metadata[$i-1] ne '') {
                $aux =~ s/%%nif\(metadata$i\)%%.+?%%\/nif%%//isg;
            }
        }
        $aux =~ s/%%ext%%/$ext/ig;
        $aux =~ s/%%num%%/$num/ig;
        $aux =~ s/%%rel%%/$rel/ig;
        $aux =~ s/%%lnk%%/$lnk/ig;
        $aux =~ s/%%tit%%/$tit/ig;
        $fec = &lib_search::iso2fechacorta($fec);
        $aux =~ s/%%fec%%/$fec/ig;
        $aux =~ s/%%res%%/$res/ig;
        $aux =~ s/%%meta1%%/$meta1/ig;
        $aux =~ s/%%meta2%%/$meta2/ig;
        $aux =~ s/%%meta3%%/$meta3/ig;
        $aux =~ s/%%sec%%/$sec/ig;
        $aux =~ s/%%tem%%/$tem/ig;
        $aux =~ s/%%sub%%/$sub/ig;
        $aux =~ s/%%fechac%%/$fechac/ig;
        $aux =~ s/%%ts%%/$ts/ig;

        for (my $i=1;$i<=20;$i++) {
            $aux =~ s/%%metadata$i%%/$metadata[$i-1]/ig;
        }
        $num++;
        $searchloop .= $aux;
    }

    # En forma automatica, crea y parsea variables checked y las variables del formulario.
    foreach my $key (keys %FORM) {
        $aux = $FORM{$key};
        if ($aux =~ /^[\w]+$/) {
            $plantilla =~ s/%%chk_$key\_$aux%%/checked=\"checked\"/isg;
            $plantilla =~ s/%%sel_$key\_$aux%%/selected=\"selected\"/isg;
        }
        $aux =~ s/(['"\<\>])//g;
        $plantilla =~ s/%%$key%%/$aux/isg;
    }
    if ($FORM{'search_orden'} eq 'cro') {
        $msg = $MSGS{'order_cron'}; # cronológico;
    } else {
        $msg = $MSGS{'order_rel'}; # de importancia;
    }
    if ($RESULTADOSTOTALES > 0) {
        $msgsn = $MSGS{'results'} . ' ' . $msg; # "Resultados en orden $msg";
        $msg = $MSGS{'results'} ." $min ". $MSGS{'to'} ." $max " . $MSGS{'of'} . ' '. &lib_search::formatInteger($RESULTADOSTOTALES) . ' ' . $msg;
    } else {
        if ($FORM{'search_texto'} ne '') {
            $msg = $MSGS{'no_results'}; # 'No se encontraron resultados.';
        } else {
            $msg = '';
        }
        $msgsn = $msg;
        $plantilla =~ s/<!--pags-->(.+?)<!--\/pags-->//is;
    }

    # Se agrega marca %%search_total%%
    $plantilla =~ s/%%search_total%%/$RESULTADOSTOTALES/ig;
    $plantilla =~ s/%%search_pag%%/$search_pag/ig;

    if ($paginastotales < 2) { $plantilla =~ s/<!--pags-->(.+?)<!--\/pags-->//is; }
    if ($MSG ne '') { $msg = $MSG; }

    $plantilla =~ s/%%msg%%/$msg/i;
    $plantilla =~ s/%%msgsn%%/$msgsn/i;
    $plantilla =~ s/%%pags%%/$pags/i;
    $plantilla =~ s/<!--resloop-->(.+?)<!--\/resloop-->/$searchloop/is;

    # Parsea nombre de la CGI.
    $plantilla =~ s/%%scriptname%%/$scriptname/g;
    # Elimina tags sin parsear.
    $plantilla =~ s/%%[^%]+%%//sg;
    # Recuperamos caracteres % literales.
    $plantilla =~ s/\[reemplazo_caracter_porciento\]/%/g;
    return $plantilla;

    # print "debug: fin plantilla<br>\n";
}; # parsea_plantilla2

# -------------------------------------------------------------------------#
# Muestra los resultados en el orden requerido.
# En $RESULTADOS{"$idxdir$fidx"} esta la cantidad de veces que las palabras se encuentran dentro del archivo.
# En $NUMRESULT{"$idxdir$fidx"} esta el numero de palabras que está en ese archivo.
# En $FECHAP{"$idxdir$fidx"} esta la fecha de publicacion ese archivo.
# En $FILEDATA{"$idxdir$fidx"} esta los datos de ese archivo.
sub muestra_resultados {
    my ($resultcounter);
    my ($anofidx,$ano,$fidx,$idx,$contenido,$counter);
    my $search_resxpag = $FORM{'search_resxpag'};
    my $search_pag = $FORM{'search_pag'};
    my $min = ($search_pag - 1) * $search_resxpag + 1;
    my $max = $search_pag * $search_resxpag;
    my @resultados;

    $resultcounter = 0;
    $MAXWORDS = 0;
    if ($FORM{'search_orden'} eq 'rel') {

        # Muestra resultados en orden relativo.
        # Usa truco de suponer que las palabras no apareceran mas de 1000 veces dentro de un archivo.
        foreach my $idxdirfidx (sort {(1000 * $NUMRESULT{$b} + $RESULTADOS{$b}) <=> (1000 * $NUMRESULT{$a} + $RESULTADOS{$a})} keys %FILEDATA) {
            $resultcounter++;
            if ( ($min <= $resultcounter) && ($resultcounter <= $max) ) {
                push @resultados, $NUMRESULT{$idxdirfidx} . '|' . $RESULTADOS{$idxdirfidx} . '|' . $FILEDATA{$idxdirfidx};
                if ($MAXWORDS < $NUMRESULT{$idxdirfidx}) {
                    $MAXWORDS = $NUMRESULT{$idxdirfidx};
                }

                # print "<p>$anofidx pushed! "; # debug
            }
        }
    }else{
        # Muestra resultados en orden cronologico.
        foreach my $idxdirfidx (sort {$FECHAP{$b} <=> $FECHAP{$a}} keys %FILEDATA) {
            $resultcounter++;
            if ( ($min <= $resultcounter) && ($resultcounter <= $max) ) {
                push @resultados, $NUMRESULT{$idxdirfidx} . '|' . $RESULTADOS{$idxdirfidx} . '|' . $FILEDATA{$idxdirfidx};

                # print "<p>[" . $FECHAP{$idxdirfidx} . "] $idxdirfidx pushed! "; # debug
                if ($MAXWORDS < $NUMRESULT{$idxdirfidx}) {
                    $MAXWORDS = $NUMRESULT{$idxdirfidx};
                }
            }
        }
    }
    return &parsea_plantilla2(@resultados);
} # muestra_resultados

# -------------------------------------------------------------------------#
# Busca las palabras en el indice de palabras, encontrando los indices de archivo.
# En $RESULTADOS{"$idxdir$fidx"} queda la cantidad de veces que las palabras se encuentran dentro del archivo.
# En $NUMRESULT{"$idxdir$fidx"} queda el numero de palabras que esta en ese archivo.
# En $FECHAP{"$idxdir$fidx"} queda la fecha de publicacion ese archivo.
# En $FILEDATA{"$idxdir$fidx"} quedan los datos de ese archivo.
sub busca_archivos {
    my($idxdir) = $_[0];
    my($widx,$locations,%resultados,$yeswords,$idx,$contenido);
    my $lalinea;
    my $counter = 0;
    my $wordcount = 0;
    my %nofiles; # Hash de archivos a omitir.
    my ($palabraIdx,@wordSequence);
    # print "Busca Archivos $ano:\n";

    # Busqueda binaria en el archivo de palabras.
    foreach my $theword (@FOUNDWORDSIDX) {

        # print "theword=$theword<br>\n"; # debug
        $lalinea = &lib_search::binsearch("$idxdir/$WORDSINDEX_FILENAME",$theword,'num');
        if ($lalinea ne '') {
            # print "lalinea=$lalinea<br>\n"; # debug
            ($widx,$locations) = split(/=/,$lalinea);
            if ($theword == $widx) {
                while ($locations =~ /\|(\d+) (\d+)/g) {
                    my ($fidx,$wcount) = ($1,$2);

                    # Veta los archivos donde se encuentra una not word.
                    if (grep {$widx == $_} @NOTWORDSIDX) {
                        $nofiles{$fidx} = 1;

                        # print '-';
                    }else{
                        $resultados{$fidx} = 1; # Marca resultado. Se usa un hash para eliminar automaticamente los repetidos.
                        $RESULTADOS{"$idxdir$fidx"} += $wcount;
                        $NUMRESULT{"$idxdir$fidx"}++;

                        # Detecta el maximo de palabras encontradas para construir el ranking.
                        if ($MAXPALABRAS < $RESULTADOS{"$idxdir$fidx"}) { $MAXPALABRAS = $RESULTADOS{"$idxdir$fidx"}; }
                        # print '+ '; # debug
                    }
                }
            } # if
        } # if
    } # foreach

    # Elimina los resultados vetados por not words.
    # print "Borrando de los resultados ";
    foreach my $fidx (keys %nofiles) {
        # print ".";
        delete $resultados{$fidx};
        delete $RESULTADOS{"$idxdir$fidx"};
        delete $NUMRESULT{"$idxdir$fidx"};
    }

    # Si el modo es AND, elimina los resultados con menos del total de palabras requeridas.
    if ($FORM{'search_modo'} eq 'and') {
        foreach my $fidx (keys %resultados) {
            # print ".";
            $yeswords = $#YESWORDS + 1;
            if ($NUMRESULT{"$idxdir$fidx"} < $yeswords) {
                delete $resultados{$fidx};
                delete $RESULTADOS{"$idxdir$fidx"};
                delete $NUMRESULT{"$idxdir$fidx"};
            }
        }
    }

    # Lee los datos del archivo de cada resultado, aplicando filtros si es pertinente.
    if (-f "$idxdir/$FILES_FILENAME") { # Verifica que archivo existe antes de abrirlo.
        open(my $IN,'<', "$idxdir/$FILES_FILENAME") || die "No puedo abrir $idxdir/$FILES_FILENAME $!";
        binmode $IN, ':encoding(UTF-8)';
        $counter = 0;
        foreach my $fidx (sort {$a <=> $b} keys %resultados) { # Ordena por numero, ya que asi estan ordenados en el indice.
            while (<$IN>) {
                $counter++;
                last if ($counter == $fidx); # Se salta las lineas que no son, hasta que encuentra la correcta.
            }
            chomp;
            ($idx,$contenido) = split(/=/,$_,2);
            if ($fidx != $idx) { print STDERR "Desfase $idxdir/$FILES_FILENAME: $fidx != $idx\n"; last; }
            # debug
            if (&filtraresultado($contenido) == 1) {
                # Mete los resultados al hash de resultados
                $FILEDATA{"$idxdir$fidx"} = $contenido;
                $FECHAP{"$idxdir$fidx"} = (split(/\|/,$contenido))[0];
                $RESULTADOSTOTALES++;
            }
        }
        close $IN;
    }

    # Si se ha buscado por frases, elimina los archivos donde las palabras de la frase no estan en secuencia.
    return if ($#FRASES < 0);
    if (-f "$idxdir/$FILESINDEXF_FILENAME") { # Verifica que archivo existe antes de abrirlo.
        open(my $IN ,'<', "$idxdir/$FILESINDEXF_FILENAME") || die "No puedo abrir $idxdir/$FILESINDEXF_FILENAME $!";
        binmode $IN, ':encoding(UTF-8)';
        $counter = 0;
        foreach my $fidx (sort {$a <=> $b} keys %resultados) { # Ordena por numero, ya que asi estan ordenados en el indice.
            while (<$IN>) {
                $counter++;
                last if ($counter == $fidx); # Se salta las lineas que no son, hasta que encuentra la correcta.
            }
            chomp;
            ($idx,$contenido) = split(/=/,$_,2);
            if ($fidx != $idx) { print STDERR "Desfase $idxdir/$FILESINDEXF_FILENAME: $fidx != $idx\n"; last; }
            # Verifica que las palabras de las frases son secuenciales en el archivo.
            # Tenemos "palabra1 palabra2 ..." contra widx ubic1 ubic2 ubic3...|widx2 ubic1 ubic2 ...
            foreach my $frase (@FRASES) {
                # Crea secuencia patron de palabras.
                @wordSequence = ();
                foreach my $palabra (split(/ /,$frase)) {
                    if ($palabra =~ /[a-z0-9A-Z]{2}/) {
                        push @wordSequence, $WORDSINDEXES{$palabra};
                    }
                }

                # Comprueba si la secuencia se cumple para el archivo.
                if (&matchSequence(\@wordSequence,$contenido) == 0) {

                    # Elimina el resultado del hash de resultados
                    delete $FILEDATA{"$idxdir$fidx"};
                    delete $FECHAP{"$idxdir$fidx"};
                    $RESULTADOSTOTALES--;
                }
            }
        }
        close $IN;
    }

} # busca_archivos

# -------------------------------------------------------------------------#
# Busca las palabras en los archivos de indice del ano indicado.
# Retorna punteros a los arreglos de indices de las palabras encontradas
# y de indices de stopwords encontradas.
sub busca_palabras {
    my($idxdir) = $_[0];
    my(@searchwords,$count,$wordcount,$laletra,$palabra,$widx);
    my(@foundwordsidx,@notwordsidx,%wordsindexes);

    # Si no hay wildcards, usa el indice de letras para acelerar la busqueda.
    if ($TURBOMODE == 1) {

        # Reconstruye las search words ya validadas.
        @searchwords = @YESWORDS;
        push @searchwords, @NOTWORDS;
        @searchwords = sort @searchwords;

        # Busqueda binaria.
        foreach my $lapalabra (@searchwords) {
            # print "lapalabra=$lapalabra<br>\n"; # debug
            my $lalinea = &lib_search::binsearch("$idxdir/$WORDS_FILENAME",$lapalabra,'str');
            if ($lalinea ne '') {
                ($palabra,$widx) = split(/=/,$lalinea);

                # Registra equivalencia para ser usada en la busqueda por frases.
                $wordsindexes{$palabra} = $widx;

                # print "lalinea=$lalinea widx=$widx<br>\n"; # debug
                push @foundwordsidx, $widx; # Igual la guarda como parte de los resultados.
                if (grep {$palabra eq $_} @NOTWORDS) { # Registra indices de las not words.
                    push @notwordsidx,$widx;

                    # print "Not Word idx: $palabra $widx\n"; # debug
                }
            }
        }

    }else{ # Hay wildcards y cosas asi.
        # print "NOT Turbo Mode ***\n";
        # Lee indice de palabras. Implementa match con expresion regular,
        # asi que no puede usar el indice de letras ni busqueda binaria.
        # print "laletra=$laletra offset=" . $LETRAS{$laletra} . "\n"; # debug
        if (-f "$idxdir/$WORDS_FILENAME") {
            open(my $IN,'<', "$idxdir/$WORDS_FILENAME");
            binmode $IN, ':encoding(UTF-8)';
            while (<$IN>) {
                chomp;
                ($palabra,$widx) = split(/=/,$_);
                if (grep {$palabra =~ /$_/} @NOTWORDS) { # Registra indices de las not words.
                    # print "Found Not: $palabra $widx\n"; # debug
                    push @notwordsidx,$widx;   # Registra indices de las not words.
                    push @foundwordsidx,$widx; # Igual la guarda como parte de los resultados.
                }elsif (
                    grep {
                        $palabra =~ /$_/
                    } @YESWORDS
                  ) {
                    # print "Found Yes: $ano $palabra $widx | @YESWORDS\n"; # debug
                    push @foundwordsidx,$widx;
                }
            }
            close $IN;
        }
    }
    return (\@foundwordsidx,\@notwordsidx,\%wordsindexes);
}; # busca_palabras

# -------------------------------------------------------------------------#
# Aplica los filtros pertinentes a los resultados.
# Si el resultado esta dentro de rango, retorna 1.
sub filtraresultado {
    return 1 if ($FILTROACTIVO == 0);
    my($fechap,$file,$tit,$meta1,$meta2,$meta3,$res,$sec,$tem,$sub) = split(/\|/,$_[0]);
    if ($FORM{'search_meta1'} ne '') {
        return 0 if ($FORM{'search_meta1'} !~ /(^| |,)$meta1($| |,)/i); # Busca coincidencia con alguno de los parametros.
    }
    if ($FORM{'search_meta2'} ne '') {
        return 0 if ($FORM{'search_meta2'} !~ /(^| |,)$meta2($| |,)/i); # Busca coincidencia con alguno de los parametros.
    }
    if ($FORM{'search_meta3'} ne '') {
        return 0 if ($FORM{'search_meta3'} !~ /(^| |,)$meta3($| |,)/i); # Busca coincidencia con alguno de los parametros.
    }
    if ($FORM{'search_seccion'} ne '') {
        return 0 if ($FORM{'search_seccion'} ne $sec);
    }
    if ($FORM{'search_tema'} ne '') {
        return 0 if ($FORM{'search_tema'} ne $tem);
    }
    if ($FORM{'search_subtema'} ne '') {
        return 0 if ($FORM{'search_subtema'} ne $sub);
    }
    if ($FORMfechaini ne '') {
        return 0 if ($FORMfechaini > $fechap);
    }
    if ($FORMfechafin ne '') {
        return 0 if ($FORMfechafin < $fechap);
    }
    return 1;
}; # filtraresultado

# -------------------------------------------------------------------------#
# Encuentra los anos indexados.
sub get_anos {
    my($dir) = $_[0];
    my(@anos);
    my(@files) = &lib_search::lee_dir($dir);
    foreach my $entry (@files) {

        # print $entry; # debug
        if ($entry =~ /^\d{4}$/) {
            push @anos, $entry;
        }
    }
    return @anos;
}; # get_anos

# -------------------------------------------------------------------------#
# Encuentra los indices presentes en el directorio search.
sub get_idx {
    my($dir) = $_[0];
    my(@idx);
    my(@files) = &lib_search::lee_dir($dir);
    foreach my $entry (sort @files) {
        # print $entry; # debug
        if ( ($entry !~ /^\./) && (-d "$dir/$entry") ) {
            push @idx, $entry;
        }
    }
    return @idx;
}; # get_idx

# -------------------------------------------------------------------------#
# Depura el texto ingresado para determinar las palabras a buscar.
sub depura_palabras {
    my($texto) = $FORM{'search_texto'};
    my(@searchwords,$caracteres,$frase);
    my(@yeswords,@notwords,@frases,$turbomode,@aux);

    $texto = &lib_search::notildesUtf8($texto);
    $texto = lc $texto;
    $texto =~ s/[^0-9a-z\*\?\"]/ /g;
    # Si no hay letras validas, retorna al tiro.
    if ($texto !~ /[0-9a-z]{2}/) {
        return(\@yeswords,\@notwords,1,\@frases);
    }

    # Si esta definido el archivo, loguea la busqueda.
    if ($CFG{'SEARCH_LOGFILE'} ne '') {
        if ( open(my $LOG,'>>', $DOCUMENT_ROOT.$CFG{'SEARCH_LOGFILE'}) ) {
            binmode $LOG, ':encoding(UTF-8)';
            print $LOG &lib_search::timestamp_iso() . ' ' . $ENV{'REMOTE_ADDR'} . ' ' . $texto . "\n";
            close $LOG;
        }
    }

    # Detecta las frases.
    while ($texto =~ /\"([^\"]+)\"/g) {
        $frase = $1;
        $frase =~ s/[^0-9a-z]/ /g; # Depura la frase. Elimina stopwords, wildcards, etc.
        $frase =~ s/(^| )[0-9a-z]( |$)/ /g; # Elimina letras sueltas.
        $frase =~ s/ +/ /g; # Elimina los espacios multiples.
        push @frases, $frase;
    }
    if ($#frases >= 0) { @NOBUSCAR = (); }
    # Cuando se busca por frases se busca por todo.
    # Por el momento no hay mas procesamiento con las frases, lo cual significa que
    # si alguien metio un comodin o una stopword dentro de la frase, habra resultados
    # incorrectos.
    $texto =~ s/\"/ /g; # Elimina las comillas.
    $texto =~ s/^ +//g; # Elimina los espacios de adelante.
    $texto =~ s/ +$//g; # Elimina los espacios de atras.
    $texto =~ s/ +/ /g; # Elimina los espacios multiples.
    if ($DEBUG) { print "texto=[$texto]\n"; }
    @searchwords = split(/ /,$texto);

    # Detecta los "nots", valida palabras y procesa wildcards: * y ?
    $turbomode = 1; # TURBOMODE=1 => no hay wildcards. Es busqueda exacta.
    @notwords = ();
    foreach my $palabra (@searchwords) {
        # Omite la palabra si tiene menos de dos caracteres significativos.
        $caracteres = $palabra;
        $caracteres =~ s/[^0-9a-z]//g;
        next if (length($caracteres) < 2);
        next if (grep {$_ eq $caracteres} @NOBUSCAR);
        # Detecta stop words.
        if (substr($palabra,0,1) eq '-') {
            $palabra = substr($palabra,1);

            # Detecta wildcards si es que estan habilitadas.
            if ( (lc $FORM{'search_comodines'}) eq 'yes') {
                if ( ($palabra =~ s/\*/\.\*/g) || ($palabra =~ s/\?/\./g) ) {
                    $turbomode = 0;
                }
            }else{
                # Elimina wildcards.
                $palabra =~ s/[\*\?]+//g;
            }
            push @notwords, $palabra;

            # print "Not word: $palabra\n"; # debug
        }else{
            # Detecta wildcards si es que estan habilitadas.
            if ( (lc $FORM{'search_comodines'}) eq 'yes') {
                if ( ($palabra =~ s/\*/\.\*/g) || ($palabra =~ s/\?/\./g) ) {
                    $turbomode = 0;
                }
            }else{
                # Elimina wildcards.
                $palabra =~ s/[\*\?]+//g;
            }

            # Separa palabras compuestas por guiones para no buscarlas como una sola palabra,
            # ya que el indexador no las considera como una sola palabra.
            @aux = ();
            if (@aux = split(/-/,$palabra)) {
                push @yeswords, @aux;
            }else{
                push @yeswords, $palabra;
            }

            # print "Yes word: $palabra\n"; # debug
        }
    }
    if ($turbomode == 0) {
        foreach my $palabra (@yeswords) {
            $palabra = "\^$palabra\$"; # Agrega delimitadores regexp.
        }
        foreach my $palabra (@notwords) {
            $palabra = "\^$palabra\$"; # Agrega delimitadores regexp.
        }
    }
    return(\@yeswords,\@notwords,$turbomode,\@frases);
} # depura_palabras

# -------------------------------------------------------------------------#
# Lee el indice de letras.
sub leeletras {
    my($idxdir) = $_[0];
    my($letra,$posicion);

    # Lee indice de letras.
    if (-f "$idxdir/$CARS_FILENAME") {
        open(my $IN,'<', "$idxdir/$CARS_FILENAME");
        binmode $IN, ':encoding(UTF-8)';
        while (<$IN>) {
            chomp;
            ($letra,$posicion) = split(/=/,$_);
            $LETRAS{$letra} = $posicion;
        }
        close $IN;
    }
} # leeletras

# ------------------------------------------------------------------------- #
# Elimina los archivos de mas de 10 minutos de antiguedad.
sub garbageCollection {
    my($eldir) = $_[0];
    my(@entries);

    # Abre directorio.
    if (opendir(my $DIR, $eldir)) {
        @entries = readdir($DIR);
        closedir $DIR;
        foreach my $file (@entries) {

            # Por seguridad, solo limpia archivos con la firma de esta aplicacion.
            next if ($file !~ /search/);

            # Detecta si son archivos.
            if (-f "$eldir/$file") {
                if ((stat("$eldir/$file"))[9] < (time - 600)) {
                    unlink "$eldir/$file";
                }
            }
        }
    }
} # garbageCollection

# ------------------------------------------------------------------------- #
# Escribe y entrega el archivo temporal.
sub muestraResultado {
    my $answerid = 'search' . time . $$;
    &lib_search::escribe_archivo("$CACHE_DIR/$answerid\.$EXT",$RESULTADO);
    print "Status: 303 See Other\n";
    print "Location: $CACHE_REL_DIR/$answerid\.$EXT\n\n";
    &garbageCollection($CACHE_DIR); # Limpia directorio de cache.
    exit;
}; # muestraResultado

# -------------------------------------------------------------------#
# Comprueba si la secuencia se cumple para el archivo.
# Los parametros son un arreglo de palabras y una linea del archivo xxx:
sub matchSequence {
    my($wordSeq,$line) = @_;
    my @wordSequence = @$wordSeq;
    my(@words,@sequences,$i,$j,$k,$ubi2match,$ubi,@matches1,@matches2,$matched);

    # Extrae las secuencias de cada palabra, o sea, la lista de ubicaciones de esa palabra dentro del archivo,
    # representado por la linea $line que es asi: |id_palabra1 ubic11 ubic12 ubic13 ...|idpalabra2 ubic21 ubic22 ...
    foreach my $widx (@wordSequence) {
        if ($DEBUG) { print "widx=[$widx]\n"; }

        # Extrae parte de la linea que contiene la palabra.
        if ($line =~ /\|$widx ([^\|]+)/) {
            if ($DEBUG) { print "Encuentra $1\n"; }
            push @sequences, [ split(/ /,$1) ];
        }else{
            if ($DEBUG) { print "palabra no esta en $line\n"; }
            return 0; # No hay match pq la palabra no se encuentra en la linea.
        }
    }

    # Matchea cada secuencia con la siguiente hasta encontrar 1 match.
    # Para que la frase exista dentro del archivo se debe dar que al menos una de las series de ubicaciones
    # debe ser consecutiva, o sea, p. e., ubic12 + 1 = ubic21; ubic21 + 1 = ubic34 y asi ...
    # Para determinar esto hay que verificar todos los "caminos" posibles, partiendo del primer
    # conjunto de ubicaciones, comparando con el siguiente conjunto para encontrar numeros que sean sucesivos.
    $matched = 0;
    @matches1 = @{$sequences[0]}; # Parte con todas las primeras palabras matcheando.
    for($i=0;$i<$#sequences;$i++) {

        # Matchea cada ubicacion de la secuencia i contra las ubicaciones de las secuencias i+i.
        @matches2 = ();
        for($j=0;$j<=$#{$sequences[$i]};$j++) {

            # Solo explora el nivel si fue matcheado.
            if (grep {$sequences[$i][$j] == $_} @matches1) {
                $ubi = $sequences[$i][$j];
                $ubi2match = $ubi + 1;
                if ($DEBUG) { print 'busca ' . $sequences[$i][$j] . ":\n"; }
                for($k=0;$k<=$#{$sequences[$i+1]};$k++) {

                    # if ($DEBUG) { print '  ' . $sequences[$i+1][$k]; };
                    if ($ubi2match == $sequences[$i+1][$k]) {
                        if ($DEBUG) { print $sequences[$i+1][$k] . " esta! "; }
                        push @matches2, $sequences[$i+1][$k]; # Recuerda que este nivel fue matcheado.
                        # Si se trata del ultimo elemento en la secuencia, entonces da por matcheada la frase y sale.
                        if (($i+1) == $#sequences) {
                            $matched = 1;
                            last;
                        }
                    }else{
                        # if ($DEBUG) { print " NO esta! "; };
                    }
                }
                if ($DEBUG) { print "\n"; }
            }
        }
        @matches1 = @matches2;
    }
    if ($matched == 1) {
        if ($DEBUG) { print "\nFrase encontrada en $line\n"; }
        return 1;
    }else{
        if ($DEBUG) { print "\nFrase NO encontrada en $line\n"; }
        return 0;
    }
}; # matchSequence

# -------------------------------------------------------------------#
sub carga_variables_prontus {
    my $buffervarcfg = &lib_search::lee_archivo("$PRONTUS_DIR/cpan/$PRONTUS-var.cfg");

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} = 'N';
    if ($buffervarcfg =~ m/\s*FRIENDLY_URLS_VERSION\s*=\s*["'](.*?)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_LARGO_TITULAR'} = 75;
    if ($buffervarcfg =~ m/\s*FRIENDLY_URLS_LARGO_TITULAR\s*=\s*["'](\d+)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_LARGO_TITULAR'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_INCLUDE_VIEW_NAME'} = 'NO';
    if ($buffervarcfg =~ m/\s*FRIENDLY_INCLUDE_VIEW_NAME\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_INCLUDE_VIEW_NAME'} = $1;
    }

    $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_V4_INCLUDE_PRONTUS_ID'} = 'SI';
    if ($buffervarcfg =~ m/\s*FRIENDLY_V4_INCLUDE_PRONTUS_ID\s*=\s*["'](SI|NO)["']/) {
        $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_V4_INCLUDE_PRONTUS_ID'} = $1;
    }
}

# --------------------------------------------------------------------------- #
sub correct_encode_utf8_string {
    if (!utf8::is_utf8($_[0])) {
        utf8::decode($_[0]);
    }
}

# -------------------------------------------------------------------#
# Rescata y valida las variables del chorro.
sub getFormData {
    my $buffer;
    if ($ENV{'REQUEST_METHOD'} eq 'GET') {
        $buffer = $ENV{'QUERY_STRING'};
    } else {
        read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
    }
    my @pairs = split(/&/, $buffer);
    foreach my $pair (@pairs) {
        my($name, $value) = split(/=/,$pair);

        # Un-Webify plus signs and %-encoding
        $value =~ tr/+/ /;
        $value =~ s/%00//sg; # 1.17
        $value =~ s/%([0-9A-Ha-h]{2})/pack("c",hex($1))/ge;

        $value =~ s/\.\.\///g; # Elimina toda referencia de directorios hacia atras.
        $value =~ s/\x00//g;   # Elimina nulls.
        $value =~ s/\x1B//g;   # Elimina escapes.
        $value =~ s/[<>%!\|\\\~\$]/ /g; # Elimina caracteres peligrosos
        $name = lc $name;

        &correct_encode_utf8_string($name);
        &correct_encode_utf8_string($value);
        # print "$name = [$value]\n"; # debug

        if ( ($name ne 'search_seccion') && ($name ne 'search_tema') && ($name ne 'search_subtema') ) {
            $FORM{$name} = lc $value;
        } else {
            $FORM{$name} = $value;
        }
    }

    # Valida variables.
    if(!&lib_prontus::valida_prontus($FORM{'search_prontus'})) {
        print "Content-type: text/html\n";
        print "\nDirectorio Prontus no v&aacute;lido";
        exit;
    }

    $FORM{'search_tmp'} =~ s/[^\w\-\.]//g; # Elimina caracteres no validos como nombres de archivo.
    if ($FORM{'search_tmp'} eq '')     { $FORM{'search_tmp'} = 'search.html'; }
    $FORM{'search_idx'} =~ s/[^\w\-\.]//g;
    if ($FORM{'search_idx'} eq '')     { $FORM{'search_idx'} = $FORM{'search_prontus'}; }
    $FORM{'search_resxpag'} =~ s/[^0-9]//g;
    $FORM{'search_maxpags'} =~ s/[^0-9]//g;
    $FORM{'search_pag'} =~ s/[^0-9]//g;
    if ($FORM{'search_pag'} eq '')     { $FORM{'search_pag'} = 1; }
    $FORM{'search_orden'} =~ s/[^\w]//g;
    if ($FORM{'search_orden'} eq '')   { $FORM{'search_orden'} = 'cro'; }
    $FORM{'search_form'} =~ s/[^\w]//g;
    if ($FORM{'search_form'} eq '')    { $FORM{'search_form'} = 'yes'; }
    $FORM{'search_modo'} =~ s/[^\w]//g;
    if ($FORM{'search_modo'} eq '')    { $FORM{'search_modo'} = 'or'; }
    $FORM{'vista'} =~ s/[^\w]//g;
    $VISTA = '';
    if ($FORM{'vista'} ne '')  { $VISTA = '-' . $FORM{'vista'}; };

    if (   ($FORM{'search_seccion'} ne '')
        || ($FORM{'search_tema'} ne '')
        || ($FORM{'search_subtema'} ne '')
        || ($FORM{'search_fechaini'} ne '')
        || ($FORM{'search_fechafin'} ne '')
        || ($FORM{'search_meta1'} ne '')
        || ($FORM{'search_meta2'} ne '')
        || ($FORM{'search_meta3'} ne '')) {
        $FILTROACTIVO = 1;
    }else{
        $FILTROACTIVO = 0;
    }
    if (length($FORM{'search_texto'}) > 64) {
        $FORM{'search_texto'} = &lib_prontus::ajusta_nchars($FORM{'search_texto'}, 64);
        $FORM{'search_texto'} =~ s/\.\.\.$//g; # elimina puntos finales.
    }
    if ($FORM{'search_fechaini'} ne '') {
        $FORMfechaini = &lib_search::fecha2iso($FORM{'search_fechaini'});

        # Para limpiar/formatear la fecha
        if($FORMfechaini eq '') {
            $FORM{'search_fechaini'} = '';
        } else {
            $FORM{'search_fechaini'} = &lib_search::iso2fechacorta($FORMfechaini);
        }
    }
    if ($FORM{'search_fechafin'} ne '') {
        $FORMfechafin = &lib_search::fecha2iso($FORM{'search_fechafin'});

        # Para limpiar/formatear la fecha
        if($FORMfechaini eq '') {
            $FORM{'search_fechafin'} = '';
        } else {
            $FORM{'search_fechafin'} = &lib_search::iso2fechacorta($FORMfechafin);
        }
    }

    # se limpian las variables restantes
    $FORM{'search_comodines'} =~ s/[^\w]//g;
    if ($FORM{'search_comodines'} ne 'yes')    { $FORM{'search_comodines'} = 'no'; }
}; # getFormData
