#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------

#-------------------------------COMENTARIO GLOBAL---------------
#---------------------------------------------------------------
# PROPOSITO.
#-----------
# Manipulacion general de strings
#---------------------------------------------------------------

#---------------------------------------------------------------
# DECLARACIONES GLOBALES.
#------------------------

package glib_str_03;
use utf8;
use strict;

#---------------------------------------------------------------
# SUB-RUTINAS.
#-------------
# Retorna un nro. formateado con punto separador de miles.
sub format_miles {
    my $num = $_[0];

    $num =~ s/\D//g;
    $num += 0;

    my $sal = '';
    if ($num =~ s/((\d){3})$//) {
        $sal = $1;
    } else {
        $sal = $num;
        $num = '';
    }
    while ($num =~ s/((\d){3})$//) {
        $sal = "$1\.$sal";
    }
    if ($num ne '') {
        $sal = "$num\.$sal";
    }

    return $sal;
}

#---------------------------------------------------------------
# Recibe numero real y lo formatea con separadores de miles y cant. de decimales deseada.
sub format_miles_dec {
    my ($valor1, $valor2, $dec) = ($_[0], $_[0], $_[1]);

    my $signo = '';
    if ($valor1 < 0) {
        $signo = '-';
    }

    $valor1 = $valor2 = abs $valor1;
    # Si el valor hay que devolverlo como entero sin decimales.
    if ($dec == 0) {
        if ($valor1 =~ /\.(\d)/) {
            my $d = $1;
            $valor1 += 1 if ($d >= 5);
        }
    }

    # Si el valor hay que devolverlo como entero con decimales.
    $valor1 += '0.' . '0' x ($dec - 1) . (1 - ($valor2 - int($valor2))) if ($valor2 - int($valor2) and $dec > 0) >= 0.5;
    $valor2 = sprintf("%.${dec}f", $valor2);
    $valor2 =~ s/^\d+\.//g;

    $valor1 =~ s/\..*//g;

    $valor1 = &format_miles($valor1);

    if ($dec > 0) {
        $valor1 .= ',' . $valor2;
    }

    if ($signo eq '-') {
        $valor1 = '-' . $valor1;
    }

    return $valor1;
}

#---------------------------------------------------------------
sub bytes2kb {
    my $bytes = $_[0];
    $bytes = $bytes / 1024;
    $bytes = 0 if ($bytes < 0);
    $bytes = &format_miles_dec($bytes, 0);
    $bytes .= ' KiB';
    return $bytes;
}

#-------------------------------------------------------------------------#
# Valida nombres de directorios ingresados por el usuario.
# Param :
# 0) Nombre del directorio (sin path) a validar

# Retornos : 1 si esta ok, 0 en caso contrario
sub chars_dir_ok {
    my $nombre = $_[0];

    if ($nombre =~ /[^a-z0-9_~]/) {
        return 0;
    } else {
        return 1;
    }
}

#-------------------------------------------------------------------------#
# Valida que el dato ingresado contenga sólo digitos.

# Param :
# 0) String a validar

# Retornos : 1 si esta ok, 0 en caso contrario
sub is_digit {
    my $valor = $_[0];

    if ($valor =~ /[^0-9]/) {
        return 0;
    } else {
        return 1;
    }

}

#-----------------------------------------------------------------------#
# Devuelve solo los digitos de un string.

# Param :
# 0) String de entrada.

# Retorna : string con todos los caracteres que no eran digitos
# eliminados.
sub solo_digitos {
    my $texto = $_[0];
    $texto =~ s/[^0-9]//g;
    return $texto;
}

#------------------------------------------------------------------------------#
# Retorna valor con un 0 adelante si el largo del string pasado
# es < N, y 00 si el largo es 0.

# Param :
# 0) String de entrada.
# 1) N (nro. de posiciones a las que se desea formatear el nro.)

# Retorna : string formateado.
sub format_n {
    my ($aux, $length) = @_;

    $aux = &solo_digitos($aux);

    $aux = substr($aux, 0, $length);

    while (length($aux) < $length) {
        $aux = "0$aux";
    }
    return $aux;
}

#------------------------------------------------------------------------------#
sub trim {
    my $valor = $_[0];
    $valor =~ s/^[\s\n\r]+//;
    $valor =~ s/[\s\n\r]+$//;
    return $valor;
}

#------------------------------------------------------------------------------#
# Funcion que calcula el digito verificador de un rut dado.
# Parametro:
# 0) rut sin dv.
sub devuelve_dv_rut {
    my $r;
    for ($_ = $_[0] ; s/\d$// || ($_ = (11 - $a % 11) % 10) * 0 ; $a += $& * (++$r - 7 ? $r : ($r = 1)) + $&) {
        # nada.
    }
    return $_ || "K";
}

#------------------------------------------------------------------------------#
# Funcion que encuentra el caracter buscado dentro de un string
# Parametros:
# 0) String;
# 1) caracter(es) a buscar.
sub instr {
    my ($contador, $campo_pre);
    if (length($_[0]) > 0) {
        for ($contador = 1 ; $contador le length($campo_pre) ; $contador++) {
            last if (substr($campo_pre, $contador, length($_[1])) = $_[1]);
        }
        return $contador;
    } else {
        return 0;
    }
}

# --------------------------------------------------------------------------- #
# Retorna un string aleatorio de n letras.
sub random_string {
    my $n       = $_[0];
    # 51 caracteres
    my $charset = 'ABCDEFGHJKLMNPRTUVWXY23456789abcdeghijkmnoprsuvwxyz';
    my $i;
    my $salida = '';
    for ($i = 0 ; $i < $n ; $i++) {
        $salida .= substr($charset, int(rand(51)), 1);
    }
    # print "$salida\n"; # debug
    return $salida;
}

# --------------------------------------------------------------------------- #
sub solo_texto {
    my $str = $_[0];
    $str =~ s/[^\wáéíóúÁÉÍÓÚüÜñÑ]/ /sg;    # solo permite nombres normales
    return $str;
}

# --------------------------------------------------------------------------- #
sub correct_encode_utf8 {
    my $hashref = $_[0];
    foreach my $key (keys %{$hashref}) {
        if (!utf8::is_utf8($hashref->{$key})) {
            utf8::decode($hashref->{$key});
        }
    }
}

# --------------------------------------------------------------------------- #
sub correct_encode_utf8_string {
    if (!utf8::is_utf8($_[0])) {
        utf8::decode($_[0]);
    }
}

1;
