#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------

#-------------------------------COMENTARIO GLOBAL---------------
#---------------------------------------------------------------
# PROPOSITO.
#-----------
# Funciones centralizadas para el sistema de tags
#---------------------------------------------------------------

#---------------------------------------------------------------
# DECLARACIONES GLOBALES.
#------------------------

package lib_tags;
use strict;
use utf8;
use prontus_varglb;
use DBI;
use JSON;

use glib_dbi_03;
use glib_fildir_03;
use glib_hrfec_03;
use lib_helper;
use prontus_varglb;
use lib_urlf5;

require lib_prontus;
require Artic;

our $MAX_ARTICS_PER_SEARCH  = 20;
our $MAX_TAGS_SEARCH_RESULT = 20;
our $MAX_LIMIT = 50; # Limite maximo permitido para la generación de relacionados.
our $RELDIR_ARTIC = '';
our $RELDIR_ARTIC_RELAC = '';
our $NUM_RELAC_DEFAULT = '';

our $RELDIR_DST_ARTIC_RELAC = '';
# ---------------------------------------------------------------
# SUB-RUTINAS.
# ---------------------------------------------------------------

# configura las variables globales.
sub set_vars {
    my ($dir_contenido, $dir_artic, $dir_pag, $dir_temp, $dir_tags, $num_relac_default) = @_;
    $RELDIR_ARTIC           = $dir_contenido . $dir_artic . '/%%dirfecha%%' . $dir_pag;
    $RELDIR_ARTIC_RELAC     = $dir_temp . $dir_tags;
    $RELDIR_DST_ARTIC_RELAC = $dir_contenido . $dir_tags;
    $NUM_RELAC_DEFAULT      = $num_relac_default;
}

# Verifica que el nombre no exista en la tabla.
sub check_nom {
    my ($db, $id, $nombre) = ($_[0], $_[1], $_[2]);
    my @params;
    my $sql;

    if ($id) {
        $sql = "select count(TAGS_TAG) from TAGS where lower(TAGS_TAG) = lower(?) and TAGS_ID <> ?";
        push @params, $nombre;
        push @params, $id;
    } else {
        $sql = "select count(TAGS_TAG) from TAGS where lower(TAGS_TAG) = lower(?)";
        push @params, $nombre;
    }

    my $cant;
    my $salida = &glib_dbi_03::ejecutar_sql_bind_param($db, $sql, \@params, \$cant);

    $salida->fetch;
    $salida->finish;

    if ($cant > 0) {
        return 'Nombre de ítem ya existe.';
    }

    return '';
}

# ---------------------------------------------------------------
sub get_tags_for_fid {
    my $base = $_[0];
    my $ts   = $_[1];
    my $tags_order = $_[2];
    my @params;

    # tags asignados
    my $sql = "select TAGSART_IDTAGS,TAGS_TAG from TAGSART join TAGS on TAGS_ID=TAGSART_IDTAGS where TAGSART_IDART = ?";
    push @params, $ts;

    if ($tags_order) {
        $sql .= " order by FIELD(TAGS_ID, ?);";
        push @params, $tags_order;
    } else {
        $sql .= " order by TAGS_TAG asc;";
    }

    my $tags_query = &glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);
    my ($idtag_asignado, $tag_name);

    $tags_query->bind_columns(undef, \($idtag_asignado, $tag_name));

    # Se leen los tags de la BD
    my $tags_asignados = '';

    while ($tags_query->fetch) {
        &glib_str_03::correct_encode_utf8_string($tag_name);
        $tag_name = &lib_prontus::escape_xml($tag_name);
        $tags_asignados = $tags_asignados . "$idtag_asignado|$tag_name,";
    }

    $tags_asignados =~ s/,$//;
    $tags_query->finish;

    # warn("tags_asignados[$tags_asignados]");
    return $tags_asignados;
}

# ---------------------------------------------------------------
sub get_all_tags {
    my $base = $_[0];
    my %hash_tags;

    my $sql = "select TAGS_ID, TAGS_TAG from TAGS where TAGS_MOSTRAR = 1 order by TAGS_TAG asc";
    my $tab_output = &glib_dbi_03::ejecutar_sql($base, $sql);

    my ($idtag, $tag_name);
    $tab_output->bind_columns(undef, \($idtag, $tag_name));

    while ($tab_output->fetch) {
        &glib_str_03::correct_encode_utf8_string($tag_name);
        $hash_tags{$idtag} = $tag_name;
    }
    return %hash_tags;
}

# ---------------------------------------------------------------
sub get_tag_noms {
    my $base   = shift;
    my $tag_id = shift;
    my %tag_noms;
    my @params;
    my $sql = "select TAGS_TAG, TAGS_NOM4VISTAS from TAGS where TAGS_ID = ?";

    push @params, $tag_id;

    my $tag;
    my $nom4vistas;
    my $salida = &glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($tag, $nom4vistas));

    while ($salida->fetch) {
        &glib_str_03::correct_encode_utf8_string($tag);
        &glib_str_03::correct_encode_utf8_string($nom4vistas);

        $tag_noms{''} = &lib_prontus::escape_html($tag);    # nombre en la vista default o ppal

        while ($nom4vistas =~ /(^|\n)(\w+)\t(.*?)($|\n)/sg) {
            my $vista = $2;
            my $nom   = $3;

            $tag_noms{$vista} = &lib_prontus::escape_html($nom);    # nombre en cada vista
        }
    }

    $salida->finish;

    return %tag_noms;
}

sub get_tags_noms_hash {
    my $base = $_[0];
    my $tags_id = $_[1];
    my @tags_id_arr = split("," , $tags_id);
    my %tags_noms;

    if (!$tags_id) {
        return %tags_noms;
    }

    my $placeholders = ', ?' x scalar @tags_id_arr;
    $placeholders = substr $placeholders, 2;
    my $sql = "select TAGS_ID, TAGS_TAG, TAGS_NOM4VISTAS from TAGS where TAGS_ID IN ($placeholders)";
    my $tag;
    my $tag_id;
    my $nom4vistas;
    my $salida = &glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@tags_id_arr, \($tag_id, $tag, $nom4vistas));

    while ($salida->fetch) {
        &glib_str_03::correct_encode_utf8_string($tag);
        &glib_str_03::correct_encode_utf8_string($nom4vistas);

        $tags_noms{$tag_id}{''} = &lib_prontus::escape_html($tag);

        while ($nom4vistas =~ /(^|\n)(\w+)\t(.*?)($|\n)/sg) {
            my $vista = $2;
            my $nom   = $3;

            $tags_noms{$tag_id}{$vista} = &lib_prontus::escape_html($nom);
        }
    }

    $salida->finish;

    return %tags_noms;
}

# Dada una id de tag, devuelve la URL custom, si existe en db.
# De otro modo devuelve la URL estándar de tagport
# Intenta leer el archivo json de tags.
# @param int $tag_id
# @return string
sub get_tag_url {
    my $tag_id = $_[0];
    my $url = '';

    my @params = ($tag_id);
    my $sql = "SELECT TAGS_PORT";
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} eq '5') {
        $sql .= ", URLF5_URL FROM TAGS LEFT JOIN URLF5 ON TAGS_ID=URLF5_SUBTYPEID";
    } else {
        $sql .= " FROM TAGS";
    }
    $sql .= " WHERE TAGS_ID = ?";

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} eq '5') {
        $sql .= " AND URLF5_TYPE = ?";
        push(@params, 'tag')
    }

    my $salida = &glib_dbi_03::ejecutar_sql_param($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $sql, \@params);
    my $row = $salida->fetchrow_hashref;
    if ($row->{'URLF5_URL'}) {
        $url = $row->{'URLF5_URL'};
    } elsif ($row->{'TAGS_PORT'}) {
        $url = $row->{'TAGS_PORT'};
    } else {
        $url = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/tag/port/all/tagport_".$tag_id."_1.html";
    }

    $salida->finish;
    return $url;
}

#---------------------------------------------------------------
# genera un listado de tags en formato json.
sub generar_json {
    my $bd = shift;
    my ($sql, $salida, $tag_id, $tag_nom);

    my %tags_url = ();
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} eq '5') {
        my ($salida2, $urlf5_subtypeid, $urlf5_file, $urlf5_url);
        my $sql2 = "SELECT URLF5_SUBTYPEID, URLF5_FILE, URLF5_URL FROM URLF5 WHERE URLF5_TYPE = ?";
        my @params = ('tag');
        $salida2 = &glib_dbi_03::ejecutar_sql_bind_param($bd, $sql2, \@params, \($urlf5_subtypeid, $urlf5_file, $urlf5_url));

        while ($salida2->fetch) {
            $urlf5_file =~ s/\/$prontus_varglb::PRONTUS_ID//;
            $tags_url{$urlf5_subtypeid}{$urlf5_file} = $urlf5_url;
        }
        $salida2->finish;
    }

    $sql = "SELECT TAGS_ID, TAGS_TAG FROM TAGS WHERE TAGS_MOSTRAR = 1";
    $salida = &glib_dbi_03::ejecutar_sql_bind($bd, $sql, \($tag_id, $tag_nom));
    my @tags;
    while ($salida->fetch) {
        &glib_str_03::correct_encode_utf8_string($tag_nom);
        my %urls = ();
        if (defined($tags_url{$tag_id})) {
            %urls = %{$tags_url{$tag_id}};
        }
        push @tags, {'id' => $tag_id, 'nom' => $tag_nom, 'urls' => \%urls};
    }

    $salida->finish;

    my $str_json = encode_json(\@tags);
    utf8::decode($str_json);

    my $dst_path = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}$prontus_varglb::DIR_DATA4FIDS";

    &glib_fildir_03::check_dir($dst_path);
    &glib_fildir_03::write_file("$dst_path/tags.json", $str_json);
}

# ---------------------------------------------------------------
# subrutinas de generación de relacionados
# ---------------------------------------------------------------

# Genera un archivo con los relacionados por tags.
sub generar_relacionados_tags {
    my ($tag_list_str) = $_[0]; #un listado de tag ids separado por comas.
    my ($dst_dir) = $_[1];
    my ($ts) = $_[2];
    my ($base) = $_[3];
    my ($mv) = $_[4];

    # borra extension, en caso de que venga con ella
    $ts =~ s/\..+$//;

    my $ruta_dir = $prontus_varglb::DIR_SERVER . $RELDIR_ARTIC_RELAC;
    $ruta_dir =~ s/\/tags\/pags/\/tags\/pags-$mv/ if ($mv);

    $dst_dir =~ s/(\d{8})\/pags/$1\/pags-$mv/ if ($mv);

    my @lisdir;
    @lisdir = &glib_fildir_03::lee_dir($ruta_dir) if (-d $ruta_dir);

    @lisdir = grep !/^\./, @lisdir; # Elimina directorios . y ..
    foreach my $k (@lisdir) {
        if (-f "$ruta_dir/$k") {
            my $pagina = &glib_fildir_03::read_file("$ruta_dir/$k");

            my $loop = '';
            if ($pagina =~ /%%loop%%(.*?)%%\/loop%%/s) {
                $loop = $1;
            }

            # Generar listas con los articulos especificados manualmente
            my ($lista) = &make_lista_tag($ts, $tag_list_str, $loop, $base, $mv);
            # print STDERR "lista[$lista]\n";

            $pagina =~ s/%%_num_relac%%//g;
            # Parsear y escribir paginas para incluir en articulos.
            &parse_and_write_tag($lista, $pagina, $dst_dir, $ts, $k);
        }
    }
}

# ---------------------------------------------------------------
sub borrar_relacionados_tag {
    my ($dst_dir) = $_[0];
    my ($ts) = $_[1];

    my $pat = $dst_dir . '/' . $ts . '_*.*'; # Borra imagenes
    my $res = unlink glob($pat);
}

# ---------------------------------------------------------------
sub parse_and_write_tag {
    my ($lista, $pagina, $dst_dir, $ts, $nomfile) = @_;
    $pagina =~ s/%%loop%%.*?%%\/loop%%/$lista/sg;

    $pagina =~ s/%%_vermas%%.*?%%\/_vermas%%//sg;

    $pagina = lib_prontus::parse_globals($pagina, '');

    my ($crlf) = qr/\x0a\x0d|\x0d\x0a|\x0a|\x0d/;

    $pagina =~ s/$crlf/\x0a/sg;

    $pagina = lib_prontus::parser_custom_function($pagina);

    # Eliminar comentarios html y marcas prontus sobrantes.
    $pagina =~ s/<!--\w+-->//sg;
    $pagina =~ s/<!--\/\w+-->//sg;
    $pagina =~ s/%%.*?%%//sg;

    # Escribe pagina en el disco.
    &glib_fildir_03::check_dir($dst_dir);
    &glib_fildir_03::write_file("$dst_dir/" . $ts . '_tags_' . $nomfile , $pagina);
}

# ---------------------------------------------------------------
# genera la query que buscará los tags para relacionados y devuelve los resultados.
sub make_lista_tag {
    my ($art_ts, $tag_ids_str, $loop, $base, $mv) = @_;
    my ($art_extension, $id_tag, $art_fechap);

    my $dthr_system = &glib_hrfec_03::get_dtime_pack4();
    $dthr_system =~ /^(\d{8})(\d\d\d\d)/;
    my $hhmm_system = $2;
    my $dt_system = $1;

    my ($filtros, $params_ref) = &genera_filtros_tag($art_ts, $tag_ids_str, $dt_system, $hhmm_system);
    my $sql = "SELECT TAGSART.TAGSART_IDART, TAGSART.TAGSART_IDTAGS, ART.ART_FECHAPHORAP, ART.ART_EXTENSION FROM TAGSART INNER JOIN ART ON TAGSART.TAGSART_IDART=ART.ART_ID %%FILTROS%% group by TAGSART.TAGSART_IDART order by ART_FECHAPHORAP desc LIMIT ?";
    my @params = @{$params_ref};

    $sql =~ s/%%FILTROS%%/WHERE $filtros/;

    my $tot_artics = &get_tot_artics($filtros, $params_ref, $base, $MAX_LIMIT);

    # el limit se agrega despues para que no sobren parametros en la invocacion
    push @params, $MAX_LIMIT;

    my $salida = &glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($art_ts, $id_tag, $art_fechap, $art_extension));
    my $nro_filas = 0;
    my $filas;
    my $loopcounter = 0;

    while ($salida->fetch) {
        $loopcounter++;
        my ($una_fila, $filler) = &generar_fila($RELDIR_ARTIC, $art_ts, $art_extension, $loop, $loopcounter, $tot_artics, $mv);
        $filas .= $una_fila;
        $nro_filas++;
    }

    $salida->finish;

    return $filas;
}

# Genera segmento variable del sql para encontrar los articulos.
sub genera_filtros_tag {
    my ($art_ts, $tag_ids, $dt_system, $hhmm_system) = @_;
    my @params;
    my $filtros = ' 1 = 1 ';
    if ($tag_ids) {
        $filtros .= " AND (";
        my $i = 0;
        foreach my $id_tag (split(/,\s*/ , $tag_ids)){
            if ($i == 0) {
                $filtros = $filtros." TAGSART.TAGSART_IDTAGS = ?";
            } else {
                $filtros = $filtros." OR TAGSART.TAGSART_IDTAGS = ?";
            }
            push @params, $id_tag;
            $i++;
        }
        $filtros .= ")";

        $filtros = $filtros." AND TAGSART.TAGSART_IDART <> ?";
        push @params, $art_ts;

        $filtros .= " and (ART_FECHAPHORAP <= ?) ";
        push @params, "$dt_system$hhmm_system";
        $filtros .= " and (ART_ALTA = '1') ";

        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CONTROL_FECHA'} eq 'SI') {
            $filtros .= " and ( (ART_FECHAEHORAE >= ?) OR ( (ART_FECHAEHORAE < ?) AND (ART_SOLOPORTADAS = '1') ) )";

            push @params, "$dt_system$hhmm_system";
            push @params, "$dt_system$hhmm_system";
        }

        # procesamos solamente los artículos que generan vistas.
        my %fids_hash = &lib_helper::get_fids_with_view();
        my @fids_con_vista;
        foreach my $fid (keys %fids_hash) {
            push @fids_con_vista, "'$fid'";
        }
        my $fids_string = join(',', @fids_con_vista);
        $filtros .= " and ART_TIPOFICHA in ($fids_string) " if $fids_string;
    }
    return ($filtros, \@params);

}

sub get_tot_artics {
    my $filtros = shift;
    my $params_ref = shift;
    my $base = shift;
    my $maxartics = shift;
    my ($sql, $salida, $tot);
    my ($count_art);
    my @params = @{$params_ref};

    $sql = "SELECT COUNT(ART_ID) FROM ART INNER JOIN TAGSART ON ART.ART_ID = TAGSART.TAGSART_IDART %%FILTRO%%";

    if ($filtros ne '') {
        $sql =~ s/%%FILTRO%%/WHERE $filtros/;
    } else {
        $sql =~ s/%%FILTRO%%//;
    }

    $salida = &glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($count_art));
    $salida->fetch;
    $salida->finish;

    $count_art = '0' if $count_art eq '';
    $count_art = $maxartics if ($count_art > $maxartics);

    return $count_art;
}

sub generar_fila {
    # Genera y retorna cada fila de la tabla.
    my ($reldir_artic, $art_ts, $art_extension, $loop, $loopcounter, $tot_artics, $mv, $refhash_campos_xml, $refhash_campos_xdata, $nro_pag) = @_;
    my (%campos_xml, %claves_adicionales);
    my $fila = $loop;

    if ($art_ts ne '') {
        if (!defined($lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$art_ts})) {
            $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$art_ts} = Artic->new(
                'prontus_id'         => $prontus_varglb::PRONTUS_ID,
                'public_server_name' => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'},
                'cpan_server_name'   => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'},
                'document_root'      => $prontus_varglb::DIR_SERVER,
                'ts'                 => $art_ts,
                'no_check_dirs'      => 1) || die "Error inicializando objeto articulo: $Artic::ERR\n";
        }

        if (!ref $refhash_campos_xml) {
            %campos_xml = $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$art_ts}->get_xml_content();
        } else {
            %campos_xml = %$refhash_campos_xml;
        }

        # agrega vista a datos de artículo
        $campos_xml{'_mv'} = $mv;

        if (!ref $refhash_campos_xdata) {
            %claves_adicionales = $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$art_ts}->get_xdata();
        } else {
            %claves_adicionales = %$refhash_campos_xdata;
        }

        # Buscamos las eventuales fixed_urls en db.
        $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$art_ts}->get_fixed_urls(\%claves_adicionales);
        $claves_adicionales{_nropagina} = $nro_pag;
        $claves_adicionales{_ts} = $art_ts;
        $claves_adicionales{_loopcounter} = $loopcounter;
        $claves_adicionales{_totartics} = $tot_artics;

        my $art_dirfecha = &lib_prontus::get_dirfecha_by_ts($art_ts);

        $reldir_artic =~ s/%%dirfecha%%/$art_dirfecha/g;
        $reldir_artic =~ s/%%_fechac%%/$art_dirfecha/g;

        my $fullpath_vista = "$reldir_artic/$art_ts" . '.' . $art_extension;

        $fila = &lib_prontus::procesa_loop_artic($fila,'##');
        # procesar loops e if de multitag
        $fila = $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$art_ts}->parsea_loop_multitag($fila, \%campos_xml, \%claves_adicionales);
        $fila = &lib_prontus::procesa_condicional($fila, \%campos_xml, \%claves_adicionales);
        $fila = &procesar_condicional_extra($fila, $loopcounter);
        $fila = $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$art_ts}->parse_artic_data($fullpath_vista, $fila, \%campos_xml, \%claves_adicionales);
        $fila = &lib_prontus::parser_custom_function($fila);

        # Borra marcas no sustituidas
        $fila =~ s/%%.+?%%//g;
    } else {
        # Armar la fila sin datos.
        $fila = '';
    }

    my $p = \%campos_xml;
    my $x = \%claves_adicionales;

    return ($fila, $p, $x);
}

sub procesar_condicional_extra {
    my ($buffer, $loopcounter) = @_;
    my $localbuf = $buffer;

    while ($buffer =~ /%%ifv\((\d+)\, *(\d+)\)%%.+?%%\/ifv%%/sg) {
        my $div = $1;
        my $res = $2;

        if (!$div) {
            $localbuf =~ s/%%(ifv\($div\, *$res\))%%(.+?)%%\/(ifv)%%/<h1>Error: división por 0<\/h1>/s;

            next;
        }

        my $mod = $loopcounter % $div;

        if ($mod != $res) {
            $localbuf =~ s/%%(ifv\($div\, *$res\))%%(.+?)%%\/(ifv)%%//s;
        }
    }

    while ($buffer =~ /%%nifv\((\d+)\, *(\d+)\)%%.+?%%\/nifv%%/sg) {
        my $div = $1;
        my $res = $2;

        if (!$div) {
            $localbuf =~ s/%%(ifv\($div\, *$res\))%%(.+?)%%\/(ifv)%%/<h1>Error: división por 0<\/h1>/s;

            next;
        }

        my $mod = $loopcounter % $div;

        if ($mod == $res) {
            $localbuf =~ s/%%(nifv\($div\, *$res\))%%(.+?)%%\/(nifv)%%//s;
        }
    }

    return $localbuf;
}

1;
