#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------
package lib_tax;

use strict;
use utf8;
use DBI;
use JSON;
use glib_dbi_03;
use glib_fildir_03;
use glib_hrfec_03;
use lib_helper;
use prontus_varglb;
use lib_urlf5;

require lib_prontus;
require Artic;

use POSIX qw(strftime ceil);

# Limite maximo permitido.
our $MAX_LIMIT = 50;
our $RELDIR_ARTIC = '';

our $RELDIR_ARTIC_RELAC = '';
our $NUM_RELAC_DEFAULT = '';

our $RELDIR_DST_ARTIC_RELAC = '';
our %CFG_FIL_TAXPORT = ();
our %TAX = ();
our $MODTIME = 0;

# ---------------------------------------------------------------
# SUB-RUTINAS.
# ---------------------------------------------------------------
sub set_vars {
    my ($dir_contenido, $dir_artic, $dir_pag, $dir_temp, $dir_taxonomia, $num_relac_default) = @_;
    $RELDIR_ARTIC           = $dir_contenido . $dir_artic . '/%%DIRFECHA%%' . $dir_pag;
    $RELDIR_ARTIC_RELAC     = $dir_temp . $dir_taxonomia;
    $RELDIR_DST_ARTIC_RELAC = $dir_contenido . $dir_taxonomia;
    $NUM_RELAC_DEFAULT      = $num_relac_default;
}

# --------------------------------------------------------------------
# Obtiene seccion, tema, stema del articulo desde la base de datos.
sub get_taxonomia {
    my ($id) = $_[0];
    my ($base) = $_[1];
    my ($nivel) = $_[2];
    my ($secc, $tem, $stem);

    if ($nivel !~ /^\d+$/) {
        $nivel = 1;
    }

    $nivel = 1 if (!$nivel);
    $nivel = 3 if ($nivel > 3);

    my $sql = "select ART_IDSECC$nivel, ART_IDTEMAS$nivel, ART_IDSUBTEMAS$nivel from ART where ART_ID = ?";
    my @params = ($id);
    my $salida = &glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($secc, $tem, $stem));

    $salida->fetch;
    $salida->finish;

    if ($secc) {
        $tem = '0' if ($tem eq '');
        $stem = '0' if ($stem eq '');

        return ($secc, $tem, $stem);
    } else {
        return ('-1','0','0');
    }
}

# Obtiene fixed_urls para seccion, tema, stema del articulo desde la base de datos.
sub get_fixed_urls {
    my ($seccion_id, $tema_id, $subtema_id) = @_;

    &carga_tax_json();

    if ($subtema_id) {
        return ($TAX{$seccion_id}{port}, $TAX{$seccion_id}{temas}{$tema_id}{port}, $TAX{$seccion_id}{temas}{$tema_id}{subtemas}{$subtema_id}{port});
    }

    if ($tema_id) {
        return ($TAX{$seccion_id}{port}, $TAX{$seccion_id}{temas}{$tema_id}{port}, '');
    }

    if ($seccion_id) {
        return ($TAX{$seccion_id}{port}, '', '');
    }

    return ('', '','');
}

# ---------------------------------------------------------------
sub get_tot_artics {
    my $filtros = shift;
    my $params_ref = shift;
    my $base = shift;
    my $maxartics = shift;
    my ($sql, $salida, $tot);
    my ($count_art);
    my @params = @{$params_ref};

    $sql = "select count(ART_ID) from ART %%FILTRO%%";

    if ($filtros ne '') {
        $sql =~ s/%%FILTRO%%/where $filtros/;
    } else {
        $sql =~ s/%%FILTRO%%//;
    }

    $salida = &glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($count_art));
    $salida->fetch;
    $salida->finish;

    $count_art = '0' if $count_art eq '';
    $count_art = $maxartics if ($count_art > $maxartics);

    return $count_art;
}

sub parse_tax_url {
    my $buffer = $_[0];
    my $campos_ref = $_[1];
    my $mv = $_[2];
    my $inc_pagnum = $_[3];
    my %campos = %{$campos_ref};
    my $aux = $buffer;
    my $first_page = 1;
    my $reldir_port_dst = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_TAXPORT";

    $first_page = 0 if ($inc_pagnum eq 0);

    while ($aux =~ /(%%_seccion([1-3])\.url\(['"](\w+)['"],\s*['"]([\w\.]+)['"]\)%%)/isg) {
        my $marca = $1;
        my $id = $2;
        my $type = $3;
        my $plt = $4;
        my $path = "$reldir_port_dst/$type/$plt" . '_'. $campos{"_seccion$id"}. '___1.html';

        if ($campos{"_seccion$id"} eq ''){
            $buffer =~ s/\Q$marca\E//g;
            next;
        }

        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} eq '5') {
            my $lnk_tax = lib_urlf5::get_link_tax($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $path, $first_page);
            if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_INCLUDE_VIEW_NAME'} eq 'SI' && $mv ne '') {
                $lnk_tax = "/$mv$lnk_tax";
            }

            $buffer =~ s/\Q$marca\E/$lnk_tax/g;
        } else {
            $buffer =~ s/\Q$marca\E/$path/g;
        }
    }

    # buscamos marcas de url taxonomicas, tema
    while ($aux =~ /(%%_tema([1-3])\.url\(['"](\w+)['"],\s*['"]([\w\.]+)['"]\)%%)/isg) {
        my $marca = $1;
        my $id = $2;
        my $type = $3;
        my $plt = $4;

        if ($campos{"_tema$id"} eq ''){
            $buffer =~ s/\Q$marca\E//g;
            next;
        }

        my $path = "$reldir_port_dst/$type/$plt" . '_'. $campos{"_seccion$id"}. '_'.$campos{"_tema$id"}.'__1.html';
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} eq '5') {
            my $lnk_tax = lib_urlf5::get_link_tax($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $path, $first_page);
            if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_INCLUDE_VIEW_NAME'} eq 'SI' && $mv ne '') {
                $lnk_tax = "/$mv$lnk_tax";
            }

            $buffer =~ s/\Q$marca\E/$lnk_tax/g;
        } else {
            $buffer =~ s/\Q$marca\E/$path/g;
        }
    }

    # buscamos marcas de url taxonomicas, subtema
    while ($aux =~ /(%%_subtema([1-3])\.url\(['"](\w+)['"],\s*['"]([\w\.]+)['"]\)%%)/isg) {
        my $marca = $1;
        my $id = $2;
        my $type = $3;
        my $plt = $4;

        if ($campos{"_subtema$id"} eq ''){
            $buffer =~ s/\Q$marca\E//g;
            next;
        }

        my $path = "$reldir_port_dst/$type/$plt" . '_'. $campos{"_seccion$id"}. '_'.$campos{"_tema$id"}.'_'.$campos{"_subtema$id"}.'_1.html';
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} eq '5') {
            my $lnk_tax = lib_urlf5::get_link_tax($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $path, $first_page);
            if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_INCLUDE_VIEW_NAME'} eq 'SI' && $mv ne '') {
                $lnk_tax = "/$mv$lnk_tax";
            }

            $buffer =~ s/\Q$marca\E/$lnk_tax/g;
        } else {
            $buffer =~ s/\Q$marca\E/$path/g;
        }
    }

    return $buffer;
}

sub get_tax_if_last_art {
    my $base = $_[0];
    my $secc_id = $_[1];
    my $tema_id = $_[2];
    my $subtema_id = $_[3];
    my $sql;
    my $salida;
    my $count_secc;
    my $count_tems;
    my $count_subtems;
    my @params;

    $sql = "SELECT COUNT(*) FROM ART WHERE ART_IDSECC1 = ? OR ART_IDSECC2 = ? OR ART_IDSECC3 = ?";

    @params = ($secc_id, $secc_id, $secc_id);

    $salida = &glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($count_secc));
    $salida->fetch;
    $salida->finish;

    if ($tema_id) {
        $sql = "SELECT COUNT(*) FROM ART WHERE ART_IDTEMAS1 = ? OR ART_IDTEMAS2 = ? OR ART_IDTEMAS2 = ?";

        @params = ($tema_id, $tema_id, $tema_id);

        $salida = &glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($count_tems));
        $salida->fetch;
        $salida->finish;
    }

    if ($subtema_id) {
        $sql = "SELECT COUNT(*) FROM ART WHERE ART_IDSUBTEMAS1 = ? OR ART_IDSUBTEMAS2 = ? OR ART_IDSUBTEMAS3 = ?";

        @params = ($subtema_id, $subtema_id, $subtema_id);

        $salida = &glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($count_subtems));
        $salida->fetch;
        $salida->finish;
    }

    if ($secc_id && !$tema_id && !$subtema_id) {
        return 1 if ($count_secc == 1);
    } elsif ($secc_id && $tema_id && !$subtema_id) {
        return 1 if ($tema_id == 1);
    } elsif ($secc_id && $tema_id && $subtema_id) {
        return 1 if ($subtema_id == 1);
    } else {
        return 0;
    }

    return 0;
};

# ---------------------------------------------------------------
sub generar_relacionados {
    my ($id_secc1, $id_tema1, $id_subtema1, $base, $mv) = @_;

    # corregimos para que no genere archivos "[secc]_0_0_art_relac.html", sino "[secc]___art_relac.html".
    if ($id_secc1 eq '0') {
        $id_secc1 = '';
    }

    if ($id_tema1 eq '0') {
        $id_tema1 = '';
    }

    if ($id_subtema1 eq '0') {
        $id_subtema1 = '';
    }

    # Procesa Plantillas que hayan en el dir.
    my ($plantilla, $pagina, $loop, $lista);
    my $ruta_dir = $prontus_varglb::DIR_SERVER . $RELDIR_ARTIC_RELAC;
    $ruta_dir =~ s/\/taxonomia\/pags/\/taxonomia\/pags-$mv/ if ($mv);

    my @lisdir;
    @lisdir = &glib_fildir_03::lee_dir($ruta_dir) if (-d $ruta_dir);
    my $limit = $NUM_RELAC_DEFAULT;

    foreach my $k (@lisdir) {
        if (-f "$ruta_dir/$k") {
            # print STDERR "DIR[$ruta_dir/$k]\n";
            $plantilla = "$ruta_dir/$k";
            $pagina = &glib_fildir_03::read_file($plantilla);

            if ($pagina =~ /%%_NUM_RELAC\s*=\s*(\d+)%%/is) {
                $limit = $1;
            }

            # %%_TAX_LEVEL=seccion-tema-subtema%%
            my $taxlevel;
            if ($pagina =~ /%%_TAX_LEVEL=(seccion|seccion-tema|seccion-tema-subtema)%%/i) {
                $taxlevel = $1;
            } else {
                $taxlevel = 'seccion-tema-subtema'; # lo por defecto
            }

            $pagina =~ s/%%_TAX_LEVEL\s*=[^%]*%%//ig; # borra marca
            $pagina =~ s/%%_NUM_RELAC\s*=[^%]*%%//ig; # borra marca

            my ($exclude_port, $exclude_port_area, $fids);

            if ($pagina =~ /%%_EXCLUDE_PORT\s*=([^%]+)%%/is) {
                $exclude_port = $1;

                $exclude_port = $prontus_varglb::DIR_SERVER . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'} . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_EDIC'} . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_BASEEDIC'} . "/xml/$exclude_port";

                if (!-f $exclude_port && $exclude_port =~ /\.xml$/) {
                    $exclude_port = '';
                }
            }

            $pagina =~ s/%%_EXCLUDE_PORT\s*=[^%]*%%//ig; # borra marca

            if ($pagina =~ /%%_EXCLUDE_PORT_AREA=([\d,]+)%%/is) {
                $exclude_port_area = $1;
                $exclude_port_area =~ s/,{2,}/,/g;
                $exclude_port_area =~ s/^,+//g;
                $exclude_port_area =~ s/,+$//g;

                #print STDERR "exclude_port_area[$exclude_port_area]\n";
            }

            $pagina =~ s/%%_EXCLUDE_PORT_AREA=.*?%%//ig; # borra marca

            if ($pagina =~ /%%_FIDS=([^%]+)%%/is) {
                $fids = $1;
                $fids =~ s/,{2,}/,/g;
                $fids =~ s/^,+//g;
                $fids =~ s/,+$//g;

                #print STDERR "fids[$fids]\n";
            }

            $pagina =~ s/%%_FIDS=.*?%%//ig; # borra marca

            if ($pagina =~ /%%LOOP%%(.*?)%%\/LOOP%%/is) {
                $loop = $1;
            }

            $limit++;

            if ($limit > $MAX_LIMIT) {
                $limit = $MAX_LIMIT;
            }

            my $nom_tabla_exclude;

            if ($exclude_port) {
                $nom_tabla_exclude = &lib_prontus::set_exclude_port_table($exclude_port, $exclude_port_area, $base);
            }

            # Generar lista.
            my $hay_mas = 0;

            ($lista, $hay_mas) = &make_lista($id_secc1, $id_tema1, $id_subtema1, $loop, $limit, $base, $mv, $taxlevel, $fids, $nom_tabla_exclude);
            $limit--;
            $pagina =~ s/%%_NUM_RELAC%%/$limit/isg;

            # print STDERR "lista[$lista]\n";
            &parse_and_write($id_secc1, $id_tema1, $id_subtema1, $lista, $pagina, $loop, $k, $hay_mas, $mv);
        }
    }
}

# ---------------------------------------------------------------
sub make_lista {
    # Construye listas de articulos relacionados, para ser considerados, deben tener seccion a lo menos.
    my ($id_secc1, $id_tema1, $id_subtema1, $loop, $limit, $base, $mv, $taxlevel, $fids, $nom_tabla_exclude) = @_;
    my ($art_id, $art_extension);

    my $dthr_system = &glib_hrfec_03::get_dtime_pack4();
    my $hhmm_system = substr($dthr_system, 8, 4);
    my $dt_system = substr($dthr_system, 0, 8);

    my ($filtros, $params_ref) = &genera_filtros($id_secc1, $id_tema1, $id_subtema1, $dt_system, $hhmm_system, $taxlevel);
    my $sql = "select ART_ID, ART_EXTENSION from ART %%FILTRO%% order by ART_FECHAPHORAP desc LIMIT ?";
    my @params = @{$params_ref};

    # print STDERR "filtros[$filtros]\n";
    if ($nom_tabla_exclude) {
        if ($filtros ne '') {
            $filtros .= " AND";
        }
        $filtros .= " ART_ID NOT IN (SELECT EXCLUDE_ART_ID FROM $nom_tabla_exclude)";
    }

    if ($fids) {
        my @arrfids = split /,/, $fids;

        my @tipo;
        foreach my $f (@arrfids) {
            push(@tipo, "ART_TIPOFICHA = ?");
            push(@params, $f);
        }

        if ($filtros ne '') {
            $filtros .= " AND";
        }
        $filtros .= " (".join(' or ', @tipo).") ";
    }

    if ($filtros ne '') {
        $sql =~ s/%%FILTRO%%/ WHERE $filtros /;
    } else {
        return '';
    }

    my $tot_artics = &get_tot_artics($filtros, \@params, $base, ($limit-1));
    # el limit se agrega despues para que no sobren parametros en la invocacion
    push @params, $limit;
    my $salida = &glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($art_id, $art_extension));
    my $nro_filas = 1;
    my $filas;
    my $hay_mas = 0;
    my $loopcounter = 0;

    while ($salida->fetch) {
        if ($nro_filas >= $limit) {
            $hay_mas = 1;
            last;
        }

        $loopcounter++;
        my ($una_fila, $filler, $filler2) = &generar_fila($RELDIR_ARTIC, $art_id, $art_extension, $loop, $loopcounter, $tot_artics, $mv);

        $filas .= $una_fila;
        $nro_filas++;
    }

    $salida->finish;

    return ($filas, $hay_mas);
}

# ---------------------------------------------------------------
sub generar_fila {
    # Genera y retorna cada fila de la tabla.
    my ($reldir_artic, $art_ts, $art_extension, $loop, $loopcounter, $tot_artics, $mv, $refhash_campos_xml, $refhash_campos_xdata, $nro_pag) = @_;
    my (%campos_xml, %claves_adicionales);
    my $fila = $loop;

    if ($art_ts ne '') {
        if (!defined($lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$art_ts})) {
            $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$art_ts} = Artic->new(
                'prontus_id'         => $prontus_varglb::PRONTUS_ID,
                'public_server_name' => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'},
                'cpan_server_name'   => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'},
                'document_root'      => $prontus_varglb::DIR_SERVER,
                'ts'                 => $art_ts,
                'no_check_dirs'      => 1) || die "Error inicializando objeto articulo: $Artic::ERR\n";
        }

        if (!ref $refhash_campos_xml) {
            %campos_xml = $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$art_ts}->get_xml_content();
        } else {
            %campos_xml = %$refhash_campos_xml;
        }

        # agrega vista a datos de artículo
        $campos_xml{'_mv'} = $mv;

        if (!ref $refhash_campos_xdata) {
            %claves_adicionales = $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$art_ts}->get_xdata();
        } else {
            %claves_adicionales = %$refhash_campos_xdata;
        }

        # Buscamos las eventuales fixed_urls en db.
        $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$art_ts}->get_fixed_urls(\%claves_adicionales);

        $claves_adicionales{_nropagina} = $nro_pag;
        $claves_adicionales{_ts} = $art_ts;
        $claves_adicionales{_loopcounter} = $loopcounter;
        $claves_adicionales{_totartics} = $tot_artics;

        my $art_dirfecha = &lib_prontus::get_dirfecha_by_ts($art_ts);

        $reldir_artic =~ s/%%DIRFECHA%%/$art_dirfecha/ig;
        $reldir_artic =~ s/%%_FECHAC%%/$art_dirfecha/ig;

        # Path completo al articulo a generar
        my $fullpath_vista = $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$art_ts}->get_fullpath_artic($mv, "$art_ts.$art_extension");

        $fila = &lib_prontus::procesa_loop_artic($fila,'##');
        # procesar loops e if de multitag
        $fila = $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$art_ts}->parsea_loop_multitag($fila, \%campos_xml, \%claves_adicionales);
        $fila = &lib_prontus::procesa_condicional($fila, \%campos_xml, \%claves_adicionales);
        $fila = &procesar_condicional_extra($fila, $loopcounter);
        $fila = $lib_prontus::ARTICS{$prontus_varglb::PRONTUS_ID}{$art_ts}->parse_artic_data($fullpath_vista, $fila, \%campos_xml, \%claves_adicionales);
        $fila = &lib_prontus::parser_custom_function($fila);

        # Borra marcas no sustituidas
        $fila =~ s/%%.+?%%//g;
    } else {
        # Armar la fila sin datos.
        $fila = '';
    }

    my $p = \%campos_xml;
    my $x = \%claves_adicionales;

    return ($fila, $p, $x);
}

# ---------------------------------------------------------------
sub parse_and_write {
    my ($id_secc1, $id_tema1, $id_subtema1, $lista, $pagina, $loop, $nomfile, $hay_mas, $mv) = @_;

    if ($lista eq '')  {
        $pagina =~ s/%%_VERMAS%%.*?%%\/_VERMAS%%//isg;
        $lista = 'Sin art&iacute;culos relacionados.';
    }

    if (!$hay_mas) {
        $pagina =~ s/%%_VERMAS%%.*?%%\/_VERMAS%%//isg;
    }

    $pagina =~ s/%%LOOP%%.*?%%\/LOOP%%/$lista/isg;
    $pagina =~ s/%%_SECCION1%%/$id_secc1/ig;
    $pagina =~ s/%%_TEMA1%%/$id_tema1/ig;
    $pagina =~ s/%%_SUBTEMA1%%/$id_subtema1/ig;

    $pagina = lib_prontus::parse_globals($pagina, '');

    if ($id_secc1 eq '-1') {
        $id_secc1 = '';
        $lista = '';
    }

    my ($crlf) = qr/\x0a\x0d|\x0d\x0a|\x0a|\x0d/;

    $pagina =~ s/$crlf/\x0a/sg;

    $pagina = &lib_prontus::parser_custom_function($pagina);

    # Eliminar comentarios html sobrantes.
    $pagina =~ s/<!--\w+-->//sg;
    $pagina =~ s/<!--\/\w+-->//sg;
    $pagina =~ s/%%.*?%%//sg;

    # Escribe pagina en el disco.
    my $reldir_dst_artic_relac = $RELDIR_DST_ARTIC_RELAC;

    if ($mv) {
        $reldir_dst_artic_relac =~ s/\/taxonomia\/pags/\/taxonomia\/pags-$mv/;

        &glib_fildir_03::check_dir("$prontus_varglb::DIR_SERVER$reldir_dst_artic_relac");
    }

    my $dst2write = "$prontus_varglb::DIR_SERVER$reldir_dst_artic_relac/" . $id_secc1 . '_' . $id_tema1 . '_' . $id_subtema1 . '_' . $nomfile;

    &glib_fildir_03::write_file($dst2write, $pagina);
    &lib_prontus::purge_cache($dst2write) if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_ART_RELAC'} eq 'SI');
}

# ---------------------------------------------------------------
sub genera_filtros {
    # Genera segmento variable del sql para encontrar los articulos relacionados.
    my ($id_secc1, $id_tema1, $id_subtema1, $dt_system, $hhmm_system, $taxlevel) = @_;
    # taxlevel: seccion | seccion-tema | seccion-tema-subtema, siempre al menos es "seccion"
    my @params;
    my $filtros = '1 = 1 AND ';
    if ($id_secc1) {
        $filtros .= "(ART_IDSECC1 = ? or ART_IDSECC2 = ? or ART_IDSECC3 = ?)";

        push @params, $id_secc1;
        push @params, $id_secc1;
        push @params, $id_secc1;

        if (($id_tema1) && (lc $taxlevel ne 'seccion')) {
            $filtros .= " and (ART_IDTEMAS1 = ? or ART_IDTEMAS2 = ? or ART_IDTEMAS3 = ?)";

            push @params, $id_tema1;
            push @params, $id_tema1;
            push @params, $id_tema1;

            if (($id_subtema1) && (lc $taxlevel ne 'seccion') && (lc $taxlevel ne 'seccion-tema')) {
                $filtros .= " and (ART_IDSUBTEMAS1 = ? or ART_IDSUBTEMAS2 = ? or ART_IDSUBTEMAS3 = ?)" if $filtros ne '';

                push @params, $id_subtema1;
                push @params, $id_subtema1;
                push @params, $id_subtema1;
            }
        }

        push @params, "$dt_system$hhmm_system";

        $filtros .= " and (ART_FECHAPHORAP <= ?) ";
        $filtros .= " and (ART_ALTA = '1') ";

        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CONTROL_FECHA'} eq 'SI') {
            $filtros .= " and ( (ART_FECHAEHORAE >= ?) OR ( (ART_FECHAEHORAE < ?) AND (ART_SOLOPORTADAS = '1') ) )";

            push @params, "$dt_system$hhmm_system";
            push @params, "$dt_system$hhmm_system";
        }

        # procesamos solamente los artículos que generan vistas.
        my %fids_hash = &lib_helper::get_fids_with_view();
        my @fids_con_vista;
        foreach my $fid (keys %fids_hash) {
            push @fids_con_vista, "'$fid'";
        }
        my $fids_string = join(',', @fids_con_vista);
        $filtros .= " and ART_TIPOFICHA in ($fids_string) " if $fids_string;

        return ($filtros, \@params);
    } else {
        return ('', \@params);
    }

}

# ---------------------------------------------------------------
sub generar_relacionados_manualtax {
    my ($tax) = $_[0];
    my ($dst_dir) = $_[1];
    my ($ts) = $_[2];
    my ($base) = $_[3];
    my ($mv) = $_[4];

    # borra extension, en caso de que venga con ella
    $ts =~ s/\..+$//;

    my $ruta_dir = $prontus_varglb::DIR_SERVER . $RELDIR_ARTIC_RELAC;
    $ruta_dir =~ s/\/taxonomia\/pags/\/taxonomia\/pags-$mv/ if ($mv);

    $dst_dir =~ s/(\d{8})\/pags/$1\/pags-$mv/ if ($mv);

    my @lisdir;
    @lisdir = &glib_fildir_03::lee_dir($ruta_dir) if (-d $ruta_dir);

    @lisdir = grep !/^\./, @lisdir; # Elimina directorios . y ..
    foreach my $k (@lisdir) {
        if (-f "$ruta_dir/$k") {
            my $pagina = &glib_fildir_03::read_file("$ruta_dir/$k");

            my $loop = '';
            if ($pagina =~ /%%LOOP%%(.*?)%%\/LOOP%%/is) {
                $loop = $1;
            }

            # Geenerar listas con los articulos especificados manualmente
            my ($lista) = &make_lista_manualtax($tax, $loop, $base, $mv);
            # print STDERR "lista[$lista]\n";

            $pagina =~ s/%%_NUM_RELAC%%//ig; # no aplica cuando hay manual tax.
            # Parsear y escribir paginas para incluir en articulos.
            &parse_and_write_manualtax($lista, $pagina, $dst_dir, $ts, $k);
        }
    }
}

# ---------------------------------------------------------------
sub borrar_relacionados_manualtax {
    my ($dst_dir) = $_[0];
    my ($ts) = $_[1];

    my $pat = $dst_dir . '/' . $ts . '_*.*'; # Borra imagenes
    my $res = unlink glob($pat);
}

# ---------------------------------------------------------------
sub parse_and_write_manualtax {
    my ($lista, $pagina, $dst_dir, $ts, $nomfile) = @_;
    # print STDERR "[$ts, $nomfile]\n";
    $pagina =~ s/%%LOOP%%.*?%%\/LOOP%%/$lista/isg;

    $pagina =~ s/%%_VERMAS%%.*?%%\/_VERMAS%%//isg;

    $pagina = lib_prontus::parse_globals($pagina, '');

    my ($crlf) = qr/\x0a\x0d|\x0d\x0a|\x0a|\x0d/;

    $pagina =~ s/$crlf/\x0a/sg;

    $pagina = lib_prontus::parser_custom_function($pagina);

    # Eliminar comentarios html sobrantes.
    $pagina =~ s/<!--\w+-->//sg;
    $pagina =~ s/<!--\/\w+-->//sg;
    $pagina =~ s/%%.*?%%//sg;

    # Escribe pagina en el disco.
    &glib_fildir_03::check_dir($dst_dir);
    &glib_fildir_03::write_file("$dst_dir/" . $ts . '_' . $nomfile , $pagina);
}

# ---------------------------------------------------------------
sub genera_filtros_manualtax {
    # Genera segmento variable del sql para encontrar los articulos.
    my ($tax, $dt_system, $hhmm_system) = @_;
    $tax =~ s/[^0-9\,]//sg;

    my @ids = split(/\,/, $tax); # 10,20,35,112 o yyyymmddhhiiss
    my @params;

    my $filtros = '(ART_ID = 0';
    foreach my $id (@ids) {
        if ($id !~ /^\d+$/) {
            next;
        }

        if ($id =~ /^\d{14}$/) { # Es un TS.
            $filtros .= " or ART_ID = ? ";
        } else { # Es un ID autoincremental.
            $filtros .= " or ART_AUTOINC = ? ";
        }

        push @params, $id;
    }
    $filtros .= ')';

    return ($filtros, \@params);
}

# ---------------------------------------------------------------
sub make_lista_manualtax {
    my ($tax, $loop, $base, $mv) = @_;
    my ($art_ts, $art_extension);

    my $dthr_system = &glib_hrfec_03::get_dtime_pack4();
    $dthr_system =~ /^(\d{8})(\d\d\d\d)/;
    my $hhmm_system = $2;
    my $dt_system = $1;

    my ($filtros, $params_ref) = &genera_filtros_manualtax($tax, $dt_system, $hhmm_system);
    my $sql = "select ART_ID, ART_EXTENSION from ART %%FILTRO%% order by ART_FECHAPHORAP desc LIMIT ?";
    my @params = @{$params_ref};

    $sql =~ s/%%FILTRO%%/where $filtros/;

    my $tot_artics = &get_tot_artics($filtros, $params_ref, $base, $MAX_LIMIT);

    # el limit se agrega despues para que no sobren parametros en la invocacion
    push @params, $MAX_LIMIT;
    my $salida = &glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($art_ts, $art_extension));
    my $nro_filas = 0;
    my $filas;
    my $loopcounter = 0;

    while ($salida->fetch) {
        $loopcounter++;
        my ($una_fila, $filler) = &generar_fila($RELDIR_ARTIC, $art_ts, $art_extension, $loop, $loopcounter, $tot_artics, $mv);

        $filas .= $una_fila;
        $nro_filas++;
    }

    $salida->finish;

    return $filas;
}

# ---------------------------------------------------------------
sub carga_tabla_temas {
    my ($base, $id) = @_;
    my ($sql, $salida);
    my %tabla_tem;

    $sql = "select TEMAS_ID, TEMAS_NOM, TEMAS_PORT, TEMAS_IDSECC, TEMAS_NOM4VISTAS from TEMAS ";
    my @params = ();
    if ($id) {
        $sql .= " where TEMAS_ID = ?";
        @params = ($id);
    }
    $salida = &glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);

    while (my $row = $salida->fetchrow_hashref) {
        &glib_str_03::correct_encode_utf8($row);
        $tabla_tem{$row->{'TEMAS_ID'}} = "$row->{'TEMAS_NOM'}\t\t$row->{'TEMAS_PORT'}\t\t$row->{'TEMAS_IDSECC'}\t\t$row->{'TEMAS_NOM4VISTAS'}";
    }

    $salida->finish;

    return %tabla_tem;
}

# ---------------------------------------------------------------
sub carga_tabla_subtemas {
    my ($base, $id) = @_;
    my ($sql, $salida);
    my %tabla_stem;

    $sql = "select SUBTEMAS_ID, SUBTEMAS_NOM, SUBTEMAS_PORT, SUBTEMAS_IDTEMAS, SUBTEMAS_NOM4VISTAS from SUBTEMAS ";
    my @params = ();
    if ($id) {
        $sql .= " where SUBTEMAS_ID = ?";
        @params = ($id);
    }
    $salida = &glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);

    while (my $row = $salida->fetchrow_hashref) {
        &glib_str_03::correct_encode_utf8($row);
        $tabla_stem{$row->{'SUBTEMAS_ID'}} = "$row->{'SUBTEMAS_NOM'}\t\t$row->{'SUBTEMAS_PORT'}\t\t$row->{'SUBTEMAS_IDTEMAS'}\t\t$row->{'SUBTEMAS_NOM4VISTAS'}";
    }

    $salida->finish;

    return %tabla_stem;
}

# ---------------------------------------------------------------
sub carga_tabla_seccion {
    my ($base, $id) = @_;
    my ($sql, $salida);
    my %tabla_secc;

    $sql = "select SECC_ID, SECC_NOM, SECC_PORT, SECC_NOM4VISTAS from SECC ";
    my @params = ();
    if ($id) {
        $sql .= " where SECC_ID = ?";
        @params = ($id);
    }
    $salida = &glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);

    while (my $row = $salida->fetchrow_hashref) {
        &glib_str_03::correct_encode_utf8($row);
        $tabla_secc{$row->{'SECC_ID'}} = "$row->{'SECC_NOM'}\t\t$row->{'SECC_PORT'}\t\t$row->{'SECC_NOM4VISTAS'}";
    }

    $salida->finish;

    return %tabla_secc;
}

# ---------------------------------------------------------------
sub carga_hash_seccion {
    my $base = shift;
    my $only_enabled = shift;
    my ($sql, $salida, %hash_datos);

    $sql = "select SECC_ID, SECC_NOM, SECC_PORT, SECC_NOM4VISTAS, SECC_MOSTRAR from SECC ";

    if ($only_enabled) {
        $sql .= "where SECC_MOSTRAR = '1'";
    }

    $salida = &glib_dbi_03::ejecutar_sql($base, $sql);

    while (my $row = $salida->fetchrow_hashref) {
        &glib_str_03::correct_encode_utf8($row);
        my @vistas = split(/\n/, $row->{'SECC_NOM4VISTAS'});
        my %hash_temp = ();
        foreach my $vista (@vistas) {
            my @temp = split(/\t/, $vista);
            $hash_temp{$temp[0]} = $temp[1];
        }
        $hash_datos{$row->{'SECC_ID'}}{'nombre'} = $row->{'SECC_NOM'};
        $hash_datos{$row->{'SECC_ID'}}{'port'} = $row->{'SECC_PORT'};
        $hash_datos{$row->{'SECC_ID'}}{'mostrar'} = $row->{'SECC_MOSTRAR'};
        $hash_datos{$row->{'SECC_ID'}}{'vistas'} = \%hash_temp;
    }

    $salida->finish;

    return %hash_datos;
}

# ---------------------------------------------------------------
sub carga_hash_temas {
    my $base = shift;
    my $only_enabled = shift;
    my $id_padre = shift;
    my ($sql, $salida, %hash_datos, @filtros);

    $sql = "select TEMAS_ID, TEMAS_NOM, TEMAS_PORT, TEMAS_IDSECC, TEMAS_NOM4VISTAS, TEMAS_MOSTRAR from TEMAS ";

    my @params;

    if ($id_padre) {
        push @filtros, "TEMAS_IDSECC = ?";
        push @params, $id_padre;
    }

    if ($only_enabled) {
        push @filtros, "TEMAS_MOSTRAR = ?";
        push @params, $only_enabled;
    }

    if (scalar(@filtros) > 0) {
        $sql .= 'where ' . join ' and ', @filtros;
    }

    $salida = &glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);

    while (my $row = $salida->fetchrow_hashref) {
        &glib_str_03::correct_encode_utf8($row);

        my @vistas = split(/\n/, $row->{'TEMAS_NOM4VISTAS'});
        my %hash_temp = ();

        foreach my $vista (@vistas) {
            my @temp = split(/\t/, $vista);

            $hash_temp{$temp[0]} = $temp[1];
        }

        $hash_datos{$row->{'TEMAS_ID'}}{'id_padre'} = $row->{'TEMAS_IDSECC'} if (!$id_padre);
        $hash_datos{$row->{'TEMAS_ID'}}{'nombre'} = $row->{'TEMAS_NOM'};
        $hash_datos{$row->{'TEMAS_ID'}}{'port'} = $row->{'TEMAS_PORT'};
        $hash_datos{$row->{'TEMAS_ID'}}{'mostrar'} = $row->{'TEMAS_MOSTRAR'};
        $hash_datos{$row->{'TEMAS_ID'}}{'vistas'} = \%hash_temp;
    }

    $salida->finish;

    return %hash_datos;
}

# ---------------------------------------------------------------
sub carga_hash_subtemas {
    my $base = shift;
    my $only_enabled = shift;
    my $id_padre = shift;
    my ($sql, $salida, %hash_datos, @filtros, @params);

    $sql = "select SUBTEMAS_ID, SUBTEMAS_NOM, SUBTEMAS_PORT, SUBTEMAS_IDTEMAS, SUBTEMAS_NOM4VISTAS, SUBTEMAS_MOSTRAR from SUBTEMAS ";

    if ($id_padre) {
        push @params, $id_padre;
        push @filtros, "SUBTEMAS_IDTEMAS = ?";
    }

    if ($only_enabled) {
        $only_enabled = 1;

        push @params, $only_enabled;
        push @filtros, "SUBTEMAS_MOSTRAR = ?";
    }

    if (scalar(@filtros) > 0) {
        $sql .= 'where ' . join ' and ', @filtros;
    }

    $salida = &glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);

    while (my $row = $salida->fetchrow_hashref) {
        &glib_str_03::correct_encode_utf8($row);

        my @vistas = split(/\n/, $row->{'SUBTEMAS_NOM4VISTAS'});
        my %hash_temp = ();

        foreach my $vista (@vistas) {
            my @temp = split(/\t/, $vista);

            $hash_temp{$temp[0]} = $temp[1];
        }

        $hash_datos{$row->{'SUBTEMAS_ID'}}{'id_padre'} = $row->{'SUBTEMAS_IDTEMAS'} if (!$id_padre);
        $hash_datos{$row->{'SUBTEMAS_ID'}}{'nombre'} = $row->{'SUBTEMAS_NOM'};
        $hash_datos{$row->{'SUBTEMAS_ID'}}{'port'} = $row->{'SUBTEMAS_PORT'};
        $hash_datos{$row->{'SUBTEMAS_ID'}}{'mostrar'} = $row->{'SUBTEMAS_MOSTRAR'};
        $hash_datos{$row->{'SUBTEMAS_ID'}}{'vistas'} = \%hash_temp;
    }

    $salida->finish;

    return %hash_datos;
}

# ---------------------------------------------------------------
sub procesar_condicional_extra {
    my ($buffer, $loopcounter) = @_;
    my $localbuf = $buffer;

    while ($buffer =~ /%%IFV\((\d+)\, *(\d+)\)%%.+?%%\/IFV%%/isg) {
        my $div = $1;
        my $res = $2;

        if (!$div) {
            $localbuf =~ s/%%(IFV\($div\, *$res\))%%(.+?)%%\/(IFV)%%/<h1>Error: división por 0<\/h1>/is;

            next;
        }

        my $mod = $loopcounter % $div;

        if ($mod != $res) {
            $localbuf =~ s/%%(IFV\($div\, *$res\))%%(.+?)%%\/(IFV)%%//is;
        }
    }

    while ($buffer =~ /%%NIFV\((\d+)\, *(\d+)\)%%.+?%%\/NIFV%%/isg) {
        my $div = $1;
        my $res = $2;

        if (!$div) {
            $localbuf =~ s/%%(IFV\($div\, *$res\))%%(.+?)%%\/(IFV)%%/<h1>Error: división por 0<\/h1>/is;

            next;
        }

        my $mod = $loopcounter % $div;

        if ($mod == $res) {
            $localbuf =~ s/%%(NIFV\($div\, *$res\))%%(.+?)%%\/(NIFV)%%//is;
        }
    }

    return $localbuf;
}

# ---------------------------------------------------------------
# carga configuracion propia del servidor si esta disponible
# sobreescribe los valores definidos en el cpan de prontus
sub carga_configuracion_local {
    my $buffer = &glib_fildir_03::read_file("$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_taxport_conf.cfg");
    if ($buffer ne '') {
        if ($buffer =~ m/\s*TAXPORT_MAX_WORKERS\s*=\s*["']([0-9]+)["']/) {
            $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXPORT_MAX_WORKERS'} = $1;
        }
        if ($buffer =~ m/\s*TAXPORT_MAX_MASTERS\s*=\s*["']([0-9]+)["']/) {
            $prontus_varglb::TAXPORT_MAX_MASTERS = $1;
        }
    }
}
# ---------------------------------------------------------------
# genera los filtros para los listados de taxport
sub genera_filtros_taxports {
    my ($id_secc1, $id_tema1, $id_subtema1, $fid, $curr_dtime) = @_;
    my $fid_fil = $fid;

    if ($fid =~ /^fil_/) {
        $fid = '';
    }

    $id_secc1 =~ s/"/""/g;
    $id_tema1 =~ s/"/""/g;
    $id_subtema1 =~ s/"/""/g;

    my $dthhmm_system = substr($curr_dtime, 0, 12);
    my ($filtros, @params);

    if ($id_secc1) {
        $filtros = "(";
        $filtros .= "ART_IDSECC1 = ?";

        push @params, $id_secc1;

        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} =~ /^(2|3)$/) {
            $filtros .= " or ART_IDSECC2 = ?";
            push @params, $id_secc1;
        }

        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} eq '3') {
            $filtros .= " or ART_IDSECC3 = ?";
            push @params, $id_secc1;
        }

        $filtros .= ")";

        if ($id_tema1) { # Distinto de todos.
            if ($filtros ne '') {
                $filtros .= "and (";
                $filtros .= "ART_IDTEMAS1 = ?";
                push @params, $id_tema1;

                if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} =~ /^(2|3)$/) {
                    $filtros .= " or ART_IDTEMAS2 = ?";
                    push @params, $id_tema1;
                }

                if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} eq '3') {
                    $filtros .= " or ART_IDTEMAS3 = ?";
                    push @params, $id_tema1;
                }

                $filtros .= ")";
            }

            if ($id_subtema1) { # Distinto de todos.
                if ($filtros ne '') {
                    $filtros .= "and (";
                    $filtros .= "ART_IDSUBTEMAS1 = ?";

                    push @params, $id_subtema1;

                    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} =~ /^(2|3)$/) {
                        $filtros .= " or ART_IDSUBTEMAS2 = ?";
                        push @params, $id_subtema1;

                    }

                    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} eq '3') {
                        $filtros .= " or ART_IDSUBTEMAS3 = ?";
                        push @params, $id_subtema1;
                    }

                    $filtros .= ")";
                }
            }
        }
    } else {
        $filtros = "(";
        $filtros .= "ART_IDSECC1 <> \"\"";
        $filtros .= " or ART_IDSECC2 <> \"\"" if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} =~ /^(2|3)$/);
        $filtros .= " or ART_IDSECC3 <> \"\"" if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'TAXONOMIA_NIVELES'} eq '3');
        $filtros .= ")";
    }

    if ($fid) {
        $filtros .= " and " if ($filtros);
        $filtros .= " (ART_TIPOFICHA = ?) ";
        push @params, $fid;
    }

    if ($fid_fil && defined $CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fid_fil}{'FIDS'}) {
        my @fidlist = @{$CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fid_fil}{'FIDS'}};
        my $filtro_fids;

        if (scalar @fidlist) {
            foreach my $filfid (@fidlist) {
                $filtro_fids .= "ART_TIPOFICHA = ? OR ";
                push @params, $filfid;
            }

            $filtro_fids = substr($filtro_fids, 0, (length($filtro_fids)-3));

            $filtros .= " and " if ($filtros);
            $filtros .= "($filtro_fids)";
        }
    }

    $filtros .= " and " if ($filtros);

    if ($fid_fil && defined $CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fid_fil}{'FECHA_DESDE'} && $CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fid_fil}{'FECHA_DESDE'} ne '') {
        $filtros .= " (ART_FECHAP >= ?) ";
        push @params, $CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fid_fil}{'FECHA_DESDE'};
    } else {
        $filtros .= " (ART_FECHAPHORAP <= ?) ";
        push @params, $dthhmm_system;
    }

    $filtros .= " and (ART_ALTA = \"1\") ";

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CONTROL_FECHA'} eq 'SI') {
        $filtros .= " and ( (ART_FECHAEHORAE >= ?) OR ( (ART_FECHAEHORAE < ?) AND (ART_SOLOPORTADAS = \"1\") ) )";
        push @params, $dthhmm_system;
        push @params, $dthhmm_system;
    }

    return ($filtros, \@params);
}

# ---------------------------------------------------------------
# cargo los filtros para los taxport filtradas
sub cargar_fil_cfg {
    my $file = $_[0];
    my $fil = $_[1];

    return if (!-f $file); # no se hace nada si no existe el directorio y archivo del filtro.
    return if (exists $CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fil}); # para no cargarlo dos veces.

    my $cfg = &glib_fildir_03::read_file($file);

    if ($cfg =~ m/\s*TAXPORT_FIDS\s*=\s*("|')(.*?)("|')/) {
        my $value = $2;

        # Se limpian los espacios.
        $value =~ s/\s+/ /sg;
        $value =~ s/^\s//sg;
        $value =~ s/\s$//sg;
        $value =~ s/[^a-zA-Z0-9_,]//sg; # dejar solo caracteres permitidos

        my @valores = split(',', $value);

        $CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fil}{'FIDS'} = \@valores;

        #print STDERR "CFG TAXPORT_FIDS! fil[$fil] value[$value]\n";
    }

    if ($cfg =~ m/\s*TAXPORT_PLANTILLAS\s*=\s*("|')(.*?)("|')/) {
        my $value = $2;

        # Se limpian los espacios.
        $value =~ s/\s+/ /sg;
        $value =~ s/^\s//sg;
        $value =~ s/\s$//sg;
        $value =~ s/[^a-zA-Z0-9_\-,\.]//sg; # dejar solo caracteres permitidos

        my @valores = split(',', $value);

        foreach my $tpl (@valores) {
            $CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fil}{'PLANTILLAS'}{$tpl} = 1;
        }

        #print STDERR "CFG TAXPORT_PLANTILLAS! fil[$fil] value[$value]\n";
    }

    if ($cfg =~ m/\s*TAXPORT_FECHAP?_DESDE\s*=\s*("|')(.*?)("|')/s) { # fecha de publicacion, ART_FECHAP
        my $value = $2;

        # Se limpian los espacios.
        $value =~ s/\s+/ /sg;
        $value =~ s/^\s//sg;
        $value =~ s/\s$//sg;

        if ($value eq 'now') {
            $value = strftime("%Y%m%d", localtime);
        } else {
            $value =~ s/[^0-9]//sg; # dejar solo caracteres permitidos, numeros.
        }

        $value = '' if ($value !~ /^(\d{8})$/); # formato debe ser YYYYMMDD

        $CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fil}{'FECHA_DESDE'} = $value;
        #print STDERR "CFG CFG_FIL_TAXPORT! fil[$fil] value[$value]\n";
    }

    if ($cfg =~ m/\s*TAXPORT_ORDEN\s*=\s*("|')(.*?)("|')/s) { # fecha de publicacion, ART_FECHAP
        my $value = $2;
        my $taxport_orden = 'ART_FECHAP desc, ART_HORAP desc'; # valor por defecto.

        # Se limpian los espacios.
        $value =~ s/\s+/ /sg;
        $value =~ s/^\s//sg;
        $value =~ s/\s$//sg;

        if ($value =~ /^(PUBLICACION|TITULAR|CREACION)\((ASC|DESC)\)$/) {
            if ($1 eq 'PUBLICACION') {
                $taxport_orden = "ART_FECHAP $2, ART_HORAP $2";
            } elsif ($1 eq 'TITULAR') {
                $taxport_orden = "ART_TITU $2";
            } elsif ($1 eq 'CREACION') {
                $taxport_orden = "ART_AUTOINC $2";
            }
        }

        $CFG_FIL_TAXPORT{$prontus_varglb::PRONTUS_ID}{$fil}{'TAXPORT_ORDEN'} = $taxport_orden;
        #print STDERR "CFG CFG_FIL_TAXPORT! fil[$fil] value[$value] taxport_orden[$taxport_orden]\n";
    }
}

# -------------------------------------------------------------------------
# Se buscan los directorios que comiencen con fil_ en las plantillas de taxport.
sub load_taxport_fil {
    my $ruta_dir = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_ID'}}{'DIR_TEMPLATESLATES'}$prontus_varglb::DIR_TAXPORT";
    my @listado = glob("$ruta_dir/fil_*");

    foreach my $dir (@listado) {
        if ($dir =~ /fil_(.*?)$/) {
            &cargar_fil_cfg("$dir/filtros.cfg", "fil_" . $1);
        }
    }
}

# -------------------------------------------------------------------------
# obtiene datos de articulos relacionados
sub get_relac_for_fid {
    my ($base) = $_[0];
    my $ts_relacionados = $_[1]; # 1,2,3...
    # relacionados asignados
    my $sql = "SELECT ART_ID, ART_TITU FROM ART WHERE";
    my @ts_relacionados_array = split ',', $ts_relacionados;

    if ($ts_relacionados =~ /\d{14}/) {
        # son ts de articulos
        $sql .= " ART_ID";
    } else {
        # son ids de articulos
        $sql .= " ART_AUTOINC ";
    }

    my @placeholders = ('?') x scalar @ts_relacionados_array;
    my $placeholders_str = join ',', @placeholders;

    $sql .= " IN ($placeholders_str) ORDER BY ART_ID ASC";

    my ($idrel_asignado, $titular);
    my $relac_query = &glib_dbi_03::ejecutar_sql_param($base, $sql, \@ts_relacionados_array);
    $relac_query->bind_columns(\($idrel_asignado, $titular));

    # Se leen los relacionados de la BD
    my %relac_asignados;

    while ($relac_query->fetch) {
        &glib_str_03::correct_encode_utf8_string($titular);

        $titular = &lib_prontus::escape_xml($titular);
        $relac_asignados{$idrel_asignado} = $titular;
    }

    $relac_query->finish;

    return \%relac_asignados;
}

#---------------------------------------------------------------
sub carga_tax_json {
    my $num_elem = keys(%TAX);
    my $src_file = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}$prontus_varglb::DIR_DATA4FIDS/taxs.json";

    if ($num_elem) {
        # archivo fue modificado desde la ultima vez
        my $modtime = (stat($src_file))[9];
        return if ($MODTIME >= $modtime);
    }

    my $str_json = &glib_fildir_03::read_file_bin($src_file);

    if ($str_json =~ /^\{.*\}$/) {
        %TAX = %{decode_json($str_json)};
        $MODTIME = (stat($src_file))[9];
    }
}
#---------------------------------------------------------------
sub generar_json {
    my $bd = $_[0];
    my %taxonomia = &carga_hash_seccion($bd, 1);

    foreach my $secc_id (keys %taxonomia) {
        $taxonomia{$secc_id}{'temas'} = {&carga_hash_temas($bd, 1, $secc_id)};

        if (scalar(keys %{$taxonomia{$secc_id}{'temas'}}) > 0) {
            foreach my $tema_id (keys %{$taxonomia{$secc_id}{'temas'}}) {
                $taxonomia{$secc_id}{'temas'}{$tema_id}{'subtemas'} = {&carga_hash_subtemas($bd, 1, $tema_id)};
            }
        }
    }

    my $str_json = encode_json(\%taxonomia);
    utf8::decode($str_json);
    my $dst_path = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}$prontus_varglb::DIR_DATA4FIDS";

    &glib_fildir_03::check_dir($dst_path);
    &glib_fildir_03::write_file("$dst_path/taxs.json", $str_json);
}

# ---------------------------------------------------------------
sub check_taxport_running {
    my ($res) = qx/ps axww | grep 'prontus_cron_taxport.cgi' | grep -v grep | grep -v 'sh -c' | grep -v tail | wc -l/;

    $res =~ s/\D//gs;
    $res += 0; # para forzar el casteo a numero

    return $res;
}

# ---------------------------------------------------------------
sub check_worker_running {
    my ($res) = qx/ps axww | grep 'prontus_cron_taxport_worker.cgi' | grep -v grep | grep -v 'sh -c' | grep -v tail | wc -l/;

    $res =~ s/\D//gs;
    $res += 0; # para forzar el casteo a numero

    return $res;
}

sub borrar_taxport {
    my $taxport = $_[0];
    my ($fid, $seccion, $tema, $subtema, $pagina) = split /\//, $taxport;
    my $reldir_port_dst = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_TAXPORT";
    my $reldir_port_plt = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_TEMPLATES'}$prontus_varglb::DIR_TAXPORT";
    my $comb;
    my @patrones;

    $seccion = '' if ($seccion eq '0');
    $tema = '' if ($tema eq '0');
    $subtema = '' if ($subtema eq '0');

    print STDERR "borrar_taxport [$taxport] fid[$fid] seccion[$seccion] tema[$tema] subtema[$subtema] pagina[$pagina]\n";

    # /prontus_develop/site/tax/port/all/taxport_1___1.html
    if ($seccion ne '') {
        $comb = "_$seccion\_*.*";
    }

    my @lisdir = &glib_fildir_03::lee_dir("$prontus_varglb::DIR_SERVER$reldir_port_plt");

    foreach my $subdir (@lisdir) {
        next if (!-d "$prontus_varglb::DIR_SERVER$reldir_port_plt/$subdir");

        my @lisdir_files = &glib_fildir_03::lee_dir("$prontus_varglb::DIR_SERVER$reldir_port_plt/$subdir");
        @lisdir_files = grep !/^\./, @lisdir_files;

        foreach my $arch (@lisdir_files) {
            next if (!-f "$prontus_varglb::DIR_SERVER$reldir_port_plt/$subdir/$arch");
            next if (!-s "$prontus_varglb::DIR_SERVER$reldir_port_plt/$subdir/$arch");
            next if ($subdir =~ /^(.*?)-(.*?)$/); # si es vista se ignora.

            $arch =~ /^(.*?)\.(\w+)$/;
            my $filename = $1;
            my $ext = $2;

            push @patrones, "$reldir_port_dst/$subdir/$filename\____*.*";

            foreach my $mv (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTAS'}}) {
                push @patrones, "$reldir_port_dst/$subdir-$mv/$filename\____*.*";
            }

            if ($comb) {
                push @patrones, "$reldir_port_dst/$subdir/$filename$comb";

                foreach my $mv (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTAS'}}) {
                    push @patrones, "$reldir_port_dst/$subdir-$mv/$filename$comb";
                }
            }
        }
    }

    foreach my $patron (@patrones) {
        my $file = "$prontus_varglb::DIR_SERVER$patron";

        # print STDERR "borrando [$file]\n";
        unlink(glob($file));
    }
}


sub get_nom_seccion {
    my ($bd, $id) = @_;
    my ($sql, $salida, $nom);
    my @params = ($id);

    $sql = "select SECC_NOM from SECC WHERE SECC_ID = ?";
    $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($nom));
    $salida->fetch;
    $salida->finish;

    glib_str_03::correct_encode_utf8_string($nom);

    return $nom;
}

sub get_nom_tema {
    my ($bd, $id) = @_;
    my ($sql, $salida, $nom);
    my @params = ($id);

    $sql = "select TEMAS_NOM from TEMAS WHERE TEMAS_ID = ?";

    $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($nom));
    $salida->fetch;
    $salida->finish;

    glib_str_03::correct_encode_utf8_string($nom);

    return $nom;
}

sub get_nom_subtema {
    my ($bd, $id) = @_;
    my ($sql, $salida, $nom);
    my @params = ($id);

    $sql = "select SUBTEMAS_NOM from SUBTEMAS WHERE SUBTEMAS_ID = ?";

    $salida = glib_dbi_03::ejecutar_sql_bind_param($bd, $sql, \@params, \($nom));
    $salida->fetch;
    $salida->finish;

    glib_str_03::correct_encode_utf8_string($nom);

    return $nom;
}

1;
