#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------

# Realiza una revision en el sistema para ver si los modulos perl
# requeridos por Prontus estan presentes.

# Invocaciones:
# - por web: /cgi-cpn/prontus_check_platform.cgi
# - por S.O.: perl prontus_check_platform.cgi
# En ambas modalidades el resultado es identico.
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
}

use lib_prontus;
use lib_profile;
use prontus_varglb; &prontus_varglb::init();

# ---------------------------------------
# Check versions of dependencies.  0 for version = any version acceptable
our $MODULES = [
    {name => 'DBI', version => '1.50'},
    {name => 'DBD::mysql', version => '3.0002'},
    {name => 'GD',version => '2.30'},
    {name => 'JSON',version => '2.14'},
    {name => 'LockFile::Simple',version => '0.205'},
    # email
    {name => 'Email::Simple', version => '2.203'},
    {name => 'Email::MIME', version => '1.926'},
    {name => 'Email::Sender',version => '1.300016'},
    {name => 'IO::All', version => '0.79'},
    {name => 'File::LibMagic', version => '1.02'},
    {name => 'Try::Tiny', version => '0.22'},

    {name => 'XML::Parser',version => '2.34'},
    {name => 'HTTP::Response',version => '1.53',from => 'libwww'},
    {name => 'LWP::UserAgent',version => '2.033',from => 'libwww'},
    {name => 'LWP::Protocol::https',version => '6.06'},
    {name => 'Net::DNS', version => '0.65'},
    {name => 'URI::Escape', version => '3.20'},
    {name => 'Crypt::Eksblowfish::Bcrypt', version => '0.009'},
    {name => 'MIME::Base64', version => '3.15'},
    {name => 'Crypt::JWT', version => '0.023'},
    {name => 'Digest::SHA', version => '5.60'},
    {name => 'MIME::Parser', version => '5'},
    {name => 'Image::ExifTool', version => '9'},
    {name => 'Net::Amazon::S3', version => '0.85'},
];

our $ERROR_CSS_CLASS = 'check-error'; # usada para resaltar con CSS mensajes de error en la vista web.

# ---------------------------------------
our $AMBIENTE_WEB;
main: {
    # este script puede correr por consola o via web. En este caso usa una plantilla en la que reemplaza los valores que va recogiendo.
    &lib_prontus::setUtf8();
    my $pagina;
    if ($ENV{'SERVER_NAME'} ne '') {
        $AMBIENTE_WEB = 1;
        $pagina = &lib_prontus::load_cpan_core_plt("core/prontus_check_platform.html");
        $pagina = &lib_prontus::set_coreplt_ppal_simple($pagina);
        print "Content-Type: text/html\n\n";
    } else {
        $pagina = &lib_prontus::load_cpan_core_plt("core/prontus_check_platform.txt");
        $pagina = &lib_prontus::set_coreplt_ppal_simple($pagina);
    }

    # S.O.
    my $os = uc $^O; # solo esta en algunas plataformas

    $pagina =~ s/%%sistema_operativo%%/$os/ig;

    # Perl
    my $perlv = $];
    if ($perlv < 5.020002) {
        my $error_perl = "[Error] Perl version incorrecta, se requiere Perl 5.008 o superior";
        $pagina =~ s/%%error_perl%%/$error_perl/ig;
        $pagina =~ s/%%error_status_perl%%/$ERROR_CSS_CLASS/ig;
        $pagina =~ s/%%.*%%//gis;
        print $pagina;
        exit;
    }

    $perlv =~ /(\d+).(\d{3})(\d{3})/;
    my ($version, $subversion, $revision) =  ($1, $2, $3);
    $version =~ s/^0+//;
    $subversion =~ s/^0+//;
    $revision =~ s/^0+//;

    $perlv = "versión $version.$subversion.$revision - OK.";

    $pagina =~ s/%%perlv%%/$perlv/ig;
    $pagina =~ s/%%error_perl%%//ig;
    $pagina =~ s/%%error_status_perl%%//ig;

    my $result;
    my $error_status;
    my ($loop, $looptmp, $looptot);
    $pagina =~ /%%loop_modules%%(.*)%%\/loop_modules%%/ig;
    $loop = $1;
    foreach my $module (@{$MODULES}) {
        $looptmp = $loop;

        ($result, $error_status) = &have_vers($module->{name}, $module->{version}, $module->{from});
        $looptmp =~ s/%%module_info%%/$result/ig;
        $looptmp =~ s/%%error_status_module%%/$error_status/ig;
        $looptot .= $looptmp;
    }
    $pagina =~ s/%%loop_modules%%.*%%\/loop_modules%%/$looptot/gis;

    my $resp = `unzip`;
    my $unzip_status;
    $error_status = '';
    if ($resp =~ /(unzip) (\d+\.\d+)(.*?)[\n\r]/is) {
        my $ver = $2;
        my $vok = (vers_cmp($ver,'1.00') > -1);
        if ($vok) {
            $unzip_status = "OK (se encontró versión $ver)";
        } else {
            $unzip_status = "[Error] Se encontró versión $ver";
            $error_status = $ERROR_CSS_CLASS;
        }
    } else {
        $unzip_status = "[Error] No encontrado";
        $error_status = $ERROR_CSS_CLASS;
    }

    $pagina =~ s/%%unzip_status%%/$unzip_status/ig;
    $pagina =~ s/%%error_status_unzip%%/$error_status/ig;

    # chequeo de xcoding
    my @paths_ffmpeg;

    # Se busca en las rutas especificadas en la variable de entorno PATH.
    my $str = "echo \$PATH";
    my $ret_path = `$str`;
    $ret_path =~ s/[\n\t\s\r]//sg;

    if ($ret_path) {
        @paths_ffmpeg = split(/:/, $ret_path);
    } else {
        push @paths_ffmpeg, '/usr/local/bin';
        push @paths_ffmpeg, '/usr/bin';
    }

    # ffmpeg
    my $found_path_ffmpeg = 0;
    my $results_check_xcoding;
    foreach my $path_ffmpeg (@paths_ffmpeg) {
        if (-f "$path_ffmpeg/ffmpeg") {
            $found_path_ffmpeg = 1;
            $results_check_xcoding .= &check_xcoding("$path_ffmpeg/ffmpeg");
            $results_check_xcoding .= "\n" if !$AMBIENTE_WEB; # TODO intentar sacar esto.
        }
    }
    $pagina =~ s/%%results_check_xcoding%%/$results_check_xcoding/ig;

    my $error_ffmpeg;
    if (!$found_path_ffmpeg) {
        $error_ffmpeg = "[Error] No se pudo detectar ffmpeg instalado en el sistema. Rutas analizadas: " . join(', ', @paths_ffmpeg);
        $error_status = $ERROR_CSS_CLASS;

    } else {
        $error_ffmpeg = 'La verificación básica fue exitosa.';
        $error_status = '';
    }

    $version = $prontus_varglb::RAMA_INSTALADA;
    $version =~ s/\./_/;
    my $url_manual_desa = $prontus_varglb::URL_MANUAL . '/prontus_desarrollo_v' . $version;
    $pagina =~ s/%%url_manual_desa%%/$url_manual_desa/ig;
    $pagina =~ s/%%error_ffmpeg%%/$error_ffmpeg/gi;
    $pagina =~ s/%%error_status_ffmpeg%%/$error_status/gi;
    print $pagina;
} # main

# ---------------------------------------
# ------------- SUBRUTINAS --------------
# ---------------------------------------

# vers_cmp is adapted from Sort::Versions 1.3 1996/07/11 13:37:00 kjahds,
# which is not included with Perl by default, hence the need to copy it here.
# Seems silly to require it when this is the only place we need it...
sub vers_cmp {
    if (@_ < 2) { die "not enough parameters for vers_cmp" }
    if (@_ > 2) { die "too many parameters for vers_cmp" }
    my ($a, $b) = @_;
    my (@A) = ($a =~ /(\.|\d+|[^\.\d]+)/g);
    my (@B) = ($b =~ /(\.|\d+|[^\.\d]+)/g);
    my ($A,$B);
    while (@A and @B) {
        $A = shift @A;
        $B = shift @B;
        if ($A eq "." and $B eq ".") {
            next;
        } elsif ( $A eq "." ) {
            return -1;
        } elsif ( $B eq "." ) {
            return 1;
        } elsif ($A =~ /^\d+$/ and $B =~ /^\d+$/) {
            return $A <=> $B if $A <=> $B;
        } else {
            $A = uc $A;
            $B = uc $B;
            return $A cmp $B if $A cmp $B;
        }
    }
    @A <=> @B;
}

# ---------------------------------------
# This was originally clipped from the libnet Makefile.PL, adapted here to
# use the above vers_cmp routine for accurate version checking.
sub have_vers {
    my $error_status = '';
    my ($pkg, $wanted, $realname) = @_;
    my ($msg, $vnum, $vstr, $result);
    my $pkg4display = $pkg;
    $pkg4display .= "($realname)" if ($realname);

    my $version = !$wanted?'(any version)':"(v$wanted)";

    $result = sprintf("%28s %-12s ", $pkg4display, !$wanted?'(any)':"(v$wanted)");

    # Modules may change $SIG{__DIE__} and $SIG{__WARN__}, so localise them here
    # so that later errors display 'normally'
    local $::SIG{__DIE__};
    local $::SIG{__WARN__};

    my $file = $pkg . ".pm";
    $file =~ s/::/\//g;
    eval {require $file;};

    # do this twice to avoid a "used only once" error for these vars
    if ($@) {
        $vnum = -1;
    } else {
        use Data::Dumper;
        $vnum = 0;
        eval { $vnum = $pkg->VERSION; };
        if ($@) {
            eval { $vnum = $pkg->Version; };
        }
    }

    if ($vnum eq "-1") { # string compare just in case it's non-numeric
        $vstr = "no encontrado";
    } elsif (vers_cmp($vnum,"0") > -1) {
        $vstr = "se encontró versión $vnum";
    } else {
        $vstr = "versión desconocida";
    }

    my $vok = (vers_cmp($vnum,$wanted) > -1);

    $result .= $vok ? "OK, $vstr\n" : "[Error]: $vstr\n";
    if ($vok) {
        $error_status = '';
    } else {
        $error_status = $ERROR_CSS_CLASS;
    }

    my $datos_gd = '';
    if ($pkg eq 'GD') {
        my $template_name;
        $template_name = "core/check_soporte_gd.html" if $AMBIENTE_WEB;
        $template_name = "core/check_soporte_gd.txt" if !$AMBIENTE_WEB;
        $datos_gd = &lib_prontus::load_cpan_core_plt($template_name);
        $datos_gd = &lib_prontus::set_coreplt_ppal_simple($datos_gd);

        my $resultado_check;
        ($resultado_check, $error_status) = &check_soporte_gd('gif');
        $datos_gd =~ s/%%soporte_gif%%/$resultado_check/gi;
        $datos_gd =~ s/%%error_status_gif%%/$error_status/gi;

        ($resultado_check, $error_status) = &check_soporte_gd('jpeg');
        $datos_gd =~ s/%%soporte_jpeg%%/$resultado_check/gi;
        $datos_gd =~ s/%%error_status_jpeg%%/$error_status/gi;

        ($resultado_check, $error_status) = &check_soporte_gd('png');
        $datos_gd =~ s/%%soporte_png%%/$resultado_check/gi;
        $datos_gd =~ s/%%error_status_png%%/$error_status/gi;
    }
    $result .= $datos_gd;

    return $result, $error_status;
}

# ---------------------------------------
# Checks if the object or class has a method called METHOD. If it does then a reference to the sub is returned.
# If it does not then undef is returned. This includes methods inherited or imported by $obj, CLASS, or VAL.
sub check_soporte_gd { # jpeg, libungif y libpng
    my $format = $_[0];
    my $nomlib;
    my $status;
    $nomlib = 'jpeg' if ($format eq 'jpeg');
    $nomlib = 'libungif' if ($format eq 'gif');
    $nomlib = 'libpng' if ($format eq 'png');
    if (GD::Image->can($format)) {
        return sprintf("%28s %-12s ", "GD - soporte $format", "($nomlib)") . ' OK', '';
    } else {
        return '[Error] ' . sprintf("%28s %-12s ", "GD - soporte $format", "($nomlib) no encontrado"), $ERROR_CSS_CLASS;
    }
}

# ---------------------------------------
sub check_xcoding {
    my $path_ffmpeg = shift;
    my $xcoding_ver = '2.1.0';
    my $result;
    my $error_status = '';
    my $template_name;
    $template_name = "core/check_xcoding.html" if $AMBIENTE_WEB;
    $template_name = "core/check_xcoding.txt" if !$AMBIENTE_WEB;
    my $datos_xcoding = &lib_prontus::load_cpan_core_plt($template_name);
    $datos_xcoding = &lib_prontus::set_coreplt_ppal_simple($datos_xcoding);
    $datos_xcoding =~ s/%%path_ffmpeg%%/$path_ffmpeg/gi;

    # Primero se chequea la version
    my $resp = `$path_ffmpeg -version 2>&1`;
    if($resp =~ /^FFmpeg version ([^\s,]+)/i) {
        my $ver = $1;
        my $origver = $ver;
        $result = sprintf("%28s %-12s ", 'FFmpeg', "($xcoding_ver)");
        $datos_xcoding =~ s/%%result_xcoding_ver%%/$result/gi;

        # eliminamos el texto '.git' si es una version de git, lo reemplazamos por un 0
        $ver =~ s/(git)/0/;
        # si es una version sin subversion agregamos 0
        $ver =~ s/^(\d+\.\d+)$/$1.0/;
        # deja solo los 3 numeros de la versión
        $ver =~ s/((\d+\.\d+\.\d+).*)/$2/;

        if($ver =~ /\d+\.\d+\.\d+/) {
            my $vok = (vers_cmp($ver,$xcoding_ver) > -1);
            my $error = "[Error] Se encontró $origver";
            $result = ((($vok) ? "OK (se encontró $origver)\n" : $error));
            if ($vok) {
                $error_status = $ERROR_CSS_CLASS;
            }

        } elsif ($resp =~ /(built on .*?) with/) {
            my $built = $1;
            $result = "[Error] No se pudo comparar la versión ($built)";
            $datos_xcoding =~ s/%%result_xcoding_ver%%/$result/gi;
        } else {
            $result = "[Error] No se pudo comparar versión ($ver)";
        }

        $datos_xcoding =~ s/%%result_comparar_version%%/$result/gi;

        # Se comprueba soporte para libx264
        $result = sprintf("%28s %-12s ", 'FFmpeg con soporte libx264', '');
        if($resp =~ /--enable-libx264/) {
            $result .= "OK";
        } else {
            my $resp = `$path_ffmpeg -codecs 2> /dev/null | grep x264`;
            if($resp ne '') {
                $result .= "OK";
            } else {
                $result = "[Error] ".$result . " No habilitado";
            }
        }
        $datos_xcoding =~ s/%%result_soporte_libx264%%/$result/gi;

        # Se comprueba soporte para aac
        $result = sprintf(" %28s %-12s ", 'FFmpeg con soporte aac', '');
        my $resp = `$path_ffmpeg -codecs 2> /dev/null | grep " aac "`;
        if ($resp =~ /\(encoders:.*?aac.*?\)/ || $resp =~ /DEA.*?aac/) {
            $result .= "OK";
        } else {
            $result = "[Error] ".$result. " No habilitado";
        }
        $datos_xcoding =~ s/%%result_soporte_aac%%/$result/gi;

        # Se comprueba soporte para libfdk_aac
        $result = sprintf("%28s %-11s ", 'FFmpeg con soporte libfdk_aac', '');
        if ($resp =~ /--enable-libfdk-aac/) {
            $result .= "OK";
        } else {
            $resp = `$path_ffmpeg -codecs 2> /dev/null | grep libfdk_aac`;
            if($resp ne '') {
                $result .= "OK";
            } else {
                $result = "[Error] ".$result. "No habilitado";
            }
        }
        $datos_xcoding =~ s/%%result_soporte_libfdk_aac%%/$result/gi;

        # Se comprueba soporte para libmp3lame
        $result = sprintf("%28s %-11s ", 'FFmpeg con soporte libmp3lame', '');
        $resp = `$path_ffmpeg -codecs 2> /dev/null | grep libmp3lame`;
        if($resp ne '') {
            $result .= "OK";
        } else {
            $result = "[Error] ".$result. "No habilitado";
        }
        $datos_xcoding =~ s/%%result_soporte_libmp3lame%%/$result/gi;

        $datos_xcoding =~ s/%%ffmpeg_no_legible%%//gi;
    } else {
        $result = "[Error] No se pudo leer la versión de ffmpeg.";
        $datos_xcoding =~ s/%%ffmpeg_no_legible%%/$result/gi;
    }

    return $datos_xcoding;
}
