#!/usr/bin/perl

# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# SCRIPT.
# -----------
# twitter_posting.cgi

# ---------------------------------------------------------------
# UBICACION.
# -----------
# /cgi-cpn/
# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# Ejecuta la publicación a Youtube de un video del artículo.
# ---------------------------------------------------------------
# INVOCACIONES ACEPTADAS.
# ------------------------
# via web.
# -------------------------------BEGIN SCRIPT--------------------
# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ---------------------------
use utf8;
use strict;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
    $pathLibsProntus =~ s/\/[^\/]+\/?$//;
    unshift(@INC,$pathLibsProntus); # Para dejar disponibles las librerias de prontus
}

# Captura STDERR
use lib_stdlog;
&lib_stdlog::set_stdlog($0, 51200);

use lib_prontus;
use prontus_varglb; &prontus_varglb::init();
use glib_fildir_03;
use glib_hrfec_03;
use File::Copy;
use Artic;
use JSON;
use lib_mam;

main:{
    if (scalar @ARGV < 1) {
        print "Status: 404 Not Found\n\n";
        print STDERR "Parametros insuficientes Terminando\n";
        exit;
    }

    # Rescata el origen (html del artículo) y obtiene el XML.
    my $origen = $ARGV[0];
    #print STDERR "ORIGEN[$origen] \n";
    if ($origen eq '') {
        print STDERR "[youtube_posting.cgi: debe invocarse con un parámetro con el path al artículo.\n";
        exit;
    }
    my $document_root;
    my $PRONTUS_ID;
    my $ts;
    my $fechac;

    # hash con el status del upload. Se escribe como JSON al log de upload de cada
    # video en [prontus_id]/cpan/data/youtube_post/[fechac]/files/resp[ts].json
    my %status_hash = ('video_id'=>'null', 'video_state'=>'pending');
    my $status_fname = '';
    # Descompone el Path de entrada en datos útiles
    if($origen =~ m|^(.*)/(.*?)/site/artic/(\d{8})/\w+/(\d{14})\.\w+$|) {
        $document_root = $1;
        $PRONTUS_ID = $2;
        $fechac = $3;
        $ts = $4;
    } else {
        print STDERR "[youtube_posting.cgi: error al descomponer el path de entrada: [$origen]\n";
        # no es posible escribir al log de status :(
        exit;
    }

    # cargamos la configuración del Prontus
    &lib_prontus::load_config_by_id($PRONTUS_ID);

    my $buffer = &lib_prontus::get_xml_data($document_root . "/$PRONTUS_ID/site/artic/$fechac/xml/$ts.xml");
    my %hash = &lib_prontus::getCamposXml($buffer, 'multimedia_video1');

    # Lo primero que hacemos, es ver si se debe procesar o no
    if(!$hash{'multimedia_video1'}) {
        print STDERR "[$ts][youtube] No se hace nada, no existe multimedia_video1\n";
        $status_hash{'video_id'} = 'null';
        $status_hash{'video_state'} = 'failed';
        &glib_fildir_03::write_file($status_fname, encode_json(\%status_hash));
        exit;
    }

    # revisamos si ya esta corriendo el proceso para este articulo para no abrirlo de nuevo
    my $cmd = "ps axww | grep 'youtube_post' | grep $ts | grep $PRONTUS_ID | grep -v grep | grep -v 'nice' | grep -v 'editor' | grep -v 'sh -c' | grep -v 'tail' | grep -v execIfMaster";
    my(@result) = qx/$cmd/; # wc -l

    # si ya esta corriendo no se hace nada
    if (scalar @result > 1) {
        print STDERR "[$ts][youtube] Ya esta corriendo el proceso para este artículo\n";
        exit;
    }

    # revisamos el contenido del json de status, si existe fue porque un proceso lo creó
    my $youtube_status_dir = "$document_root/$PRONTUS_ID/cpan/data/youtube_post/files/";

    if (-d $youtube_status_dir) {
        $status_fname = $youtube_status_dir . substr($ts, 0, 6)."/resp$ts.json";
        if (-s $status_fname) {
            my $youtube_buffer = &glib_fildir_03::read_file($status_fname);
            my %hash_youtube = JSON->new->utf8->decode($youtube_buffer);
            # si el estado no es vacío significa que se esta procesando
            if ($hash_youtube{'video_state'} ne '' ) {
                print STDERR "[$ts][youtube] Ya se procesó el upload de este artículo [$hash_youtube{'video_state'}]\n";
                exit;
            }
        }
    } else {
        &glib_fildir_03::check_dir($youtube_status_dir);
    }

    my $path_video = "$document_root/$PRONTUS_ID/site/mm/$fechac/mmedia/";

    # Si efectivamente hay que procesar, sacamos una copia para hacer upload, porque Prontus puede procesar el video, transcodificarlo o similares.
    # Se hace una copia sin cambios, para poder subir el video original ke cargo el usuario y no el transcodificado por Prontus.
    # Ademas, se espera que la transferencia sea mas lenta que lo que demora en transcodificar el Prontus/servidor,
    # entonces podria pasar que el archivo que se esta subiendo (si se usa el mismo que esta procesando el Prontus)
    # pueda ser reemplazado en medio de la transferencia y queede mal el upload
    my $video = $hash{'multimedia_video1'};
    my $video_original = $path_video . $video;
    # print STDERR "[$ts][youtube] $video_original\n";

    # Se chequea que hay video
    if (! -f $video_original) {
        print STDERR "No existe original [$video_original]\n";
        exit;
    }

    my $temp_path = "$document_root/ramdisk/youtube/";
    &glib_fildir_03::check_dir($temp_path);
    my $video_nuevo = $temp_path . $prontus_varglb::PRONTUS_ID . $video;
    print STDERR "[$ts][youtube] Copiando video [$video_original  ->  $video_nuevo]\n";

    if (!copy($video_original, $video_nuevo)) {
        $status_hash{'video_state'} = 'failed';
        $status_hash{'video_id'} = 'null';
        &glib_fildir_03::write_file($status_fname, encode_json(\%status_hash));
        die "[youtube] Copy failed: $!";
    }

    # Se detecta la ubicación de PHP
    my $phploc = '/usr/bin/php';
    if(! -f $phploc) {
        $phploc = '/usr/local/bin/php';
        if(! -f $phploc) {
            $status_hash{'status'} = 'failed';
            $status_hash{'video_id'} = 0;
            &glib_fildir_03::write_file($status_fname, encode_json(\%status_hash));
            die("No se pudo detectar el binario de PHP");
        }
    }

    # Se revisa si hay un php.ini especial para el sitio
    if($document_root =~ /^(.*?)\/web$/) {
        my $phpinimaybe = $1;
        $phpinimaybe = "$phpinimaybe/conf/php.ini";
        if(-f $phpinimaybe) {
            $phploc = "$phploc --php-ini $phpinimaybe";
        }
    }

    # Finalmente se ejecuta el script PHP
    $cmd = "$phploc $document_root/prontus/cpan/youtube_post/youtube_post.php $ts $prontus_varglb::PRONTUS_ID $video_nuevo";
    # print STDERR "[$ts][youtube] $cmd\n";
    $status_hash{'video_state'} = 'pending';
    $status_hash{'video_id'} = 'null';
    &glib_fildir_03::write_file($status_fname, encode_json(\%status_hash));
    my $video_final = `$cmd`;

    #######################
    my $marca = "url_youtube";

    my $ARTIC_OBJ = Artic->new(
                'prontus_id'         => $prontus_varglb::PRONTUS_ID,
                'public_server_name' => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'},
                'cpan_server_name'   => $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'},
                'document_root'      => $prontus_varglb::DIR_SERVER,
                'ts'                 => $ts,
                'no_check_dirs'      => 1) || return "Error inicializando objeto articulo: $Artic::ERR TS[$ts]";

    my $parse_as_cdata = '';
    my $xml_base = &glib_fildir_03::read_file($ARTIC_OBJ->{fullpath_xml});
    $xml_base = &lib_prontus::replace_in_xml($xml_base, $marca, $video_final, $parse_as_cdata);
    $ARTIC_OBJ->{xml_data} = $xml_base;
    $ARTIC_OBJ->_flush_xml();

    # Regenerar vista principal del artículo
    $ARTIC_OBJ->generar_vista_art('') || return $Artic::ERR;

    my %datos = &lib_prontus::getCamposXml($xml_base, '_fid,_plt');
    my $fid = $datos{'_fid'};

    # Parsear plantillas paralelas.
    foreach my $plt_paralela (@{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FORM_PLTS'}{$fid}{'plt_paralela'}}) {
        $ARTIC_OBJ->generar_vista_art('', $plt_paralela, 1) || return $Artic::ERR;
    }

    # Generar vistas secundarias
    foreach my $mv (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTAS'}}) {
        $ARTIC_OBJ->generar_vista_art($mv) || return $Artic::ERR;
    }

    # Dropbox.
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX'} eq 'SI') {
        &lib_prontus::dropbox_backup("art;$ts");
    }
}
