#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
    $pathLibsProntus =~ s/\/[^\/]+$//;
    unshift(@INC,$pathLibsProntus); # Para dejar disponibles las librerias de prontus
}

use lib_prontus;
# Captura STDERR
use lib_stdlog;
&lib_stdlog::set_stdlog($0, 51200, 'wizard_error_log');

use glib_cgi_05;
use glib_fildir_03;
use glib_hrfec_03;
use prontus_varglb; &prontus_varglb::init();
use File::Copy;
use glib_dbi_03;
use DBI;
use lib_setbd;
use wizard_lib;

our $CRLF = qr/\x0a\x0d|\x0d\x0a|\x0a|\x0d/;

main:{
    &lib_prontus::setUtf8();
    &glib_cgi_05::new();

    my ($msg_err,
        $prontus_id,
        $extension,
        $smtp,
        $model_nom,
        $title_site_name,
        $new_title_site_name,
        $server_bd,
        $nom_bd,
        $user_bd,
        $pwd_bd,
        $superuser_bd,
        $superpwd_bd) = &check_paso_anterior();

    my $lnk_paso1 = '<br/><br/><a href="/wizard_prontus/core/paso1.html">Volver a Paso 1</a><br/><br/>';

    if ($msg_err) {
        $prontus_varglb::DIR_CORE = '/wizard_prontus/core'; # solo para efectos de la plantilla de mensaje
        &lib_prontus::print_pag_result('Error', $msg_err . $lnk_paso1, 0, 1, 1, "nolink");
    }

    my ($err_basedatos, $err_postprocesos) = &deploy($prontus_id, $extension, $smtp, $model_nom, $title_site_name, $new_title_site_name, $server_bd, $nom_bd, $user_bd,$pwd_bd,$superuser_bd,$superpwd_bd);

    if ($err_basedatos ne '') {
        $prontus_varglb::DIR_CORE = '/wizard_prontus/core'; # solo para efectos de la plantilla de mensaje
        &lib_prontus::print_pag_result('Error', $err_basedatos . "<br/><br/>El Asistente Prontus no pudo instalar correctamente la base de datos, no es posible continuar con el proceso.<br/><br/>Verifique los datos indicados para la base de datos.<br /><br />$lnk_paso1", 0, 1, 1, "nolink");
    }

    if ($err_postprocesos ne '') {
        $prontus_varglb::DIR_CORE = '/wizard_prontus/core'; # solo para efectos de la plantilla de mensaje
        &lib_prontus::print_pag_result('¡Atención!', $err_postprocesos . "<br/><br/>Aunque ocurrió este error, Wizard Prontus de igual manera instaló el publicador, intente ejecutar nuevamente el proceso que falló, desde el Panel de Control del publicador instalado.<br/><br/>Haga click <a href=\"wizard_reporte.cgi?DIR=/$prontus_id&TOPE=/\">aquí</a> para continuar y ver el reporte de instalación.<br /><br />", 0, 1, 1, "nolink");
    }

    print "Status: 303 See Other\n";
    print "Location: /$prontus_varglb::DIR_CGI_CPAN/wizard/wizard_reporte.cgi?DIR=/$prontus_id&TOPE=/\n\n";
}

sub deploy {
    my ($prontus_id, $extension, $smtp, $model_nom, $title_site_name, $new_title_site_name, $server_bd, $nom_bd, $user_bd,$pwd_bd,$superuser_bd,$superpwd_bd) = @_;

    # Crea la BD si es que se indicaron los datos y si es que no existe.
    my $err_createdb;
    if (($superuser_bd ne '') && ($superpwd_bd ne '')) {

        # ver si existe
        my @databases = DBI->data_sources("mysql", {"host" => $server_bd, "user" => $superuser_bd, "password" => $superpwd_bd});
        my $existe_base = 0;
        foreach my $k (@databases) {
            if ($k eq $nom_bd) {
                $existe_base = 1;
                last;
            }
        }

        # si no existe, crearla
        if (!$existe_base) {
            # crear BD
            my $drh = DBI->install_driver('mysql') || warn "No se pudo obtener handler para crear BD. DBI Error Code: [$DBI::err][$DBI::errstr] ";
            if ($DBI::err) {
                $err_createdb = "No fue posible crear base de datos MySQL '$nom_bd'. Cod[$DBI::err][$DBI::errstr]";
            } else {
                my $rc = $drh->func("createdb", $nom_bd, $server_bd, $superuser_bd, $superpwd_bd, 'admin') || warn "Error en createdb. DBI Error Code: [$DBI::err][$DBI::errstr]";
                $err_createdb = "No fue posible crear base de datos MySQL '$nom_bd'. Cod[$DBI::err][$DBI::errstr]" if ($DBI::err);
            }

            return ($err_createdb, '') if ($err_createdb ne '');
            # Si llega hasta aca, es porque se ha creado ok.
        }

        # Si se especifico superusuario, haya existido o no la bd, igual regulariza el user y el charset

        # Conectarse a la bd re100 creada, para asignar user y setear charset
        my ($base_admin, $conn_result_admin) = &lib_prontus::conectar_prontus_bd($nom_bd, $server_bd, $superuser_bd, $superpwd_bd,);
        return ($conn_result_admin, '') if ($conn_result_admin ne '');

        # asignar el usuario, previa conn normal a la BD
        my $sql_grant = "grant all privileges on $nom_bd.* to '$user_bd'\@'$server_bd' identified by '$pwd_bd'";
        $base_admin->do($sql_grant)
                     || warn "Error en grant, DBI Error Code: [$DBI::err][$DBI::errstr] sql[$sql_grant]";
        return ("No es posible asignar privilegios al usuario de la base de datos MySQL. Cod[$DBI::err][$DBI::errstr]", '') if ($DBI::err);

        # asignar el charset, previa conn normal a la BD
        $base_admin->do("alter database $nom_bd CHARACTER SET=utf8mb4 COLLATE=utf8mb4_unicode_ci")
                     || warn "Error al setear charset, DBI Error Code: [$DBI::err][$DBI::errstr]";
        return ("No es posible establecer el charset para base de datos MySQL, es probable que el superusuario especificado no tenga privilegios de 'alter' para la BD. Cod[$DBI::err][$DBI::errstr]", '') if ($DBI::err);

        $base_admin->disconnect;
    }

    # Intenta conectarse a la BD existente, si no resulta, aborta todo.
    my ($base, $conn_result) = &lib_prontus::conectar_prontus_bd($nom_bd, $server_bd, $user_bd, $pwd_bd);
    return ($conn_result, '') if ($conn_result ne '');

    # En este punto, se supone que ya existe la BD y esta creada la conn,
    # Por tanto ya se pueden crear las tablas.
    # Si ya existen, no crea nada y arroja error, puede significar que se esta usando la misma BD para 2 prontuses!

    my ($msg_ret, $hay_err);
    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_secc($base);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_temas($base);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_subtemas($base);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_tags($base);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_tagsart($base);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_asset($base);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_art($base);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_art_publicar($base);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_art_despublicar($base);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_url($base);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_urlf5($base, $prontus_varglb::URL_MAX_LENGTH);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_multitag_s($base);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_multitag_t($base);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_multitag_st($base);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_multitag_art_s($base);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_multitag_art_t($base);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_multitag_art_st($base);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_usuarios($base);
    return ($msg_ret, '') if ($hay_err ne '');

    # insertar usuario admin
    my $sql = "INSERT INTO USERS (USERS_ID, USERS_NOM, USERS_USR, USERS_PSW, USERS_PROFILE_ID, USERS_EMAIL, USERS_EXP_DAYS, USERS_FEC_EXP, USERS_RENEW_PSW, USERS_2FA, USERS_OIDC, USERS_DELETED) VALUES(1, 'Administrador', 'admin', '', 1, '', 0, 0, 0, 0, 0, 0)";
    my $res = $base->do($sql);
    if (!$res) {
        return ("Error actualizando la tabla USERS: sql[$sql][$!]", '');
    }

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_perfiles($base);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_old_psw($base);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_port_users($base);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_art_users($base);
    return ($msg_ret, '') if ($hay_err ne '');

    ($msg_ret, $hay_err) = &lib_setbd::crear_tabla_prontus_users($base);
    return ($msg_ret, '') if ($hay_err ne '');

    $base->disconnect;

    my $dir_models = "$prontus_varglb::DIR_SERVER/wizard_prontus/models";
    my $path_model = "$dir_models/$model_nom";

    # copiar /wizard_prontus/models/<modelo> --> /<prontus_id>
    &glib_fildir_03::copy_tree($dir_models, $model_nom, $prontus_varglb::DIR_SERVER, $prontus_id);

    my $path_to_cfg = "$prontus_varglb::DIR_SERVER/wizard_prontus/models/$model_nom/$model_nom.cfg";

    # Existencia de cfg del modelo
    if (!-f $path_to_cfg) {
        return ('', 'No se econtró cfg del modelo [$model_nom]');
    }

    # Extension de paginas del modelo
    my $buffer_cfg = &glib_fildir_03::read_file($path_to_cfg);

    # Rescata nom del prontus_base
    my $prontus_base = $model_nom;
    if ($buffer_cfg =~ /PRONTUS_BASE\s*=\s*["']([^'"]+)["']/) {
        $prontus_base = $1;
    } else {
        return ('', 'No esta la configuracion PRONTUS_BASE en el cfg del modelo');
    }

    # No copiar el id, lo crea la aplicacion
    # al copiar la carpeta completa del modelo ya se copiaron los cfg, ahora solo hay que renombrarlos.
    &File::Copy::move("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_base" . '-art.cfg', "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id" . '-art.cfg');
    &File::Copy::move("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_base" . '-port.cfg', "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id" . '-port.cfg');
    &File::Copy::move("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_base" . '-tax.cfg', "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id" . '-tax.cfg');
    &File::Copy::move("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_base" . '-usr.cfg', "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id" . '-usr.cfg');
    &File::Copy::move("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_base" . '-var.cfg', "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id" . '-var.cfg');
    &File::Copy::move("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_base" . '-coment.cfg', "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id" . '-coment.cfg');
    &File::Copy::move("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_base" . '-posting.cfg', "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id" . '-posting.cfg');
    &File::Copy::move("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_base" . '-xcoding.cfg', "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id" . '-xcoding.cfg');
    &File::Copy::move("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_base" . '-cache.cfg', "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id" . '-cache.cfg');
    &File::Copy::move("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_base" . '-cloudflare.cfg', "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id" . '-cloudflare.cfg');
    &File::Copy::move("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_base" . '-dropbox.cfg', "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id" . '-dropbox.cfg');
    &File::Copy::move("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_base" . '-dropbox.cfg', "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id" . '-dropbox.cfg');
    &File::Copy::move("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_base" . '-list.cfg', "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id" . '-list.cfg');
    &File::Copy::move("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_base" . '-posting.cfg', "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id" . '-posting.cfg');
    &File::Copy::move("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_base" . '-sitemap.cfg', "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id" . '-sitemap.cfg');
    &File::Copy::move("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_base" . '-tag.cfg', "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id" . '-tag.cfg');

    # Si el CFG de BD del modelo antiguo existe, se borra.
    unlink("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_base" . '-bd.cfg') if(-f "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_base" . '-bd.cfg');

    # eliminar de -var.cfg el server_smtp
    &remove_var_cfg($prontus_id, 'SERVER_SMTP', '-var');

    # Agrega smtp al inicio del -var.cfg
    &add_prontus_cfg($prontus_id, "\n[SERVIDOR SMTP]\nSERVER_SMTP = '$smtp'\n", 1, '-var');
    &add_prontus_cfg($prontus_id, "\n[NOMBRE DEL SERVER NECESARIO PARA CRONS]\nPUBLIC_SERVER_NAME = '$ENV{'HTTP_HOST'}'\n", 0, '-var');

    # Elimina el -id.cfg que traia el modelo, no se usa
    unlink "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_base" . '-id.cfg';

    # Agrega datos de la BD al -bd.cfg
    &add_prontus_cfg($prontus_id, "[BASE DE DATOS]\nNOM_BD = '$nom_bd'\nUSER_BD = '$user_bd'\nPWD_BD = '$pwd_bd'\nSERVER_BD = '$server_bd'\n", 1, '-bd');

    # Cambia referencias
    &cambia_referencias("$prontus_varglb::DIR_SERVER/$prontus_id", $prontus_base, $prontus_id, $title_site_name, $new_title_site_name);

    # Elimina archivos de usuario
    my $dir2del = "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/data/users";
    if (&glib_fildir_03::check_dir($dir2del)) {
        my @lisdir = &glib_fildir_03::lee_dir($dir2del);
        foreach my $k (@lisdir) {
            next if ($k eq '.' or $k eq '..');
            unlink "$dir2del/$k" if (-f "$dir2del/$k");
        }
    }

    # Elimina archivos del modelo, utiles solo para el wizard
    my $file2del = "$prontus_varglb::DIR_SERVER/$prontus_id/$prontus_base.cfg";
    unlink $file2del if (-f $file2del);

    $file2del = "$prontus_varglb::DIR_SERVER/$prontus_id/$prontus_base-big.png";
    unlink $file2del if (-f $file2del);

    $file2del = "$prontus_varglb::DIR_SERVER/$prontus_id/$prontus_base-thumb.png";
    unlink $file2del if (-f $file2del);

    $file2del = "$prontus_varglb::DIR_SERVER/$prontus_id/$prontus_base.tgz";
    unlink $file2del if (-f $file2del);

    $file2del = "$prontus_varglb::DIR_SERVER/$prontus_id/$prontus_base.tgz.md5";
    unlink $file2del if (-f $file2del);

    # Renombra carpeta de indices del buscador
    if (-d "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/data/search/$prontus_base") {
        &File::Copy::move("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/data/search/$prontus_base",
        "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/data/search/$prontus_id");
    }

    # Borra cache de no publicados
    &glib_fildir_03::borra_dir("$prontus_varglb::DIR_SERVER/$prontus_id/cpan/data/cache");

    # Regenerar tabla de articulos, para que se pueble en base a los XML
    system "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_regenerabd_real.cgi $prontus_id";
    return ('', 'Error al poblar tabla de artículos') if ($? != 0);

    system "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/dam/prontus_dam_regen_real.cgi $prontus_id";
    return ('', 'Error al poblar tabla multimedia') if ($? != 0);


    # Importa XML de tax que venia en el modelo en la carpeta $path_model/cpan/procs/tax_import.xml y que se copio al prontus re100 instalado
    if (!-f "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/procs/tax_import.xml") {
        return ('', "Error al importar Categorías del Modelo. No se encuentra, en el prontus instalado, el archivo XML con la información: /$prontus_id/cpan/procs/tax_import.xml")
    }

    system "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_tax_import_real.cgi $prontus_varglb::DIR_SERVER $prontus_id";
    return ('', 'Error al importar Categorías del Modelo') if ($? != 0);

    # Regenerar articulos.
    system "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_art_regen_real.cgi $prontus_id";
    return ('', 'Error al regenerar los articulos') if ($? != 0);

    # Regenerar portadas.
    system "$prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/prontus_port_regen_real.cgi $prontus_id";
    return ('', 'Error al regenerar las portadas') if ($? != 0);

    return ('', '');
}

sub cambia_referencias {
    # Cambia sintaxis de includes a plantillas y site en prontus destino
    my ($dir) = $_[0];
    my ($prontus_base) = $_[1]; # nom del publicador en base al cual se hizo este modelo
    my ($prontus_id) = $_[2];
    my ($title_site_name) = $_[3];
    my ($new_title_site_name) = $_[4];

    if (-d $dir) {
        # Abre directorio.
        opendir(DIR, $dir) || die "Can't opendir" . $dir . $!;
        my @entries = readdir(DIR);
        closedir DIR;

        my $new_tsn = &lib_prontus::escape_xml($new_title_site_name);
        foreach my $entry (@entries) {
            if (($entry ne '.') and ($entry ne '..')) {
                if (-d "$dir/$entry") {
                    &cambia_referencias("$dir/$entry", $prontus_base, $prontus_id, $title_site_name, $new_title_site_name);
                } else {
                    $entry =~ /\.(\w+)$/;
                    my $ext = $1;
                    if ($ext eq 'html' || $ext eq 'shtml' || $ext eq 'php' || $ext eq 'asp' || $ext eq 'xml' || $ext eq 'js' || $ext eq 'css' || $ext eq 'idx') {
                        my $buffer = &glib_fildir_03::read_file("$dir/$entry");
                        $buffer =~ s/$CRLF/\x0a/sg;

                        # Cambia nombre del publicador de origen por el que escogio el usuario
                        $buffer =~ s/$prontus_base/$prontus_id/ig;
                        $buffer =~ s/%%new_prontus_id%%/$prontus_id/ig;

                        # CAMBIA SITE NAME
                        if ($ext eq 'html' || $ext eq 'shtml' || $ext eq 'php' || $ext eq 'asp' || $ext eq 'xml') {
                            if ($buffer =~ /<title>(.*?)<\/title>/i) {
                                my $title_content = $1;

                                $title_content =~ s/$title_site_name/$new_tsn/ig;
                                $buffer =~ s/<title>.*?<\/title>/<title>$title_content<\/title>/i;
                            }
                        }

                        &glib_fildir_03::write_file("$dir/$entry", $buffer);
                    }
                }
            }
        }
    }
};

sub remove_var_cfg {
    # Agrega texto al cfg del publicador destino.
    my $prontus_id = $_[0];
    my $nom_var = $_[1];
    my $tipocfg = $_[2];
    my $cfg_file = "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id$tipocfg.cfg";
    my $buffer_cfg = &glib_fildir_03::read_file($cfg_file);

    $buffer_cfg =~ s/$CRLF/\x0a/sg;
    $buffer_cfg =~ s/(\s*\[[^\]]*\]\n+)?\s*$nom_var\s*=\s*("|').*?(\n|$)//;

    &glib_fildir_03::write_file($cfg_file, $buffer_cfg);
};

sub add_prontus_cfg {
    # Agrega texto al cfg del publicador destino.
    my $prontus_id = $_[0];
    my $text = $_[1];
    my $infront = $_[2];
    my $tipocfg = $_[3];
    my $cfg_file = "$prontus_varglb::DIR_SERVER/$prontus_id/cpan/$prontus_id$tipocfg.cfg";
    my $buffer_cfg = &glib_fildir_03::read_file($cfg_file);

    $buffer_cfg =~ s/$CRLF/\x0a/sg;

    if ($infront) {
        $buffer_cfg = $text . $buffer_cfg;
    } else {
        $buffer_cfg .= $text;
    }

    &glib_fildir_03::write_file($cfg_file, $buffer_cfg);
};

sub check_paso_anterior {
    # Chequea que se haya pasado por el paso 1
    my $INF_FILE = "$prontus_varglb::DIR_SERVER/$wizard_lib::INF_DIR/$wizard_lib::INF_FILE";
    if (!-f $INF_FILE) {
        return "[errInfFile] Solicitud de ejecución no válida.";
    }

    # Leer y cargar y validar contenido del INF.
    my $buffer = &glib_fildir_03::read_file($INF_FILE);
    my ($prontus_id, $extension, $smtp);
    my $title_site_name; # del segmento MODEL (obt. desde cfg del modelo)
    my $new_title_site_name; # del segmento PRONTUS (ingresado por el usuario)
    my ($server_bd, $nom_bd, $user_bd,$pwd_bd,$superuser_bd,$superpwd_bd);

    if ($buffer =~ /(\[PRONTUS\].*\[\/PRONTUS\])/s) {
        my $buffer_prontus = $1;
        # Validar id
        $prontus_id = &wizard_lib::get_prontus_id($buffer_prontus);

        if ($prontus_id eq '') {
            return 'Información de paso 1 está corrupta. [0]';
        }

        # smtp
        if ($buffer_prontus =~ /SERVER_SMTP=([\w\.\-]+)\n/) {
            $smtp = $1;
        }

        if ($buffer_prontus !~ /NEW_TITLE_SITE_NAME=(.+?)\n/) {
            return 'Información de paso 1 está corrupta. [1]';
        } else {
            $new_title_site_name = $1;
        }

        $server_bd = $1 if ($buffer_prontus =~ /server_bd=(.*?)\n/i);
        $nom_bd = $1 if ($buffer_prontus =~ /nom_bd=(.*?)\n/i);
        $user_bd = $1 if ($buffer_prontus =~ /user_bd=(.*?)\n/i);
        $pwd_bd = $1 if ($buffer_prontus =~ /pwd_bd=(.*?)\n/i);
        $superuser_bd = $1 if ($buffer_prontus =~ /superuser_bd=(.*?)\n/i);
        $superpwd_bd = $1 if ($buffer_prontus =~ /superpwd_bd=(.*?)\n/i);
    } else {
        return 'Información de paso 1 está corrupta. [2]';
    }

    print STDERR "Instalar prontus_id[$prontus_id]\n";

    # Validar que no exista el dir destino del prontus.
    # Esto ya se chequea en el paso 1 pero se hace nuevamente por seguridad.
    my $dir_prontus = "$prontus_varglb::DIR_SERVER/$prontus_id";

    if (-d $dir_prontus) {
        return "El directorio [/$prontus_id] ya existe. Para concretar el proceso de instalación, Ud. debe cambiar el nombre especificado para el publicador, o bien, <br>eliminar manualmente el directorio existente que genera el conflicto.";
    } else {
        # Lo creo y luego lo borro para verificar que este ok.
        if (&glib_fildir_03::check_dir($dir_prontus)) {
          &glib_fildir_03::borra_dir($dir_prontus);
        } else {
            return "No se puede crear el directorio destino del publicador [/$prontus_id] No es posible continuar con la instalación.";
        }
    }

    # valida q se haya pasado por paso2
    my $model_nom;
    if ($buffer =~ /(\[MODEL\].*\[\/MODEL\])/s) {
        my $buffer_model = $1;
        # Validar id
        if ($buffer_model !~ /MODEL_NOM=(\w+)\n/) {
            return 'Información de paso 2 está corrupta. [0]';
        } else {
            $model_nom = $1;
        }

        if ($buffer_model !~ /MODEL_EXT=(\w+)\n/) {
            return 'Error en cfg del modelo: extensión no declarada.';
        } else {
            $extension = $1;
        }

        if ($buffer_model !~ /TITLE_SITE_NAME=(.+?)\n/) {
            return 'Error en cfg del modelo: falta TITLE_SITE_NAME';
        } else {
            $title_site_name = $1;
        }
    } else {
        return 'Información de paso 2 está corrupta. [1]';
    }

    return ('', $prontus_id,$extension, $smtp, $model_nom, $title_site_name, $new_title_site_name, $server_bd, $nom_bd, $user_bd,$pwd_bd,$superuser_bd,$superpwd_bd);
}
