#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
    $pathLibsProntus =~ s/\/[^\/]+$//;
    unshift(@INC,$pathLibsProntus); # Para dejar disponibles las librerias de prontus
}

use lib_prontus;
# Captura STDERR
use lib_stdlog;
&lib_stdlog::set_stdlog($0, 51200, 'wizard_error_log');

use glib_cgi_05;
use glib_fildir_03;
use wizard_lib;
use prontus_varglb; &prontus_varglb::init();
use wizard_lib;

# ---------------------------------------------------------------
# MAIN.
# ------
our %FORM = ();

main:{
    &lib_prontus::setUtf8();
    &glib_cgi_05::new();

    $FORM{'PRONTUS_ID'} = &glib_cgi_05::param('PRONTUS_ID');
    $FORM{'SERVER_BD'} = &glib_cgi_05::param('SERVER_BD');
    $FORM{'NOM_BD'} = &glib_cgi_05::param('NOM_BD');
    $FORM{'USER_BD'} = &glib_cgi_05::param('USER_BD');
    $FORM{'PWD_BD'} = &glib_cgi_05::param('PWD_BD');
    $FORM{'SUPERUSER_BD'} = &glib_cgi_05::param('SUPERUSER_BD');
    $FORM{'SUPERPWD_BD'} = &glib_cgi_05::param('SUPERPWD_BD');
    $FORM{'PRONTUS_SMTP'} = &glib_cgi_05::param('PRONTUS_SMTP');
    $FORM{'NEW_TITLE_SITE_NAME'} = &glib_cgi_05::param('NEW_TITLE_SITE_NAME');

    $FORM{'Sbm_ACCION'} = &glib_cgi_05::param('Sbm_ACCION');

    my $msg_err = &validar_datos();

    if ($msg_err) {
        $prontus_varglb::DIR_CORE = '/wizard_prontus/core'; # solo para efectos de la plantilla de mensaje
        &lib_prontus::print_pag_result('Error', $msg_err, 0, 1, 1);
    }

    &grabar_datos();

    print "Status: 303 See Other\n";
    print "Location: /$prontus_varglb::DIR_CGI_CPAN/wizard/wizard_show_paso2.cgi\n\n";
};

sub grabar_datos {
    # Graba la info del paso 1 en el arch. de texto /wizard_prontus/data/inf.txt, seccion [PRONTUS]...[/PRONTUS]
    # creando el dir. inf si no existe.
    my $buffer;

    $buffer = "[PRONTUS]\n"
            . "PRONTUS_ID=$FORM{'PRONTUS_ID'}\n"
            . "SERVER_BD=$FORM{'SERVER_BD'}\n"
            . "NOM_BD=$FORM{'NOM_BD'}\n"
            . "USER_BD=$FORM{'USER_BD'}\n"
            . "PWD_BD=$FORM{'PWD_BD'}\n"
            . "PRONTUS_ID=$FORM{'PRONTUS_ID'}\n"
            . "NEW_TITLE_SITE_NAME=$FORM{'NEW_TITLE_SITE_NAME'}\n";


    $buffer .= "SERVER_SMTP=$FORM{'PRONTUS_SMTP'}\n" if ($FORM{'PRONTUS_SMTP'});

    if (($FORM{'SUPERUSER_BD'} ne '') && ($FORM{'SUPERPWD_BD'} ne '')) {
        $buffer .= "SUPERUSER_BD=$FORM{'SUPERUSER_BD'}\n";
        $buffer .= "SUPERPWD_BD=$FORM{'SUPERPWD_BD'}\n";
    }

    $buffer .= "[/PRONTUS]\n\n";

    my $INF_DIR = "$prontus_varglb::DIR_SERVER/$wizard_lib::INF_DIR";
    if (!&glib_fildir_03::check_dir($INF_DIR)) {
        $prontus_varglb::DIR_CORE = '/wizard_prontus/core'; # solo para efectos de la plantilla de mensaje
        &lib_prontus::print_pag_result('Error', 'No se pudo crear directorio [INF_DIR]', 0, 1, 1);
    }

    my $INF_FILE = "$prontus_varglb::DIR_SERVER/$wizard_lib::INF_DIR/$wizard_lib::INF_FILE";
    &glib_fildir_03::write_file($INF_FILE, $buffer);
};

sub validar_datos {
    # Valida datos provenientes del formulario paso1.
    # Retorna msg de error si lo hay o nada si todo ok.

    if ($FORM{'PRONTUS_ID'} !~ /^[\w\_\-]+$/) {
        return 'Nombre de publicador no válido.';
    }

    if ($FORM{'PRONTUS_ID'} !~ /^[\w\_\-]{1,32}$/) {
        return 'Nombre de publicador debe tener un largo máximo de 32 caracteres.';
    }

    my $dir_prontus = "$prontus_varglb::DIR_SERVER/$FORM{'PRONTUS_ID'}";

    if (-d $dir_prontus) {
        return "El directorio Prontus ya existe. Para continuar con el proceso de instalación Ud. debe cambiar el nombre especificado para el publicador, o bien, <br>eliminar manualmente el directorio existente que genera el conflicto.";
    } else {
        # Lo creo y luego lo borro para verificar que este ok.
        if (&glib_fildir_03::check_dir($dir_prontus)) {
            &glib_fildir_03::borra_dir($dir_prontus);
        } else {
            return "No se puede crear el directorio destino del publicador. No es posible continuar con la instalación.";
        }
    }

    if ($FORM{'SERVER_BD'} !~ /^[\w\-\.]{1,128}$/) {
        return 'Servidor de BD no válido.';
    }

    if ($FORM{'NOM_BD'} !~ /^[\w\-]{1,64}$/) {
        return 'Nombre de BD no válido.';
    }

    if ($FORM{'USER_BD'} !~ /^[\w\-]{1,16}$/) {
        return 'User de BD no válido.';
    }

    if ($FORM{'PWD_BD'} !~ /^[\w\-\.\@\:\$%!#]{1,64}$/) {
        return 'Password de BD no válido.';
    }

    if ($FORM{'SUPERUSER_BD'} !~ /^[\w\-]{0,16}$/) {
        return 'User para creación de BD no válido.';
    }

    if ($FORM{'SUPERPWD_BD'} !~ /^[\w\-\.\@\:\$%!#]{0,64}$/) {
        return 'Password para creación de BD no válido.';
    }

    if ($FORM{'PRONTUS_SMTP'} ne '') {
        if ($FORM{'PRONTUS_SMTP'} !~ /^[A-Z\.a-z\_\-0-9]+$/) {
            return "SMTP no válido.";
        }
    }

    if ($FORM{'NEW_TITLE_SITE_NAME'} eq '') {
        return "Debe ingresar nombre del sitio para ser incluído en etiquetas 'title' de las páginas.";
    }

    if (lc($FORM{'Sbm_ACCION'}) !~ /siguiente/) {
        return "Solicitud de ejecución no válida.";
    }

    return '';
}
