#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
    $pathLibsProntus =~ s/\/[^\/]+$//;
    unshift(@INC,$pathLibsProntus); # Para dejar disponibles las librerias de prontus
}

# Captura STDERR
use lib_stdlog;
&lib_stdlog::set_stdlog($0, 51200, 'wizard_error_log');

use lib_prontus;
use glib_fildir_03;
use prontus_varglb; &prontus_varglb::init();
use wizard_lib;

main:{
    &lib_prontus::setUtf8();
    # Valida informacion de paso anter.
    my ($msg_err,
                $prontus_id,
                $extension,
                $smtp,
                $model_nom,
                $new_title_site_name,
                $server_bd,
                $nom_bd,
                $user_bd,
                $pwd_bd,
                $superuser_bd,
                $superpwd_bd) = &check_paso_anterior();

    if ($msg_err) {
        $prontus_varglb::DIR_CORE = '/wizard_prontus/core'; # solo para efectos de la plantilla de mensaje
        &lib_prontus::print_pag_result('Error', $msg_err, 0, 1, 1);
    }

    my $buffer = &glib_fildir_03::read_file("$prontus_varglb::DIR_SERVER/wizard_prontus/core/confirm.html");

    $buffer =~ s/%%prontus_id%%/$prontus_id/ig;
    $smtp = 'No especificado' if (!$smtp);
    $buffer =~ s/%%server_smtp%%/$smtp/ig;
    $buffer =~ s/%%prontus_extension%%/$extension/ig;
    $buffer =~ s/%%model_nom%%/$model_nom/ig;
    $buffer =~ s/%%new_title_site_name%%/$new_title_site_name/ig;

    $buffer =~ s/%%server_bd%%/$server_bd/ig;
    $buffer =~ s/%%nom_bd%%/$nom_bd/ig;
    $buffer =~ s/%%user_bd%%/$user_bd/ig;
    $buffer =~ s/%%pwd_bd%%/$pwd_bd/ig;
    $superuser_bd = 'No especificado' if (!$superuser_bd);
    $buffer =~ s/%%superuser_bd%%/$superuser_bd/ig;
    $superpwd_bd = 'No especificado' if (!$superpwd_bd);
    $buffer =~ s/%%superpwd_bd%%/$superpwd_bd/ig;

    print "Content-Type: text/html\n\n";
    print $buffer;
};

sub check_paso_anterior {
    # Chequea que se haya pasado por el paso 1
    my $INF_FILE = "$prontus_varglb::DIR_SERVER/$wizard_lib::INF_DIR/$wizard_lib::INF_FILE";
    if (! -f $INF_FILE) {
        return "[errInfFile] Solicitud de ejecución no válida.";
    }

    # Leer y cargar y validar contenido del INF.
    my $buffer = &glib_fildir_03::read_file($INF_FILE);
    my ($prontus_id, $extension, $smtp);
    my ($server_bd, $nom_bd, $user_bd,$pwd_bd,$superuser_bd,$superpwd_bd);
    my $new_title_site_name;

    if ($buffer =~ /(\[PRONTUS\].*\[\/PRONTUS\])/s) {
        my $buffer_prontus = $1;
        # Validar id
        $prontus_id = &wizard_lib::get_prontus_id($buffer_prontus);

        if ($prontus_id eq '') {
            return 'Información de paso 1 está corrupta.';
        }

        # smtp
        if ($buffer_prontus =~ /SERVER_SMTP=([\w\.\-]+)\n/) {
            $smtp = $1;
        }

        if ($buffer_prontus !~ /NEW_TITLE_SITE_NAME=(.+?)\n/) {
            return 'Información de paso 1 está corrupta.';
        } else {
            $new_title_site_name = $1;
        }

        $server_bd = $1 if ($buffer_prontus =~ /server_bd=(.*?)\n/i);
        $nom_bd = $1 if ($buffer_prontus =~ /nom_bd=(.*?)\n/i);
        $user_bd = $1 if ($buffer_prontus =~ /user_bd=(.*?)\n/i);
        $pwd_bd = $1 if ($buffer_prontus =~ /pwd_bd=(.*?)\n/i);
        $superuser_bd = $1 if ($buffer_prontus =~ /superuser_bd=(.*?)\n/i);
        $superpwd_bd = $1 if ($buffer_prontus =~ /superpwd_bd=(.*?)\n/i);
    } else {
        return 'Información de paso 1 está corrupta';
    }

    # Validar que no exista el dir destino del prontus.
    # Esto ya se chequea en el paso 1 pero se hace nuevamente por seguridad.
    my $dir_prontus = "$prontus_varglb::DIR_SERVER/$prontus_id";

    if (-d $dir_prontus) {
        return "El directorio prontus ya existe. Para continuar con el proceso de instalación Ud. debe cambiar el nombre especificado para el publicador, o bien, <br>eliminar manualmente el directorio existente que genera el conflicto.";
    } else {
        # Lo creo y luego lo borro para verificar que este ok.
        if (&glib_fildir_03::check_dir($dir_prontus)) {
            &glib_fildir_03::borra_dir($dir_prontus);
        } else {
            return "No se puede crear el directorio destino del publicador. No es posible continuar con la instalación.";
        }
    }

    # valida q se haya pasado por paso2
    my $model_nom;

    if ($buffer =~ /(\[MODEL\].*\[\/MODEL\])/s) {
        my $buffer_model = $1;

        if ($buffer_model !~ /MODEL_EXT=(\w+)\n/) {
            return 'Información de paso 1 está corrupta.[0]';
        } else {
            $extension = $1;
        }

        # Validar id
        if ($buffer_model !~ /MODEL_NOM=(\w+)\n/) {
            return 'Información de paso 2 está corrupta.[1]';
        } else {
            $model_nom = $1;
        }
    } else {
        return 'Información de paso 2 está corrupta.[2]';
    }

    return ('', $prontus_id, $extension, $smtp, $model_nom, $new_title_site_name, $server_bd, $nom_bd, $user_bd,$pwd_bd,$superuser_bd,$superpwd_bd);
}
