#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
    $pathLibsProntus =~ s/\/[^\/]+$//;
    unshift(@INC,$pathLibsProntus); # Para dejar disponibles las librerias de prontus
}

use lib_prontus;

# Captura STDERR
use lib_stdlog;
&lib_stdlog::set_stdlog($0, 51200, 'wizard_error_log');

use glib_fildir_03;
use prontus_varglb; &prontus_varglb::init();
use wizard_lib;

main:{
    &lib_prontus::setUtf8();
    # Valida informacion de paso 1.
    my $msg_err = &wizard_lib::check_paso1();

    if ($msg_err) {
        $prontus_varglb::DIR_CORE = '/wizard_prontus/core'; # solo para efectos de la plantilla de mensaje
        &lib_prontus::print_pag_result('Error', $msg_err, 0);
    }

    my $buffer = &glib_fildir_03::read_file("$prontus_varglb::DIR_SERVER/wizard_prontus/core/paso2.html");
    $msg_err = '';
    ($msg_err, $buffer) = &carga_modelos($buffer);

    if ($msg_err) {
        $prontus_varglb::DIR_CORE = '/wizard_prontus/core'; # solo para efectos de la plantilla de mensaje
        &lib_prontus::print_pag_result('Error', $msg_err, 0);
    }

    print "Content-Type: text/html\n\n";
    print $buffer;
};

sub carga_modelos {
    # Valida y carga modelos prontus disponibles.
    my $buffer = $_[0];
    my $models_dir = "$prontus_varglb::DIR_SERVER/wizard_prontus/models";

    if (!-d $models_dir) {
        my $msg = "<div class=\"mensaje\">No hay Modelos Prontus disponibles. No es posible continuar.<br>";
        $msg = $msg . "Selecciona la opci&oacute;n \"Descargar Modelos\" antes de continuar.</div>";
        $buffer =~ s/<!--LOOP_MODELS-->(.*?)<!--\/LOOP_MODELS-->/$msg<div class="separador"><\/div>/sg;
        return ('', $buffer);
    }

    my @lisdir = &glib_fildir_03::lee_dir($models_dir);
    @lisdir = grep !/^\./, @lisdir; # Elimina directorios . y ..

    my ($loop_tpl, $loop_out, $loop_item);

    if ($buffer =~ /<!--LOOP_MODELS-->(.*?)<!--\/LOOP_MODELS-->/s) {
        $loop_tpl = $1;
    } else {
        return 'Error en plantilla wizard. No es posible continuar.';
    }

    my $nro_models = 0;
    foreach my $k (@lisdir) {
        # localiza cfg descriptor del modelo,
        # si lo encuentra asume q se trata de un modelo.
        if (-f "$models_dir/$k/$k.cfg") {
            $loop_item = $loop_tpl;

            # valida icono
            my $imagen = "/wizard_prontus/models/$k/$k-thumb.png";
            if (-f "$prontus_varglb::DIR_SERVER$imagen") {
                $loop_item =~ s/%%MODEL_IMG%%/$imagen/ig;
            } else {
                $imagen = "/wizard_prontus/models/$k/$k.gif";
                if (-f "$prontus_varglb::DIR_SERVER$imagen") {
                    $loop_item =~ s/%%MODEL_IMG%%/$imagen/ig;
                } else {
                    return "En Modelo [$k] falta icono";
                }
            }

            my $buffer = &glib_fildir_03::read_file("$models_dir/$k/$k.cfg");
            my $refcfg = &wizard_lib::carga_model_cfg($buffer);
            my %cfg = %$refcfg;

            $nro_models++;
            $loop_item =~ s/%%MODEL_ID%%/$k/ig;

            if ($cfg{'MODELO_NOMBRE_REAL'}) {
                $loop_item =~ s/%%MODEL_NOM%%/$cfg{'MODELO_NOMBRE_REAL'}/ig;
            } else {
                $loop_item =~ s/%%MODEL_NOM%%/$k/ig;
            }

            $loop_item =~ s/%%MODEL_DESC%%/$cfg{'MODELO_DESCRIPTION'}/ig;

            if ($cfg{'MODELO_VERSION'}) {
                $loop_item =~ s/%%MODEL_VERSION%%/$cfg{'MODELO_VERSION'}/ig;
            } else {
                $loop_item =~ s/%%MODEL_VERSION%%/Desconocida/ig;
            }

            my $plt_port_dir = "$prontus_varglb::DIR_SERVER/wizard_prontus/models/$k/plantillas/edic/nroedic/port/";
            my @listportdir = &glib_fildir_03::lee_dir($plt_port_dir);
            @listportdir = grep !/^\./, @listportdir; # Elimina directorios . y ..
            my $num_port = scalar @listportdir;

            $loop_item =~ s/%%MODEL_NUM_PORTS%%/$num_port/ig;

            my $plt_art_dir = "$prontus_varglb::DIR_SERVER/wizard_prontus/models/$k/plantillas/artic/fecha/pags/";
            my @listartdir = &glib_fildir_03::lee_dir($plt_art_dir);
            @listartdir = grep !/^\./, @listartdir; # Elimina directorios . y ..
            my $num_art = scalar @listartdir;

            $loop_item =~ s/%%MODEL_NUM_ARTS%%/$num_art/ig;

            $loop_out .= $loop_item;
        }
    }

    # Debe haber por lo menos 1 modelo.
    if (!$nro_models) {
        my $msg = "<div class=\"mensaje\">No hay Modelos Prontus disponibles. No es posible continuar.<br>";
        $msg = $msg . "Selecciona la opci&oacute;n \"Descargar Modelos\" antes de continuar.</div>";
        $buffer =~ s/<!--LOOP_MODELS-->(.*?)<!--\/LOOP_MODELS-->/$msg<div class="separador"><\/div>/sg;
        return ('', $buffer);
    } else {
        $buffer =~ s/<!--LOOP_MODELS-->.*?<!--\/LOOP_MODELS-->/$loop_out/s;
        return ('', $buffer);
    }
}
