#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

# -----------------------COMENTARIO GLOBAL-----------------------
# ---------------------------------------------------------------
# PROPOSITO
# ---------
# Transcodificar un audio de avi, wmv, mp4, mpeg, rm y otros a mp3.
# ---------------------------------------------------------------
# LLAMADAS A ARCHIVOS EXTERNOS.
# ------------------------------
# LLama a prontus_audio_do_xcode.pl para que realizar el proceso
# prontus_audio_do_xcode.pl al terminar elimina el audio original y sustituye el nombre en el xml del articulo Prontus.
#
# ---------------------------------------------------------------
# INVOCACIONES ACEPTADAS.
# ------------------------
# Como cgi, usando metodo GET o POST. Los parametros son:
# audio  - Path relativo del audio a transcodificar.
# prontus_id
#
# Retorna 'OK' o 'Error: <mensaje de error>';
#
# Ejemplo:
#
# /cgi-cpn/xcoding/xcoding/prontus_audio_xcode.cgi?audio=/prontus_proto/site/artic/20100531/mmedia/multimedia_audio120100531182139.wav&prontus_id=prontus_proto
#
# Como script en línea de comandos:
# perl /cgi-cpn/xcoding/xcoding/prontus_audio_xcode.cgi /prontus_proto/site/artic/20100531/mmedia/multimedia_audio120100531182139.wav prontus_proto
#
# ---------------------------------------------------------------
# PLANTILLAS HTML UTILIZADAS.
# ------------------------
#  No usa plantillas.
#
# -------------------------------BEGIN SCRIPT--------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
    $pathLibsProntus =~ s/\/[^\/]+\/?$//;
    unshift(@INC,$pathLibsProntus); # Para dejar disponibles las librerias de prontus
}

# ---------------------------------------------------------------
# DECLARACIONES GLOBALES.
# ---------------------------------------------------------------
use lib_stdlog;
&lib_stdlog::set_stdlog($0, 51200);

use prontus_varglb; &prontus_varglb::init();
use glib_html_03;
use glib_cgi_05;
use lib_prontus;
use lib_profile;

our $FILE = '';        # Contenido del formulario de invocacion.
our $MODO_CLI = 0;

main: {
    &lib_prontus::setUtf8();
    # Rescatar parametros recibidos
    &glib_cgi_05::new();
    $FILE = &glib_cgi_05::param('audio');
    $prontus_varglb::PRONTUS_ID = &glib_cgi_05::param('prontus_id');

    if ($ENV{'SERVER_NAME'} eq '' ||  scalar @ARGV > 0) {
        $MODO_CLI = 1;
        $FILE = $ARGV[0];
        $prontus_varglb::PRONTUS_ID = $ARGV[1];
    }

    # Valida datos de entrada
    my $msg_err;
    $msg_err = "Parámetro [audio] no es válido [$FILE]" if ((!-f "$prontus_varglb::DIR_SERVER$FILE") || (!-s "$prontus_varglb::DIR_SERVER$FILE"));
    $msg_err = "Parámetro [prontus_id] no es válido" if (! &lib_prontus::valida_prontus($prontus_varglb::PRONTUS_ID));

    &glib_html_03::print_json_result(0, "Error: $msg_err") if ($msg_err);

    &lib_prontus::load_config_by_id($prontus_varglb::PRONTUS_ID);

    if ($FILE =~ /(\/site\/\w+\/\d+\/mmedia\/multimedia_audio\w+\d{14}\.\w+)$/i) {
        $FILE = "/$prontus_varglb::PRONTUS_ID$1";
    } else {
        &glib_html_03::print_json_result(0, "Error: Archivo de audio no valido");
    }

    # chequeo de tamaño de archivo
    if ( (-s "$prontus_varglb::DIR_SERVER$FILE") > ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MAX_XCODING'}*1048576)) {
        # no se procesa si es mas grande de MAX_XCODING MiB
        &glib_html_03::print_json_result(0, "Error: Tamaño de archivo es muy grande para ser transcodificado, límite: [$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MAX_XCODING'} MiB]");
    }

    if (!$MODO_CLI) {
    # User check
        ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_ID'}, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_PERFIL'}) = &lib_prontus::check_user();
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_ID'} eq '') {
            &glib_html_03::print_json_result(0, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_PERFIL'});
        }
        if (!lib_profile::check_profile_access('art_publish')) {
            glib_html_03::print_json_result(0, 'La funcionalidad requerida no está disponible para su usuario.');
        }
    }

    # Startea transcodificacion y devuelve respuesta json
    my ($status, $msg) = &start_xcode();
    &glib_html_03::print_json_result($status, $msg);
}

# -------------------------------------------------------------------#
# Inicia la transcodificacion.
sub start_xcode {
    my $origen = "$prontus_varglb::DIR_SERVER$FILE";

    sleep(rand(5));
    # Verifica que no haya otro transcoding identico en ejecucion.
    my $res = qx/ps auxww |grep 'prontus_audio_do_xcode.pl $origen $prontus_varglb::PRONTUS_ID'|grep -v grep/;

    if ($res ne '') {
        return (1, 'Transcodificación en proceso...');
    }

    my $pathnice = &lib_prontus::get_path_nice();
    my $cmd = "$pathnice /usr/bin/perl $prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/xcoding/prontus_audio_do_xcode.pl $origen $prontus_varglb::PRONTUS_ID";

    # Ejecutar la transcodificacion en background.
    print STDERR "Ejecutar[$cmd]\n";
    system("$cmd >/dev/null 2>&1 &");

    return (1, 'Transcodificación en proceso...');
}
