#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# SCRIPT.
# -----------
# prontus_procesar_videos.pl
# ---------------------------------------------------------------
# UBICACION.
# -----------
# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# Ejecuta la transcodificación para todos los videos de un prontus
# de la misma forma que se hace desde un FID
# ---------------------------------------------------------------
# LLAMADAS A SCRIPTS.
# ------------------------
# Usa las cgis de transcodificacion de prontus
# ---------------------------------------------------------------
# INVOCACIONES ACEPTADAS.
# ------------------------
# A traves de linea de comando.
# Parametros:
# 1: nombre del prontus a procesar
# 2: identificador del fid a procesar
# 3: marca o marcas del video, si es mas de una se separan con coma,
#  ej: multimedia_video1,multimedia_video2
#
# Ejemplo de uso:
# perl prontus_procesar_videos.pl mi_prontus_id fid_noticia multimedia_video1,multimedia_video2
# ---------------------------------------------------------------
# ARCHIVOS DE ENTRADA.
# ------------------------
# ---------------------------------------------------------------
# ARCHIVOS DE SALIDA.
# ------------------------
# ---------------------------------------------------------------
# Tablas.
# ------------------------
# No utiliza.
# ---------------------------------------------------------------
# -------------------------------BEGIN SCRIPT--------------------
# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ---------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    unshift(@INC,$Bin); # Para dejar disponibles las librerias
    my $pathLibsProntus = $Bin;
    $pathLibsProntus =~ s/\/[^\/]+\/?$//;
    unshift(@INC,$pathLibsProntus); # Para dejar disponibles las librerias de prontus
}

use lib_stdlog;
&lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb; &prontus_varglb::init();
use lib_prontus;
use glib_hrfec_03;
use glib_fildir_03;

# ---------------------------------------------------------------
# MAIN.
# ---------------------------------------------------------------
my $COUNTER = 0;

my @BUSCAR;
my @CAMPOS;
my $BUSCAR_CAMPOS = 0;
my $FID = '';
my $PRONTUS_ID = '';
my $TS = '';
my $input;

main:{
    &lib_prontus::setUtf8();
    if (scalar @ARGV < 2) {
        print "Parámetros insuficientes. Modo de uso:\n";
        print "perl prontus_procesar_videos.pl <prontus_id> <fid_id o all o ts articulo>\n";
        print "perl prontus_procesar_videos.pl <prontus_id> <fid_id o all o ts articulo> <multimedia_video1,multimedia_video2,..,multimedia_videoN>\n";
        exit;
    }
    $PRONTUS_ID = $ARGV[0];

    if (! &lib_prontus::valida_prontus($PRONTUS_ID)) {
        print "El parámetro 'prontus_id' no es válido.\n";
        exit;
    }

    if ($ARGV[1] ne '' && $ARGV[1] !~ /^(fid\_|all)/ && $ARGV[1] !~ /^\d{14}$/) {
        print "El segundo parámetro no es válido.\nDebe ser el nombre de un fid o el ts de un artículo.\n";
    }

    if ($ARGV[1] =~ /^\d{14}$/) {
        $TS = $ARGV[1];
    } elsif ($ARGV[1] =~ /^fid\_/) {
        $FID = $ARGV[1];
    }

    if ($ARGV[2] eq '') {
        $BUSCAR_CAMPOS = 1;
    } else {
        @BUSCAR = split(',', $ARGV[2]);
    }

    $input = $Bin;
    $input =~ s/\/[^\/]+\/[^\/]+\/?$//;
    $input .= "/$PRONTUS_ID/site/artic/";

    print STDERR "[".&glib_hrfec_03::fecha_human()." ". &glib_hrfec_03::hora_human()."] Inicio Proceso [$PRONTUS_ID][$TS][$FID]\n";

    if ($TS ne '') {
        $input .= substr($TS, 0, 8). "/xml/";
        &procesa_file("$TS.xml", $input);
    } else {
        &procesa_dir($input);
    }
    print STDERR 'Se procesaron ' . $COUNTER .  " archivos multimedia \n" if $COUNTER;
    print STDERR "[".&glib_hrfec_03::fecha_human()." ". &glib_hrfec_03::hora_human()."] Fin Proceso [$PRONTUS_ID][$TS][$FID]\n";
}

#-----------------------------------------------------------------------#
sub procesa_dir {
    my $input = $_[0];

    unless(-d $input) {
        die "El directorio $input no existe.";
    }
    $input = $input . '/' unless ($input =~ /\/$/);

    my @entries = &glib_fildir_03::lee_dir($input);
    foreach my $k (@entries) {
        next if($k eq '.' || $k eq '..');
        if(-f $input . $k) {
            &procesa_file($k, $input);

        } elsif(-d $input . $k) {
            &procesa_dir($input . $k);
        }
    }
}

#-----------------------------------------------------------------------#
sub procesa_file {
    my($filename, $path) = ($_[0], $_[1]);
    my $filepath = $path . $filename;
    my $titular;
    my $ts;
    if ($filename !~ /^(\d{14})\.xml/isg) {
        return;
    }
    $ts = $1;

    if (-f $filepath) {
        my $file = &glib_fildir_03::read_file($filepath);
        if ($FID ne ''  && $file !~ /$FID/is) {
            return;
        }
        # print STDERR "XML valido $filepath\n";
        if ($BUSCAR_CAMPOS) {
            my %hash_campos;
            while($file =~ /<(multimedia_\w+\d+)>/isg) {
                $hash_campos{$1} = 1;
            }
            @BUSCAR = keys %hash_campos;
        }
        my %data;
        my $cmd = '';
        my $result = '';
        foreach my $campo (@BUSCAR) {
            my $found = '';
            if ($file =~ /<$campo>([^<>]+)<\/$campo>/is) {
                $found = $1;
            } elsif ($file =~ /<$campo>.*?<\!\[CDATA\[(.*?)\]\]>.*?<\/$campo>/is) {
                $found = $1;
            }
            $found =~ s/[\n\r]//g;

            if ($found eq '') {
                next;
            }

            if ($file =~ /<_txt_titular>.*?<!\[CDATA\[(.*?)\]\]>.*?<\/_txt_titular>/is) {
                $titular = $1;
            }

            $cmd = "ps auxww |grep ffmpeg | grep -v grep | grep -v 'sh -c' | grep -v nice| wc -l";
            my $process = `$cmd`;
            $process += 0;
            # si hay menos de 3 corriendo revisamos para lanzar otro ffmpeg
            while($process > 2) {
                sleep 10;
                $process = `$cmd`;
                $process += 0;
            }

            print STDERR "[".&glib_hrfec_03::fecha_human()." ". &glib_hrfec_03::hora_human()."] Procesar [$ts][$titular][$campo][$found]\n";

            if ($campo =~ /video/) {
                my $video =  "/$PRONTUS_ID/site/mm/". substr($ts ,0, 8) . "$prontus_varglb::DIR_MMEDIA/$found";
                # si el archivo no está, no se procesa, se asume que el xml sabe que el archivo no está en el disco
                # y esta disponible en otra ruta o de otra forma
                next if (!-f "$prontus_varglb::DIR_SERVER$video");

                my $path_json = "$input$campo$ts.json";
                $path_json =~ s|artic\/(\d+)\/xml|mm\/$1\/mmedia|g;
                # se borra json de metadata para actualizar
                # si es mas antiguo que el video, entonces se subio uno nuevo o quizas se hizo un recorte
                # si el json es mas nuevo, significa que el video no ha cambiado
                if (-f $path_json && -M $path_json > -M "$prontus_varglb::DIR_SERVER$video") {
                    unlink $path_json;
                }

                if ($found =~ /\.mp4$/) { # si es mp4 revisamos los atoms
                    $cmd = "perl $Bin/prontus_qtfaststart_check.cgi $video $PRONTUS_ID";
                } else { # si no es mp4 hay que transcodificarlo
                    $cmd = "perl $Bin/prontus_videoxcodestatus.cgi $video $PRONTUS_ID";
                }
                $result = `$cmd`;
                #~ print STDERR "$result\n";
                if ($result =~ /(RECODE|XCODE|Xcoding|none|busy)/) {
                    my $versiones = 0;
                    # si es XCODE significa que faltan las versiones del video
                    $versiones = 1 if ($result =~ /XCODE/);
                    # si es none, significa que no es mp4, que no hay proceso corriendo y hay que transcodificar
                    $versiones = 0 if ($result =~ /none/);
                    $cmd = "perl $Bin/prontus_videoxcode.cgi $video $PRONTUS_ID $versiones";
                    #~ print STDERR "$cmd\n";
                    print STDERR "[".&glib_hrfec_03::fecha_human()." ". &glib_hrfec_03::hora_human()."] Transcodificar $video\n";

                    # si no se está procesando se lanza uno nuevo
                    $result = `$cmd` if ($result !~ /Xcoding/ && $result !~ /busy/);
                    #~ print STDERR $result;
                    do {
                        sleep(10);
                        $cmd = "perl $Bin/prontus_videoxcodestatus.cgi $video $PRONTUS_ID";
                        $result = `$cmd`;
                    } while ($result =~ /busy/);
                    print STDERR "[".&glib_hrfec_03::fecha_human()." ". &glib_hrfec_03::hora_human()."] Terminado $video\n";
                    $COUNTER++;
                } elsif ($result =~ /FIX/) {
                    print STDERR "[".&glib_hrfec_03::fecha_human()." ". &glib_hrfec_03::hora_human()."] Corregir $video\n";
                    do {
                        sleep(10);
                        $cmd = "perl $Bin/prontus_qtfaststart_check.cgi $video $PRONTUS_ID";
                        $result = `$cmd`;
                        #~ print STDERR $result;
                    } while ($result =~ /busy/);
                    print STDERR "[".&glib_hrfec_03::fecha_human()." ". &glib_hrfec_03::hora_human()."] Terminado $video\n";
                    $COUNTER++;
                }
            } elsif ($campo =~ /audio/) {
                my $audio =  "/$PRONTUS_ID/site/mm/". substr($ts ,0, 8) . "$prontus_varglb::DIR_MMEDIA/$found";
                # si el archivo no está, no se procesa, se asume que el xml sabe que el archivo no está en el disco
                # y esta disponible en otra ruta o de otra forma
                next if (!-f "$prontus_varglb::DIR_SERVER$audio");

                my $path_json = "$input$campo$ts.json";
                $path_json =~ s|/artic\/(\d+)\/xml|mm\/$1\/mmedia|g;
                # se borra json de metadata para actualizar
                # si es mas antiguo que el audio, entonces se subio uno nuevo o quizas se hizo un recorte
                # si el json es mas nuevo, significa que el audio no ha cambiado
                if (-f $path_json && -M $path_json > -M "$prontus_varglb::DIR_SERVER$audio") {
                    unlink $path_json;
                }

                if ($found =~ /\.mp3$/) { # si es mp3 validamos el audio
                    $cmd = "perl $Bin/prontus_audio_check.cgi $audio $PRONTUS_ID";
                } else { # si no es mp4 hay que transcodificarlo
                    $cmd = "perl $Bin/prontus_audio_xcode_status.cgi $audio $PRONTUS_ID";
                }
                $result = `$cmd`;
                #~ print STDERR  $result;
                if ($result =~ /(RECODE|XCODING|none|busy)/) {
                    # si es none, significa que no es mp3, que no hay proceso corriendo y hay que transcodificar
                    # RECODE significa un bitrate muy alto y hay que recodificar
                    # busy significa que ya se esta procesando
                    $cmd = "perl $Bin/prontus_audio_xcode.cgi $audio $PRONTUS_ID";
                    #~ print STDERR "$cmd\n";
                    print STDERR "[".&glib_hrfec_03::fecha_human()." ". &glib_hrfec_03::hora_human()."] Transcodificar $audio\n";

                    # si no se está procesando se lanza uno nuevo
                    $result = `$cmd` if ($result !~ /XCODING/ && $result !~ /busy/);
                    #~ print STDERR $result;
                    do {
                        sleep(10);
                        $cmd = "perl $Bin/prontus_videoxcodestatus.cgi $audio $PRONTUS_ID";
                        $result = `$cmd`;
                    } while ($result =~ /busy/);
                    print STDERR "[".&glib_hrfec_03::fecha_human()." ". &glib_hrfec_03::hora_human()."] Terminado $audio\n";
                    $COUNTER++;
                }
            }
        }
    }
}
