#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

# -----------------------COMENTARIO GLOBAL-----------------------
# ---------------------------------------------------------------
# PROPOSITO
# ---------
# Transcodificar un video de avi, wmv, mp4, mpeg, rm y otros a mp4.
# ---------------------------------------------------------------
# LLAMADAS A ARCHIVOS EXTERNOS.
# ------------------------------
# LLama a prontus_videodoxcode.pl para que realizar el proceso
# prontus_videodoxcode.p al terminar elimina el video original y sustituye el nombre en el xml del articulo Prontus.
#
# ---------------------------------------------------------------
# INVOCACIONES ACEPTADAS.
# ------------------------
# Como cgi, usando metodo GET o POST. Los parametros son:
# video  - Path relativo al video a transcodificar.
# prontus_id
# generar_versiones = si solamente generar las versiones alternativas.
#
# Retorna 'OK' o 'Error: <mensaje de error>';
#
# Ejemplo:
#
# /cgi-cpn/xcoding/prontus_videoxcode.cgi?video=/prontus_proto/site/artic/20100531/mmedia/multimedia_video120100531182139.wmv&prontus_id=prontus_proto&generar_versiones=1
#
# Como script en línea de comandos:
# perl /cgi-cpn/xcoding/prontus_videoxcode.cgi /prontus_proto/site/artic/20100531/mmedia/multimedia_video120100531182139.wmv prontus_proto 1
#
# ---------------------------------------------------------------
# PLANTILLAS HTML UTILIZADAS.
# ------------------------
#  No usa plantillas.
#
# -------------------------------BEGIN SCRIPT--------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
    $pathLibsProntus =~ s/\/[^\/]+\/?$//;
    unshift(@INC,$pathLibsProntus); # Para dejar disponibles las librerias de prontus
}

# ---------------------------------------------------------------
# DECLARACIONES GLOBALES.
# ---------------------------------------------------------------
use lib_stdlog;
&lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb; &prontus_varglb::init();
use glib_html_03;
use glib_cgi_05;
use lib_prontus;
use lib_profile;

our %FORM = ();        # Contenido del formulario de invocacion.
our $MODO_CLI = 0;

main: {
    &lib_prontus::setUtf8();
    # Rescatar parametros recibidos
    &glib_cgi_05::new();
    $FORM{'video'} = &glib_cgi_05::param('video');
    $FORM{'generar_versiones'} = &glib_cgi_05::param('generar_versiones');
    $prontus_varglb::PRONTUS_ID = &glib_cgi_05::param('prontus_id');

    if ($ENV{'SERVER_NAME'} eq '' ||  scalar @ARGV > 0) {
        $MODO_CLI = 1;
        $FORM{'video'} = $ARGV[0];
        $prontus_varglb::PRONTUS_ID = $ARGV[1];
        $FORM{'generar_versiones'} = $ARGV[2];
    }

    # Valida datos de entrada
    my $msg_err;
    $msg_err = "Parámetro [video] no es válido [$FORM{'video'}]" if ((!-f "$prontus_varglb::DIR_SERVER$FORM{'video'}") || (!-s "$prontus_varglb::DIR_SERVER$FORM{'video'}"));
    $msg_err = "Parámetro [prontus_id] no es válido" if (! &lib_prontus::valida_prontus($prontus_varglb::PRONTUS_ID));

    &glib_html_03::print_json_result(0, "Error: $msg_err") if ($msg_err);

    &lib_prontus::load_config_by_id($prontus_varglb::PRONTUS_ID);


    if ($FORM{'video'} =~ /(\/site\/\w+\/\d+\/mmedia\/multimedia_video\w+\d{14}\.\w+)$/i) {
        $FORM{'video'} = "/$prontus_varglb::PRONTUS_ID$1";
    } else {
        &glib_html_03::print_json_result(0, "Error: Archivo de video no valido");
    }

    # chequeo de tamaño de archivo
    if ( (-s "$prontus_varglb::DIR_SERVER$FORM{'video'}") > ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MAX_XCODING'}*1048576)) {
        # no se procesa si es mas grande de MAX_XCODING MiB
        &glib_html_03::print_json_result(0, "Error: Tamaño de archivo es muy grande para ser transcodificado, límite: [$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MAX_XCODING'} MiB]");
    }

    if (!$MODO_CLI) {
    # User check
        ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_ID'}, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_PERFIL'}) = &lib_prontus::check_user();
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_ID'} eq '') {
            &glib_html_03::print_json_result(0, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'USERS_PERFIL'});
        }
        if (!lib_profile::check_profile_access('art_publish')) {
            glib_html_03::print_json_result(0, 'La funcionalidad requerida no está disponible para su usuario.');
        }
    }

    # Startea transcodificacion y devuelve respuesta json
    my ($status, $msg) = &start_xcode();
    &glib_html_03::print_json_result($status, $msg);
}

# -------------------------------------------------------------------#
# Inicia la transcodificacion.
sub start_xcode {
    my $origen = "$prontus_varglb::DIR_SERVER$FORM{'video'}";

    sleep(rand(5));
    # Verifica que no haya otro transcoding identico en ejecucion.
    my $res = qx/ps auxww |grep 'prontus_videodoxcode.pl $origen $prontus_varglb::PRONTUS_ID'|grep -v grep/;
    # print STDERR "Execution test = [$res][ps auxww |grep 'prontus_videodoxcode.pl $origen $prontus_varglb::PRONTUS_ID'|grep -v grep]\n";

    if ($res ne '') {
        return (1, 'Transcodificación en proceso...');
    }

    my $pathnice = &lib_prontus::get_path_nice();
    my $cmd = "$pathnice /usr/bin/perl $prontus_varglb::DIR_SERVER/$prontus_varglb::DIR_CGI_CPAN/xcoding/prontus_videodoxcode.pl $origen $prontus_varglb::PRONTUS_ID";

    if ($FORM{'generar_versiones'} eq '1') {
        print STDERR "Ejecutar[$cmd 1] generar versiones.\n";
        system("$cmd 1 >/dev/null 2>&1 &");
        return (1, 'Transcodificación en proceso...');
    }

    # Ejecutar la transcodificacion en background.
    print STDERR "Ejecutar[$cmd 0]\n";
    system("$cmd 0 >/dev/null 2>&1 &");

    return (1, 'Transcodificación en proceso...');
}
